Type: Package
Package: thisplot
Title: Utility Functions for Plotting
Version: 0.4.3
Date: 2026-08-01
Authors@R: 
    c(
    person("Meng", "Xu", email = "mengxu98@qq.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-8300-1054")),
    person(given = "Haoliang", family = "Zhu", email = "hurry060215@gmail.com", role = c("aut"), comment = c(ORCID = "0009-0004-9766-5179"))
    )
Maintainer: Meng Xu <mengxu98@qq.com>
Description: Provides utility functions for plotting. Includes functions for color manipulation, plot customization, panel size control, data optimization for plots, and layout adjustments.
License: MIT + file LICENSE
URL: https://mengxu98.github.io/thisplot/
BugReports: https://github.com/mengxu98/thisplot/issues
Depends: R (>= 4.1.0)
Imports: cli, ComplexHeatmap, dendextend, ggforce, geomtextpath,
        ggrepel, ggnewscale, ggplot2, grDevices, grid, gtable, igraph,
        methods, patchwork, proxyC, reshape2, stats, thisutils (>=
        0.4.3), utils
Suggests: circlize, dplyr, forcats, ggalluvial, ggupset, ggVennDiagram,
        htmltools, Matrix, png, purrr, ragg, scales, stringr, testthat
        (>= 3.0.0), tidyr, venny
Config/Needs/website: mengxu98/thistemplate
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
Language: en-US
LazyData: true
NeedsCompilation: no
Packaged: 2026-08-01 09:55:35 UTC; mx
Author: Meng Xu [aut, cre] (ORCID: <https://orcid.org/0000-0002-8300-1054>),
  Haoliang Zhu [aut] (ORCID: <https://orcid.org/0009-0004-9766-5179>)
Repository: CRAN
Date/Publication: 2026-08-01 10:10:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(drop_data,default)
S3method(drop_data,ggplot)
S3method(drop_data,patchwork)
S3method(head,colors)
S3method(print,ChineseColors)
S3method(print,colors)
S3method(print,thisplot_logo)
S3method(slim_data,default)
S3method(slim_data,ggplot)
S3method(slim_data,patchwork)
export(Blend2Color)
export(BlendRGBList)
export(ChineseColors)
export(DimDataPlot)
export(GraphPlot)
export(RGBA2RGB)
export(StatPlot)
export(add_grob)
export(adjcolors)
export(adjustlayout)
export(annotate_quadrants)
export(annotation_block_fill_graphics)
export(annotation_block_graphics)
export(annotation_graphics)
export(as_grob)
export(as_gtable)
export(blendcolors)
export(build_heatmap_annotation)
export(build_patchwork)
export(clip_symmetric_range)
export(cluster_within_group2)
export(col2hex)
export(compute_velocity_on_grid)
export(drop_data)
export(extractgrobs)
export(geom_alluvial)
export(geom_alluvial_label)
export(geom_alluvial_text)
export(geom_sankey)
export(geom_sankey_bump)
export(geom_sankey_label)
export(geom_sankey_text)
export(get_chinese_palettes)
export(get_colors)
export(get_legend)
export(get_vars)
export(grid_draw)
export(heatmap_fixsize)
export(heatmap_rendersize)
export(jitter_highlighted_points)
export(make_long)
export(mestimate)
export(normalize_drawable)
export(palette_colors)
export(panel_fix)
export(panel_fix_overall)
export(patchwork_grob)
export(segements_df)
export(show_palettes)
export(simple_colors)
export(slim_data)
export(standardise)
export(theme_alluvial)
export(theme_blank)
export(theme_sankey)
export(theme_sankey_bump)
export(theme_this)
export(thisplot_logo)
export(to_percent)
export(visual_colors)
import(ggplot2)
import(thisutils)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,rgb)
importFrom(grDevices,rgb2hsv)
importFrom(utils,head)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# **thisplot** <img src="man/figures/logo.svg" align="right" width="120"/>

<!-- badges: start -->

[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/thisplot)](https://CRAN.R-project.org/package=thisplot)
[![Conda Version](https://img.shields.io/conda/vn/conda-forge/r-thisplot?color=green)](https://anaconda.org/conda-forge/r-thisplot)
[![version](https://img.shields.io/github/r-package/v/mengxu98/thisplot?label=version&color=blue)](https://github.com/mengxu98/thisplot/blob/main/DESCRIPTION)
[![R-CMD-check](https://github.com/mengxu98/thisplot/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/mengxu98/thisplot/actions/workflows/R-CMD-check.yaml)
[![test-coverage](https://github.com/mengxu98/thisplot/actions/workflows/test-coverage.yaml/badge.svg)](https://github.com/mengxu98/thisplot/actions/workflows/test-coverage.yaml)
[![pkgdown](https://github.com/mengxu98/thisplot/actions/workflows/pkgdown.yaml/badge.svg)](https://mengxu98.github.io/thisplot/reference/index.html)
[![RStudio CRAN mirror
downloads](https://cranlogs.r-pkg.org/badges/grand-total/thisplot)](https://CRAN.R-project.org/package=thisplot)

<!-- badges: end -->

## **Introduction**

[thisplot](https://mengxu98.github.io/thisplot/) is an *R* package
providing utility functions for data visualization and plotting. It
includes tools for color manipulation (blending, conversion, palettes),
plot customization (themes, grob operations, patchwork building), panel
size control, data optimization for plots, and layout adjustments.
Designed to enhance workflows with
[ggplot2](https://github.com/tidyverse/ggplot2),
[patchwork](https://github.com/thomasp85/patchwork), and
[ComplexHeatmap](https://github.com/jokergoo/ComplexHeatmap).

## **Chinese traditional colors**

[thisplot](https://mengxu98.github.io/thisplot/) provides a
comprehensive *Chinese traditional color* system with **1058**
representative colors. Use `ChineseColors()` to create a color object,
access colors by name (pinyin or Chinese), visualize the collection with
`visual_colors()`, and retrieve specific palettes using `get_colors()`.

Example usage:

``` r
library(thisplot)
cc <- ChineseColors()
cc
#> 
#> ── Chinese Traditional Colors System
#> 
#> ── Total 1058 colors
#> • blue: 92 colors
#> • cyan: 51 colors
#> • gray_brown: 323 colors
#> • green: 123 colors
#> • orange: 92 colors
#> • purple: 95 colors
#> • red: 181 colors
#> • yellow: 101 colors
#> 
#> ── Methods:
#> • visual_colors(loc_range, num_per_row, title, name_type)
#> 
#> ── See also:
#> • [get_colors()] for searching colors
```

``` r
cc$visual_colors(
  loc_range = c(1, 180),
  num_per_row = 30,
  title = "Chinese traditional colors",
  name_type = "chinese"
)
```

<div style="background:transparent;padding:12px;border-radius:12px;border:1px solid rgba(0,0,0,0.1);box-shadow:none;display:inline-block;overflow-x:auto;">
<div style="font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;font-size:13px;font-weight:600;color:inherit;margin-bottom:3px;">Chinese traditional colors</div>
<table style="border-collapse:separate;border-spacing:3px;">
<tbody>
<tr>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#D2F0F4;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">水<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#C6E6E8;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">海<br>天<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#B0D5DF;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">湖<br>水<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#A2D2E2;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">云<br>门</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#85C0DB;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">缥<br>色</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#8ABCD1;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">秋<br>波<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#8FB2C9;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">晴<br>山<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#93B5CF;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">星<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#8AABCC;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">品<br>月</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#8BA3C7;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">东<br>方<br>既<br>白</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#88ABDA;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">窃<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#6E9BC5;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">挼<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#619AC3;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">羽<br>扇<br>豆<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#5698C3;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">睛<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#4994C4;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">孔<br>雀<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#2486B9;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">宝<br>石<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#1781B5;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">釉<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#2983BB;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">潮<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#1177B0;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">牵<br>牛<br>花<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#1677B3;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">天<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#2B73AF;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">品<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#3271AE;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">青<br>冥</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#2474B5;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">尼<br>罗<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#2376B7;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">花<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#2177B8;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">虹<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#1772B4;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">群<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#126BAE;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">柏<br>林<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#1661AB;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">靛<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#0F59A4;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">飞<br>燕<br>草<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#15559A;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">海<br>涛<br>蓝</div>
</div>
</td>
</tr>
<tr>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#2A5390;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">青<br>花<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#2E4E7E;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">藏<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#22406A;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">琉<br>璃<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#06436F;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">蓝<br>采<br>和</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#144A74;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">鷃<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#12507B;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">碧<br>城</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#354E6B;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">青<br>雀<br>头<br>黛</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#45465E;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">青<br>黛</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#454659;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">䒌<br>靘</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#535164;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">曾<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#6B798E;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">菘<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#5E7987;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">淡<br>蓝<br>灰</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#4182A4;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">吐<br>绶<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#4E7CA1;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">蝶<br>翅<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#346C9C;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">海<br>军<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#3170A7;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">安<br>安<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#11659A;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">搪<br>磁<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#106898;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">柔<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#158BB8;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">鸢<br>尾<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#2F90B9;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">云<br>山<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#1A94BC;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">钴<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#22A2C3;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">海<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#5CB3CC;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">碧<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#63BBD0;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">霁<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#3CBEE7;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">天<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#66A9C9;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">涧<br>石<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#A3BBDB;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">晴<br>山</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#AED0EE;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">碧<br>落</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#BCD4E7;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">星<br>郎</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#BACCD9;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">云<br>水<br>蓝</div>
</div>
</td>
</tr>
<tr>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#C3D7DF;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">井<br>天<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#D0DFE6;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">远<br>天<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#C7D2D4;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">鸥<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#93D5DC;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">清<br>水<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#3EEDE7;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">碧<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#30DFF3;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">湖<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#126E82;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">玉<br>鈫<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#134857;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">苍<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#21373D;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">灰<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#132C33;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">深<br>灰<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#1C2938;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">鸽<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#142334;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">钢<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#101F30;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">暗<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#131824;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">燕<br>颔<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#0F1423;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">钢<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#1A2847;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">花<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#12264F;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">骐<br>驎</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#19325F;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">佛<br>头<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#003460;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">帝<br>释<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#003D74;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">绀<br>宇</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#1C3F73;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">阴<br>丹<br>士<br>林</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#213A70;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">靛<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#003371;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">绀<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#004EA2;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">景<br>泰<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#26499D;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">吴<br>须</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#424D99;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">湛<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#4F64AE;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">石<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#5976BA;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">苍<br>苍</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#6F94CD;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">监<br>德</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#8D93C8;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">霁<br>色</div>
</div>
</td>
</tr>
<tr>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#4B5CC4;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">宝<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#38308E;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">蓝<br>色</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#25F8CB;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">湖<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#48C0A3;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">青<br>碧</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#57C3C2;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">石<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#2BBABE;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">松<br>石</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#51C4D3;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">瀑<br>布<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#29B7CB;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">蔚<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#10AEC2;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">甸<br>子<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#0EB0C9;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">孔<br>雀<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#1E9EB3;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">翠<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#0F95B0;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">胆<br>矾<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#1491A8;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">樫<br>鸟<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#108B96;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">法<br>翠</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#3B818C;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">蜻<br>蜓<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#32788A;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">鱼<br>师<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#006D87;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">软<br>翠</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#547689;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">太<br>师<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#7397AB;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">苍<br>黑</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#7F9FAF;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">竹<br>月</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#98B6C2;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">白<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#87C0CA;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">西<br>子</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#7CABB1;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">闪<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#6CA8AF;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">正<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#5AA4AE;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">天<br>水<br>碧</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#509296;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">扁<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#648E93;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">晚<br>波<br>蓝</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#668F8B;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">空<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#5DA39D;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">二<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#3D8E86;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">铜<br>青</div>
</div>
</td>
</tr>
<tr>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#007175;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">青<br>雘</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#226B68;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">䌦<br>色</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#426666;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">黛<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#3B5554;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">黛<br>色</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#41555D;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">黯</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#424C50;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">鸦<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#284852;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">青<br>緺</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#13393E;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">螺<br>子<br>黛</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#6B7D73;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#FFFFFF;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">千<br>山<br>翠</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#80A492;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">缥<br>碧</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#99BCAC;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">苍<br>筤</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#88ADA6;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">水<br>色</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#88BFB8;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">總<br>犗</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#A4C9CC;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">井<br>天</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#BACAC6;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">老<br>银</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#BBCDC5;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">蟹<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#B1D5C8;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">沧<br>浪</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#C0EBD7;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">青<br>白</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#D5EBE1;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">天<br>缥</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#E0F0E9;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">素</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#EEF7F2;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">月<br>白</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#E0EEE8;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">鸭<br>卵<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#869D9D;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">虾<br>壳<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#A3D900;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">葱<br>黄</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#AFDD22;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">柳<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#BDDD22;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">嫩<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#C9DD22;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">柳<br>黄</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#B7D332;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">翠<br>缥</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#C3D94E;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">青<br>粲</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#BEC936;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">橄<br>榄<br>黄<br>绿</div>
</div>
</td>
</tr>
<tr>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#9EBC19;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">人<br>籁</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#98C446;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">豆<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#96CE54;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">豆<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#96C24E;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">芽<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#8CC269;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">水<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#A0C777;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">葱<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#B2CF87;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">橄<br>榄<br>石<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#C0D695;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">欧<br>碧</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#C0D09D;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">麹<br>尘</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#D0DEAA;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">姚<br>黄</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#E2E7BF;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">海<br>沬<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#EDF1BB;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">葱<br>青</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#E3EB98;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">少<br>艾</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#D8DE8A;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">绮<br>钱</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#D2D97A;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">槐<br>花<br>黄<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#CDD171;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">翠<br>樽</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#BACF65;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">苹<br>果<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#B7D07A;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">芦<br>苇<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#A9BE7B;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">春<br>辰</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#A8BF8F;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">苍<br>葭</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#A8B78C;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">兰<br>苕</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#90A07D;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">碧<br>滋</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#81A380;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">青<br>楸</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#83A78D;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">松<br>霜<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#819D8E;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">翠<br>涛</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#92B3A5;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">浪<br>花<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#9ABEAF;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">草<br>原<br>远<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#A4CAB6;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">玉<br>簪<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#B9DEC9;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">竹<br>篁<br>绿</div>
</div>
</td>
<td style="padding:1px;width:21px;min-width:21px;max-width:21px;height:50px;min-height:50px;border-radius:3px;background:#C6DFC8;box-shadow:0 1px 2px rgba(0,0,0,0.2);text-align:center;vertical-align:middle;position:relative;overflow:hidden;box-sizing:border-box;">
<div style="display:flex;align-items:center;justify-content:center;width:100%;height:100%;box-sizing:border-box;padding:2px 0;">
<div style="color:#000000;font-size:8px;font-family:&#39;Noto Sans SC&#39;,&#39;PingFang SC&#39;,&#39;Microsoft YaHei&#39;,sans-serif;letter-spacing:0.2px;line-height:1.08;font-weight:500;display:block;text-align:center;width:100%;white-space:normal;">淡<br>翠<br>绿</div>
</div>
</td>
</tr>
</tbody>
</table>
</div>

``` r
get_colors("Paired")
#> 
#> ── Found palette: "Paired"
#>       name             rgb     hex
#> 1  #A6CEE3 (166, 206, 227) #A6CEE3
#> 2  #1F78B4  (31, 120, 180) #1F78B4
#> 3  #B2DF8A (178, 223, 138) #B2DF8A
#> 4  #33A02C   (51, 160, 44) #33A02C
#> 5  #FDBF6F (253, 191, 111) #FDBF6F
#> 6  #FF7F00   (255, 127, 0) #FF7F00
#> 7  #FB9A99 (251, 154, 153) #FB9A99
#> 8  #E31A1C   (227, 26, 28) #E31A1C
#> 9  #CAB2D6 (202, 178, 214) #CAB2D6
#> 10 #6A3D9A  (106, 61, 154) #6A3D9A
#> 11 #FFFF99 (255, 255, 153) #FFFF99
#> 12 #B15928   (177, 89, 40) #B15928
```

``` r
get_colors("cyan", palettes = "ChineseSet64")
#> 
#> ── Searching in palette: "ChineseSet64"
#> 
#> ── Found in:
#> #29B7CB: "ChineseSet64"
#> #10AEC2: "ChineseSet64"
#> #0EB0C9: "ChineseSet64"
#> #1E9EB3: "ChineseSet64"
#> #0F95B0: "ChineseSet64"
#> #1491A8: "ChineseSet64"
#> #108B96: "ChineseSet64"
#>     num        name name_ch            rgb     hex category category_ch
#> 111  98      weilan    蔚蓝 (41, 183, 203) #29B7CB     cyan          青
#> 113  99   dianzilan  甸子蓝 (16, 174, 194) #10AEC2     cyan          青
#> 112 100  kongquelan  孔雀蓝 (14, 176, 201) #0EB0C9     cyan          青
#> 114 101      cuilan    翠蓝 (30, 158, 179) #1E9EB3     cyan          青
#> 115 102   danfanlan  胆矾蓝 (15, 149, 176) #0F95B0     cyan          青
#> 116 103 jianniaolan  樫鸟蓝 (20, 145, 168) #1491A8     cyan          青
#> 103 104       facui    法翠 (16, 139, 150) #108B96     cyan          青
```

## **Installation**

Install CRAN version:

``` r
install.packages("thisplot")
# or
if (!require("pak", quietly = TRUE)) {
  install.packages("pak")
}
pak::pak("thisplot")
```

Install development version from
[GitHub](https://github.com/mengxu98/thisplot) use
[pak](https://github.com/r-lib/pak):

``` r
if (!require("pak", quietly = TRUE)) {
  install.packages("pak")
}
pak::pak("mengxu98/thisplot")
```

## **Usage**

Please reference
[here](https://mengxu98.github.io/thisplot/reference/index.html).

#' @title Statistic Plot
#'
#' @description
#' Visualizes data using various plot types such as bar plots,
#' rose plots, ring plots, pie charts, trend plots, area plots,
#' dot plots, sankey plots, chord plots, venn diagrams, and upset plots.
#'
#' @md
#' @param meta.data The data frame containing the data to be plotted.
#' @param stat.by The column name(s) in `meta.data` specifying the variable(s) to be plotted.
#' @param bg.by The column name in `meta.data` specifying the background variable for bar plots.
#' @param flip Whether to flip the plot.
#' Default is `FALSE`.
#' @param NA_color The color to use for missing values.
#' @param NA_stat Whether to include missing values in the plot.
#' Default is `TRUE`.
#' @param keep_empty Whether to keep empty groups in the plot.
#' Default is `FALSE`.
#' @param individual Whether to plot individual groups separately.
#' Default is `FALSE`.
#' @param stat_level The level(s) of the variable(s) specified in `stat.by` to include in the plot.
#' Default is `NULL`.
#' @param plot_type The type of plot to create.
#' Can be one of `"bar"`, `"rose"`, `"ring"`, `"pie"`, `"trend"`,
#' `"trend_alluvial"`, `"area"`, `"dot"`, `"sankey"`, `"chord"`,
#' `"venn"`, or `"upset"`.
#' @param venn_engine The engine used when `plot_type = "venn"`.
#' Can be one of `"ggVennDiagram"` or `"venny"`.
#' Default is `"ggVennDiagram"`.
#' The `"venny"` engine supports 2 to 4 sets.
#' @param venn_args A list of additional arguments passed to
#' `venny::venny` when `plot_type = "venn"` and `venn_engine = "venny"`.
#' The `data` argument is generated internally and cannot be supplied here.
#' @param stat_type The type of statistic to compute for the plot.
#' Can be one of `"percent"`, `"count"`, or `"value"`. Continuous value mode
#' supports `plot_type = "bar"` and `plot_type = "dot"`.
#' @param value.by Numeric column used when `stat_type = "value"`.
#' @param top_n Maximum number of records retained per group and split in value
#' mode. Default is `Inf`.
#' @param rank.by Rank continuous values by absolute magnitude or signed value.
#' @param complete_groups For value dot plots, retain every available group for
#' a label selected in any group within the same split.
#' @param value_cutoff Optional minimum absolute value.
#' @param value_limits Optional continuous colour limits. Symmetric limits are
#' inferred by default.
#' @param value_midpoint Midpoint of the continuous colour scale.
#' @param value_legend_title Legend title for continuous values.
#' @param bar_fill Optional fixed fill used by value bars.
#' @param bar_width Width of bars in value mode.
#' @param point_size Size range for value dot plots.
#' @param character_width Maximum wrapped label width in value mode.
#' @param return_data Return the plot and prepared data in value mode.
#' @param position The position adjustment for the plot.
#' Can be one of `"stack"` or `"dodge"`.
#' @param alpha The transparency level for the plot.
#' @param bg_palette The name of the background color palette to use for bar plots.
#' @param bg_palcolor The color to use in the background color palette.
#' @param bg_alpha The transparency level for the background color in bar plots.
#' @param label Whether to add labels on the plot.
#' Default is `FALSE`.
#' @param label_cutoff Optional minimum plotted value required to show a label.
#' Labels are shown only for values strictly greater than this cutoff. For
#' percentage plots, use proportions; for example, `label_cutoff = 0.1` shows
#' labels only for values greater than 10%. Default is `NULL`, which labels all
#' values.
#' @param label.size The size of the labels.
#' @param label.fg The foreground color of the labels.
#' @param label.bg The background color of the labels.
#' @param label.bg.r The radius of the rounded corners of the label background.
#' @param aspect.ratio Aspect ratio of the panel. Default is `NULL`.
#' @param group.by The column name(s) in `meta.data` specifying the grouping variable(s).
#' Default is `NULL`.
#' @param split.by The column name in `meta.data` specifying the variable to split plots by.
#' Default is `NULL`.
#' @param palette The name of the color palette to use.
#' Default is `"Chinese"`.
#' @param palcolor Custom colors to use instead of palette.
#' Default is `NULL`.
#' @param title The title of the plot.
#' Default is `NULL`.
#' @param subtitle The subtitle of the plot.
#' Default is `NULL`.
#' @param xlab The label for the x-axis.
#' Default is `NULL`.
#' @param ylab The label for the y-axis.
#' Default is `NULL`.
#' @param legend.position The position of the legend.
#' Can be one of `"none"`, `"left"`, `"right"`, `"bottom"`, `"top"`,
#' or a two-element numeric vector.
#' Default is `"right"`.
#' @param legend.direction The direction of the legend.
#' Can be one of `"vertical"` or `"horizontal"`.
#' Default is `"vertical"`.
#' @param theme_use The theme to use for the plot.
#' Default is `"theme_this"`.
#' @param theme_args Additional arguments to pass to the theme function.
#' Default is `list()`.
#' @param x_text_angle Rotation angle for x-axis labels.
#' Default is `45`. Must be a finite number.
#' @param grid_major Whether to show major panel grid lines.
#' Default is `TRUE`.
#' @param grid_major_colour Color of major panel grid lines.
#' @param grid_major_linetype Linetype of major panel grid lines.
#' @param grid_major_linewidth Line width of major panel grid lines.
#' @param combine Whether to combine multiple plots into one.
#' Default is `TRUE`.
#' @param nrow Number of rows when combining plots.
#' Default is `NULL`.
#' @param ncol Number of columns when combining plots.
#' Default is `NULL`.
#' @param byrow Whether to fill plots by row when combining.
#' Default is `TRUE`.
#' @param force Whether to force plotting even when variables have more than 100 levels.
#' Default is `FALSE`.
#' @param seed Random seed for reproducibility.
#' Default is `11`.
#'
#' @export
#'
#' @examples
#' set.seed(1)
#' meta_data <- data.frame(
#'   Type = factor(
#'     sample(c("A", "B", "C"),
#'       50,
#'       replace = TRUE,
#'       prob = c(0.5, 0.3, 0.2)
#'     )
#'   ),
#'   Group = factor(sample(c("X", "Y", "Z"), 50, replace = TRUE)),
#'   Batch = factor(sample(c("B1", "B2"), 50, replace = TRUE))
#' )
#' meta_data$Region <- factor(
#'   ifelse(meta_data$Group %in% c("X", "Y"), "R1", "R2"),
#'   levels = c("R1", "R2")
#' )
#'
#' StatPlot(
#'   meta_data,
#'   stat.by = "Type",
#'   group.by = "Group",
#'   split.by = "Batch",
#'   plot_type = "bar",
#'   position = "dodge"
#' )
#'
#' StatPlot(
#'   meta_data,
#'   stat.by = "Type",
#'   group.by = "Group",
#'   stat_type = "count",
#'   plot_type = "ring",
#'   position = "dodge"
#' )
#'
#' StatPlot(
#'   meta_data,
#'   stat.by = "Type",
#'   group.by = "Group",
#'   stat_type = "count"
#' )
#'
#' StatPlot(
#'   meta_data,
#'   stat.by = "Type",
#'   plot_type = "pie"
#' )
#'
#' StatPlot(
#'   meta_data,
#'   stat.by = "Type",
#'   group.by = "Group",
#'   stat_type = "count",
#'   plot_type = "area"
#' )
#'
#' StatPlot(
#'   meta_data,
#'   stat.by = "Type",
#'   group.by = "Group",
#'   plot_type = "dot"
#' )
#'
#' StatPlot(
#'   meta_data,
#'   stat.by = "Type",
#'   group.by = "Group",
#'   stat_type = "count",
#'   plot_type = "trend"
#' )
#'
#' rank_data <- data.frame(
#'   Feature = paste0("Feature", 1:12),
#'   Group = rep(c("Control", "Treatment"), each = 6),
#'   Value = c(
#'     -2.4, 1.8, 1.2, -0.8, 0.6, 0.2,
#'     2.7, -2.1, 1.5, 0.9, -0.5, 0.3
#'   )
#' )
#'
#' StatPlot(
#'   rank_data,
#'   stat.by = "Feature",
#'   value.by = "Value",
#'   group.by = "Group",
#'   stat_type = "value",
#'   plot_type = "bar",
#'   top_n = 4,
#'   flip = TRUE,
#'   palette = "RdBu"
#' )
#'
#' StatPlot(
#'   rank_data,
#'   stat.by = "Feature",
#'   value.by = "Value",
#'   group.by = "Group",
#'   stat_type = "value",
#'   plot_type = "dot",
#'   top_n = 3,
#'   palette = "RdBu"
#' )
#'
#' importance_data <- data.frame(
#'   Variable = paste0("Variable", 1:8),
#'   Importance = c(0.91, 0.78, 0.64, 0.52, 0.43, 0.31, 0.18, 0.09)
#' )
#' StatPlot(
#'   importance_data,
#'   stat.by = "Variable",
#'   value.by = "Importance",
#'   stat_type = "value",
#'   top_n = 5,
#'   flip = TRUE,
#'   bar_fill = "#3C8DBC"
#' )
StatPlot <- function(
  meta.data,
  stat.by,
  group.by = NULL,
  split.by = NULL,
  bg.by = NULL,
  flip = FALSE,
  NA_color = "grey",
  NA_stat = TRUE,
  keep_empty = FALSE,
  individual = FALSE,
  stat_level = NULL,
  plot_type = c(
    "bar",
    "rose",
    "ring",
    "pie",
    "trend",
    "trend_alluvial",
    "area",
    "dot",
    "sankey",
    "chord",
    "venn",
    "upset"
  ),
  venn_engine = c("ggVennDiagram", "venny"),
  venn_args = list(),
  stat_type = c("percent", "count", "value"),
  value.by = NULL,
  top_n = Inf,
  rank.by = c("absolute", "value"),
  complete_groups = NULL,
  value_cutoff = NULL,
  value_limits = NULL,
  value_midpoint = 0,
  value_legend_title = NULL,
  bar_fill = NULL,
  bar_width = 0.8,
  point_size = c(2.5, 7),
  character_width = 50,
  return_data = FALSE,
  position = c("stack", "dodge"),
  palette = "Chinese",
  palcolor = NULL,
  alpha = 1,
  bg_palette = "Chinese",
  bg_palcolor = NULL,
  bg_alpha = 0.2,
  label = FALSE,
  label_cutoff = NULL,
  label.size = 3.5,
  label.fg = "black",
  label.bg = "white",
  label.bg.r = 0.1,
  aspect.ratio = NULL,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  theme_use = "theme_this",
  theme_args = list(),
  x_text_angle = 45,
  grid_major = TRUE,
  grid_major_colour = "grey80",
  grid_major_linetype = 2,
  grid_major_linewidth = 0.3,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  force = FALSE,
  seed = 11
) {
  set.seed(seed)

  stat_type <- match.arg(stat_type)
  plot_type <- match.arg(plot_type)
  rank.by <- match.arg(rank.by)
  venn_engine <- match.arg(venn_engine)
  position <- match.arg(position)
  x_text_angle <- tryCatch(
    suppressWarnings(as.numeric(x_text_angle)),
    error = function(e) NA_real_
  )
  if (length(x_text_angle) != 1L || !is.finite(x_text_angle)) {
    log_message(
      "{.arg x_text_angle} must be a finite number",
      message_type = "error"
    )
  }
  if (!is.null(label_cutoff)) {
    label_cutoff <- tryCatch(
      suppressWarnings(as.numeric(label_cutoff)),
      error = function(e) NA_real_
    )
    if (length(label_cutoff) != 1L || !is.finite(label_cutoff)) {
      log_message(
        "{.arg label_cutoff} must be a finite number or {.code NULL}",
        message_type = "error"
      )
    }
  }
  venn_args <- venn_args %||% list()

  if (identical(stat_type, "value")) {
    if (!plot_type %in% c("bar", "dot")) {
      log_message(
        "{.arg stat_type = 'value'} supports {.arg plot_type} values {.val bar} and {.val dot}",
        message_type = "error"
      )
    }
    if (length(stat.by) != 1L || length(value.by) != 1L) {
      log_message(
        "{.arg stat.by} and {.arg value.by} must each name one column in value mode",
        message_type = "error"
      )
    }
    if (length(group.by) > 1L) {
      log_message(
        "{.arg group.by} must name at most one column in value mode",
        message_type = "error"
      )
    }
    if (identical(plot_type, "dot") && is.null(group.by)) {
      group.by <- "All.groups"
      meta.data[[group.by]] <- factor("")
      xlab <- xlab %||% ""
    }
    complete_groups <- complete_groups %||% identical(plot_type, "dot")
    grid_major_element <- if (isTRUE(grid_major)) {
      element_line(
        colour = grid_major_colour,
        linetype = grid_major_linetype,
        linewidth = grid_major_linewidth
      )
    } else {
      element_blank()
    }
    return(stat_value_plot(
      data = meta.data,
      label = stat.by,
      score = value.by,
      group = group.by,
      facet = split.by,
      plot_type = plot_type,
      top_n = top_n,
      top_by = unique(c(split.by, group.by)),
      complete_groups = complete_groups,
      score_cutoff = value_cutoff,
      sort_by = if (identical(rank.by, "absolute")) "absolute" else "score",
      palette = palette,
      palcolor = palcolor,
      limits = value_limits,
      midpoint = value_midpoint,
      bar_fill = bar_fill,
      bar_width = bar_width,
      point_size = point_size,
      alpha = alpha,
      title = title,
      xlab = xlab,
      ylab = ylab %||% value_legend_title %||% value.by,
      legend_title = value_legend_title %||% value.by,
      character_width = character_width,
      flip = flip,
      theme_use = theme_use,
      theme_args = theme_args,
      x_text_angle = x_text_angle,
      grid_major_element = grid_major_element,
      legend.position = legend.position,
      legend.direction = legend.direction,
      return_data = return_data
    ))
  }

  if (identical(plot_type, "trend_alluvial")) {
    check_r("ggalluvial", verbose = FALSE)
    if (identical(position, "dodge")) {
      log_message(
        "{.arg position} is forcibly set to {.val stack} when {.arg plot_type = 'trend_alluvial'}",
        message_type = "warning"
      )
      position <- "stack"
    }
  }

  grid_major_element <- if (isTRUE(grid_major)) {
    element_line(
      colour = grid_major_colour,
      linetype = grid_major_linetype,
      linewidth = grid_major_linewidth
    )
  } else {
    element_blank()
  }

  if (nrow(meta.data) == 0) {
    log_message(
      "{.arg meta.data} is empty",
      message_type = "error"
    )
  }
  if (is.null(group.by)) {
    group.by <- "All.groups"
    xlab <- ""
    meta.data[[group.by]] <- factor("")
  }
  if (is.null(split.by)) {
    split.by <- "All.groups"
    meta.data[[split.by]] <- factor("")
  }

  for (i in unique(c(group.by, split.by, bg.by))) {
    if (!i %in% colnames(meta.data)) {
      log_message(
        "{.val {i}} is not in the {.arg meta.data}",
        message_type = "error"
      )
    }
    if (!is.factor(meta.data[[i]])) {
      meta.data[[i]] <- factor(meta.data[[i]], levels = unique(meta.data[[i]]))
    }
  }

  bg_map <- NULL
  if (!is.null(bg.by)) {
    for (g in group.by) {
      df_table <- table(meta.data[[g]], meta.data[[bg.by]])
      if (max(rowSums(df_table > 0), na.rm = TRUE) > 1) {
        log_message(
          "{.arg group.by} must be a part of {.arg bg.by}",
          message_type = "error"
        )
      } else {
        bg_map[[g]] <- stats::setNames(
          colnames(df_table)[apply(df_table, 1, function(x) which(x > 0))],
          rownames(df_table)
        )
      }
    }
  } else {
    for (g in group.by) {
      bg_map[[g]] <- stats::setNames(
        levels(meta.data[[g]]),
        levels(meta.data[[g]])
      )
    }
  }

  for (i in unique(stat.by)) {
    if (!i %in% colnames(meta.data)) {
      log_message(
        "{.val {i}} is not in the {.arg meta.data}",
        message_type = "error"
      )
    }
    if (plot_type %in% c("venn", "upset")) {
      if (!is.factor(meta.data[[i]]) && !is.logical(meta.data[[i]])) {
        meta.data[[i]] <- factor(
          meta.data[[i]],
          levels = unique(meta.data[[i]])
        )
      }
    } else if (!is.factor(meta.data[[i]])) {
      meta.data[[i]] <- factor(meta.data[[i]], levels = unique(meta.data[[i]]))
    }
  }

  plot_types <- c("sankey", "chord", "venn", "upset")
  if (length(stat.by) >= 2) {
    if (!plot_type %in% plot_types) {
      log_message(
        "{.arg plot_type} must be one of {.val {plot_types}} when multiple {.arg stat.by} provided",
        message_type = "error"
      )
    }
    if (length(stat.by) > 2 && plot_type == "chord") {
      log_message(
        "{.arg stat.by} can only be a vector of length 2 when {.arg plot_type = 'chord'}",
        message_type = "error"
      )
    }
    if (length(stat.by) > 7 && plot_type == "venn") {
      log_message(
        "{.arg stat.by} can only be a vector of length <= 7 when {.arg plot_type = 'venn'}",
        message_type = "error"
      )
    }
  }
  if (identical(plot_type, "venn") && identical(venn_engine, "venny")) {
    if (!is.list(venn_args)) {
      log_message(
        "{.arg venn_args} must be a list",
        message_type = "error"
      )
    }
    if ("data" %in% names(venn_args)) {
      log_message(
        "{.arg venn_args} cannot contain {.arg data}; it is generated internally",
        message_type = "error"
      )
    }
    if (length(stat.by) < 2 || length(stat.by) > 4) {
      log_message(
        "{.arg stat.by} must contain 2 to 4 sets when {.arg venn_engine = 'venny'}",
        message_type = "error"
      )
    }
    check_r("venny", verbose = FALSE)
  }

  levels <- unique(
    unlist(
      lapply(
        meta.data[, stat.by, drop = FALSE],
        function(x) {
          if (is.factor(x)) {
            return(levels(x))
          }
          if (is.logical(x)) {
            return(as.character(unique(x)))
          }
        }
      )
    )
  )

  if (plot_type %in% c("venn", "upset")) {
    if (is.null(stat_level)) {
      stat_level <- lapply(stat.by, function(stat) {
        levels(meta.data[[stat]])[1] %||% sort(unique(meta.data[[stat]]))[1]
      })
      log_message(
        "{.arg stat_level} is set to {.val {stat_level}}"
      )
    } else {
      if (length(stat_level) == 1) {
        stat_level <- rep(stat_level, length(stat.by))
      }
      if (length(stat_level) != length(stat.by)) {
        log_message(
          "{.arg stat_level} must be of length 1 or the same length as {.arg stat.by}",
          message_type = "error"
        )
      }
    }
    if (is.null(names(stat_level))) {
      names(stat_level) <- stat.by
    }
    for (i in stat.by) {
      meta.data[[i]] <- meta.data[[i]] %in% stat_level[[i]]
    }
  }

  if (plot_type %in% c("rose", "ring", "pie")) {
    aspect.ratio <- 1
  }

  if (any(group.by != "All.groups") && plot_type %in% plot_types) {
    log_message(
      "{.arg group.by} is not used when plot {.val {plot_types}}",
      message_type = "warning"
    )
  }
  if (stat_type == "percent" && plot_type %in% plot_types) {
    log_message(
      "{.arg stat_type} is forcibly set to {.val count} when plot {.val {plot_types}}",
      message_type = "warning"
    )
    stat_type <- "count"
  }

  dat_all <- meta.data[,
    unique(c(stat.by, group.by, split.by, bg.by)),
    drop = FALSE
  ]
  nlev <- sapply(dat_all, nlevels)
  nlev <- nlev[nlev > 100]
  if (length(nlev) > 0 && isFALSE(force)) {
    log_message(
      "The following variables have more than 100 levels: {.val {names(nlev)}}",
      message_type = "warning"
    )
    answer <- utils::askYesNo("Are you sure to continue?", default = FALSE)
    if (isFALSE(answer)) {
      return(invisible(NULL))
    }
  }

  dat_split <- split.data.frame(dat_all, dat_all[[split.by]])
  plist <- list()
  if (plot_type %in% c(
    "bar", "rose", "ring", "pie", "trend", "trend_alluvial", "area", "dot"
  )) {
    xlab <- xlab %||% group.by
    ylab <- ylab %||% ifelse(stat_type == "count", "Count", "Percentage")
    if (identical(theme_use, "theme_blank")) {
      theme_args[["xlab"]] <- xlab
      theme_args[["ylab"]] <- ylab
      if (plot_type %in% c("rose", "ring", "pie")) {
        theme_args[["add_coord"]] <- FALSE
      }
    }
    colors <- palette_colors(
      dat_all[[stat.by]],
      palette = palette,
      palcolor = palcolor,
      NA_color = NA_color,
      NA_keep = TRUE
    )

    comb_list <- list()
    comb <- expand.grid(
      stat_name = stat.by,
      group_name = group.by,
      stringsAsFactors = FALSE
    )
    if (isTRUE(individual)) {
      for (g in group.by) {
        comb_list[[g]] <- merge(
          comb,
          expand.grid(
            group_name = g,
            group_element = levels(dat_all[[g]]),
            split_name = levels(dat_all[[split.by]]),
            stringsAsFactors = FALSE
          ),
          by = "group_name"
        )
      }
    } else {
      for (g in group.by) {
        comb_list[[g]] <- merge(
          comb,
          expand.grid(
            group_name = g,
            group_element = list(levels(dat_all[[g]])),
            split_name = levels(dat_all[[split.by]]),
            stringsAsFactors = FALSE
          ),
          by = "group_name"
        )
      }
    }
    comb <- do.call(rbind, comb_list)
    rownames(comb) <- paste0(
      comb[["group_name"]],
      ":",
      sapply(comb[["group_element"]], function(x) paste0(x, collapse = ",")),
      ":",
      comb[["split_name"]]
    )

    plist <- lapply(
      stats::setNames(rownames(comb), rownames(comb)),
      function(i) {
        stat.by <- comb[i, "stat_name"]
        sp <- comb[i, "split_name"]
        g <- comb[i, "group_name"]
        single_group <- comb[[i, "group_element"]]
        dat_split_use <- dat_split[[ifelse(split.by == "All.groups", 1, sp)]]
        colors_use <- colors[
          names(colors) %in%
            dat_split_use[[stat.by]]
        ]
        if (
          any(is.na(dat_split_use[[
            stat.by
          ]])) &&
            isTRUE(NA_stat)
        ) {
          colors_use <- c(colors_use, colors["NA"])
        }
        if (stat_type == "percent") {
          dat_use <- dat_split_use |>
            stats::xtabs(
              formula = paste0("~", stat.by, "+", g),
              addNA = NA_stat
            ) |>
            as.data.frame() |>
            dplyr::group_by(
              dplyr::across(
                dplyr::all_of(g)
              ),
              .drop = FALSE
            ) |>
            dplyr::mutate(groupn = sum(Freq)) |>
            dplyr::group_by(
              dplyr::across(
                dplyr::all_of(
                  c(stat.by, g)
                )
              ),
              .drop = FALSE
            ) |>
            dplyr::mutate(value = Freq / groupn) |>
            as.data.frame()
        } else {
          dat_use <- dat_split_use |>
            stats::xtabs(
              formula = paste0("~", stat.by, "+", g),
              addNA = NA_stat
            ) |>
            as.data.frame() |>
            dplyr::mutate(value = Freq)
        }
        dat <- dat_use[dat_use[[g]] %in% single_group, , drop = FALSE]
        dat[[g]] <- factor(
          dat[[g]],
          levels = levels(dat[[g]])[levels(dat[[g]]) %in% dat[[g]]]
        )
        dat <- dat[!is.na(dat[["value"]]), , drop = FALSE]
        if (!is.null(bg.by)) {
          bg <- bg.by
          bg_color <- palette_colors(
            levels(dat_all[[bg]]),
            palette = bg_palette,
            palcolor = bg_palcolor
          )
        } else {
          bg <- g
          bg_color <- palette_colors(
            levels(dat_all[[bg]]),
            palcolor = bg_palcolor %||%
              rep(c("transparent", "grey85"), nlevels(dat_all[[bg]]))
          )
        }

        if (isTRUE(flip)) {
          dat[[g]] <- factor(dat[[g]], levels = rev(levels(dat[[g]])))
          aspect.ratio <- 1 / aspect.ratio
          if (length(aspect.ratio) == 0 || is.na(aspect.ratio)) {
            aspect.ratio <- NULL
          }
        }
        if (plot_type == "ring") {
          dat[[g]] <- factor(dat[[g]], levels = c("   ", levels(dat[[g]])))
          dat <- rbind(dat, dat[nrow(dat) + 1, , drop = FALSE])
          dat[nrow(dat), g] <- "   "
        }
        if (plot_type == "dot") {
          position_use <- position_identity()
          scalex <- scale_x_discrete(drop = !keep_empty)
        } else {
          if (position == "stack") {
            position_use <- position_stack(vjust = 0.5)
            scalex <- scale_x_discrete(drop = !keep_empty, expand = c(0, 0))
            scaley <- scale_y_continuous(
              limits = if (
                identical(plot_type, "trend_alluvial") &&
                  identical(stat_type, "percent")
              ) {
                c(0, 1)
              } else {
                NULL
              },
              labels = if (stat_type == "count") {
                scales::number
              } else {
                scales::percent
              },
              expand = c(0, 0)
            )
          } else if (position == "dodge") {
            if (plot_type == "area") {
              position_use <- position_dodge2(width = 0.9, preserve = "total")
            } else {
              position_use <- position_dodge2(width = 0.9, preserve = "single")
            }
            scalex <- scale_x_discrete(drop = !keep_empty)
            scaley <- scale_y_continuous(
              limits = c(0, max(dat[["value"]], na.rm = TRUE) * 1.1),
              labels = if (stat_type == "count") {
                scales::number
              } else {
                scales::percent
              },
              expand = c(0, 0)
            )
          }
        }
        if (position == "stack") {
          bg_layer <- NULL
        } else {
          bg_data <- stats::na.omit(unique(dat[, g, drop = FALSE]))
          bg_data[["x"]] <- as.numeric(bg_data[[g]])
          bg_data[["xmin"]] <- ifelse(
            bg_data[["x"]] == min(bg_data[["x"]]),
            -Inf,
            bg_data[["x"]] - 0.5
          )
          bg_data[["xmax"]] <- ifelse(
            bg_data[["x"]] == max(bg_data[["x"]]),
            Inf,
            bg_data[["x"]] + 0.5
          )
          bg_data[["ymin"]] <- -Inf
          bg_data[["ymax"]] <- Inf
          bg_data[["fill"]] <- bg_color[bg_map[[g]][as.character(bg_data[[g]])]]
          bg_layer <- geom_rect(
            data = bg_data,
            xmin = bg_data[["xmin"]],
            xmax = bg_data[["xmax"]],
            ymin = bg_data[["ymin"]],
            ymax = bg_data[["ymax"]],
            fill = bg_data[["fill"]],
            alpha = bg_alpha,
            inherit.aes = FALSE
          )
        }

        if (isTRUE(label)) {
          label_value <- if (is.null(label_cutoff)) {
            dat[["value"]]
          } else {
            ifelse(dat[["value"]] > label_cutoff, dat[["value"]], NA_real_)
          }
          dat[[".label_text"]] <- if (stat_type == "count") {
            label_value
          } else {
            ifelse(
              is.na(label_value),
              NA_character_,
              paste0(round(label_value * 100, 1), "%")
            )
          }
        }

        if (plot_type == "bar") {
          p <- ggplot(
            dat,
            aes(x = .data[[g]], y = value, group = .data[[stat.by]])
          ) +
            bg_layer +
            geom_col(
              aes(fill = .data[[stat.by]]),
              width = 0.8,
              color = "black",
              alpha = alpha,
              position = position_use
            ) +
            scalex +
            scaley
        }
        if (plot_type == "trend") {
          dat_area <- dat[rep(seq_len(nrow(dat)), each = 2), , drop = FALSE]
          dat_area[[g]] <- as.numeric(dat_area[[g]])
          dat_area[seq(1, nrow(dat_area), 2), g] <- dat_area[
            seq(1, nrow(dat_area), 2),
            g
          ] -
            0.3
          dat_area[seq(2, nrow(dat_area), 2), g] <- dat_area[
            seq(2, nrow(dat_area), 2),
            g
          ] +
            0.3
          p <- ggplot(
            dat,
            aes(x = .data[[g]], y = value, fill = .data[[stat.by]])
          ) +
            bg_layer +
            geom_area(
              data = dat_area,
              mapping = aes(x = .data[[g]], fill = .data[[stat.by]]),
              alpha = alpha / 2,
              color = "grey50",
              position = position_use
            ) +
            geom_col(
              aes(fill = .data[[stat.by]]),
              width = 0.6,
              color = "black",
              alpha = alpha,
              position = position_use
            ) +
            scalex +
            scaley
        }
        if (plot_type == "trend_alluvial") {
          p <- ggplot(
            dat,
            aes(
              x = .data[[g]],
              y = value,
              fill = .data[[stat.by]],
              stratum = .data[[stat.by]],
              alluvium = .data[[stat.by]]
            )
          ) +
            bg_layer +
            geom_col(
              width = 0.6,
              color = NA,
              alpha = alpha,
              position = position_use
            ) +
            ggalluvial::geom_flow(
              width = 0.6,
              alpha = alpha * 0.22,
              knot.pos = 0.35,
              color = "white"
            ) +
            ggalluvial::geom_alluvium(
              width = 0.6,
              alpha = alpha,
              knot.pos = 0.35,
              fill = NA,
              color = "white"
            ) +
            scalex +
            scaley
        }

        textpath_layer <- NULL
        if (plot_type == "rose" && g != "All.groups") {
          group_levels <- levels(dat[[g]])
          group_levels <- group_levels[group_levels != ""]
          n_grp <- length(group_levels)
          if (n_grp > 0) {
            if (position == "stack") {
              y_total <- stats::aggregate(
                dat[["value"]],
                by = list(dat[[g]]),
                FUN = sum,
                na.rm = TRUE
              )
            } else {
              y_total <- stats::aggregate(
                dat[["value"]],
                by = list(dat[[g]]),
                FUN = max,
                na.rm = TRUE
              )
            }
            colnames(y_total) <- c("group", "y_max")
            y_total <- y_total[
              y_total[["group"]] %in% group_levels,
              ,
              drop = FALSE
            ]
            y_max_global <- max(y_total[["y_max"]], na.rm = TRUE)
            y_outer <- y_max_global * 1.15
            npt <- 40
            path_margin <- 0.04
            path_df <- do.call(
              rbind,
              lapply(seq_along(group_levels), function(idx) {
                lev <- group_levels[idx]
                x_idx <- which(levels(dat[[g]]) == lev)
                x_start <- x_idx - 0.5 + path_margin
                x_end <- x_idx + 0.5 - path_margin
                data.frame(
                  x = seq(x_start, x_end, length.out = npt),
                  y = y_outer,
                  label = lev,
                  group = idx
                )
              })
            )
            if (!is.null(path_df) && nrow(path_df) > 0) {
              textpath_layer <- geomtextpath::geom_textpath(
                aes(x = x, y = y, label = label, group = group),
                data = path_df,
                inherit.aes = FALSE,
                size = label.size,
                color = label.fg,
                linewidth = 0,
                upright = TRUE
              )
              y_lim_max <- max(
                y_outer * 1.05,
                if (position == "dodge") {
                  max(dat[["value"]], na.rm = TRUE) * 1.1
                } else {
                  max(dat[["value"]], na.rm = TRUE)
                }
              )
              scaley <- scale_y_continuous(
                limits = c(0, y_lim_max),
                labels = if (stat_type == "count") {
                  scales::number
                } else {
                  scales::percent
                },
                expand = c(0, 0)
              )
            }
          }
        }

        if (plot_type == "rose") {
          p <- ggplot(
            dat,
            aes(x = .data[[g]], y = value, group = .data[[stat.by]])
          ) +
            bg_layer +
            geom_col(
              aes(fill = .data[[stat.by]]),
              width = 0.8,
              color = "black",
              alpha = alpha,
              position = position_use
            ) +
            scalex +
            scaley +
            coord_polar(theta = "x", start = ifelse(flip, pi / 2, 0)) +
            textpath_layer
        }
        if (plot_type == "ring" || plot_type == "pie") {
          p <- ggplot(
            dat,
            aes(x = .data[[g]], y = value, group = .data[[stat.by]])
          ) +
            bg_layer +
            geom_col(
              aes(fill = .data[[stat.by]]),
              width = 0.8,
              color = "black",
              alpha = alpha,
              position = position_use
            ) +
            scalex +
            scaley +
            coord_polar(theta = "y", start = ifelse(flip, pi / 2, 0))
        }
        if (plot_type == "area") {
          p <- ggplot(
            dat,
            aes(x = .data[[g]], y = value, group = .data[[stat.by]])
          ) +
            bg_layer +
            geom_area(
              aes(fill = .data[[stat.by]]),
              color = "black",
              alpha = alpha,
              position = position_use
            ) +
            scalex +
            scaley
        }
        if (plot_type == "dot") {
          p <- ggplot(dat, aes(x = .data[[g]], y = .data[[stat.by]])) +
            bg_layer +
            geom_point(
              aes(fill = .data[[stat.by]], size = value),
              color = "black",
              alpha = alpha,
              shape = 21,
              position = position_use
            ) +
            scalex +
            scale_size_area(name = capitalize(stat_type), max_size = 12) +
            guides(size = guide_legend(override.aes = list(fill = "grey30")))
        }
        if (isTRUE(label)) {
          label_data <- dat[!is.na(dat[[".label_text"]]), , drop = FALSE]
          if (plot_type == "dot") {
            p <- p +
              ggrepel::geom_text_repel(
                aes(
                  x = .data[[g]],
                  y = .data[[stat.by]],
                  label = .data[[".label_text"]],
                ),
                data = label_data,
                colour = label.fg,
                size = label.size,
                bg.color = label.bg,
                bg.r = label.bg.r,
                point.size = NA,
                max.overlaps = 100,
                min.segment.length = 0,
                force = 0,
                position = position_use
              )
          } else {
            p <- p +
              ggrepel::geom_text_repel(
                aes(
                  label = .data[[".label_text"]],
                  y = value
                ),
                data = label_data,
                colour = label.fg,
                size = label.size,
                bg.color = label.bg,
                bg.r = label.bg.r,
                point.size = NA,
                max.overlaps = 100,
                min.segment.length = 0,
                force = 0,
                position = position_use
              )
          }
        }
        if (plot_type %in% c("rose")) {
          axis_text_x <- if (!is.null(textpath_layer)) {
            element_blank()
          } else {
            element_text()
          }
        } else if (plot_type %in% c("ring", "pie")) {
          axis_text_x <- element_text()
        } else {
          axis_text_x <- element_text(
            angle = x_text_angle,
            hjust = 1,
            vjust = 1
          )
        }
        title <- title %||% sp
        p <- p +
          labs(title = title, subtitle = subtitle, x = xlab, y = ylab) +
          scale_fill_manual(
            name = paste0(stat.by, ":"),
            values = colors_use,
            na.value = colors_use["NA"],
            drop = FALSE,
            limits = names(colors_use),
            na.translate = TRUE
          ) +
          do.call(theme_use, theme_args) +
          theme(
            aspect.ratio = aspect.ratio,
            axis.text.x = axis_text_x,
            legend.position = legend.position,
            legend.direction = legend.direction,
            panel.grid.major = grid_major_element
          ) +
          guides(
            fill = guide_legend(
              title.hjust = 0,
              order = 1,
              override.aes = list(size = 4, color = "black", alpha = 1)
            )
          )
        if (isTRUE(flip) && !plot_type %in% c("pie", "rose")) {
          p <- p + coord_flip()
        }
        return(p)
      }
    )
  } else if (plot_type %in% c("chord", "sankey", "venn", "upset")) {
    colors <- palette_colors(stat.by, palette = palette, palcolor = palcolor)
    nlev <- nlevels(dat_all[[split.by]])
    chord_use_temp <- plot_type == "chord" && isTRUE(combine) && nlev > 1L
    venny_details <- list()
    if (plot_type == "chord" && isTRUE(combine)) {
      if (chord_use_temp) {
        temp <- tempfile(fileext = "png")
        grDevices::png(temp)
        grDevices::dev.control("enable")
        if (is.null(nrow) && is.null(ncol)) {
          nrow <- ceiling(sqrt(nlev))
          ncol <- ceiling(nlev / nrow)
        }
        if (is.null(nrow)) {
          nrow <- ceiling(sqrt(ncol))
        }
        if (is.null(ncol)) {
          ncol <- ceiling(sqrt(nrow))
        }
        graphics::par(mfrow = c(nrow, ncol))
      } else {
        grDevices::dev.control("enable")
      }
    }

    for (sp in levels(dat_all[[split.by]])) {
      dat_use <- dat_split[[ifelse(split.by == "All.groups", 1, sp)]]
      if (plot_type == "venn") {
        dat_list <- as.list(dat_use[, stat.by])
        dat_list <- lapply(
          stats::setNames(
            names(dat_list),
            names(dat_list)
          ),
          function(x) {
            lg <- dat_list[[x]]
            names(lg) <- rownames(dat_use)
            cellkeep <- names(lg)[lg]
            cellkeep
          }
        )
        if (identical(venn_engine, "venny")) {
          venny_result <- do.call(
            venny::venny,
            c(list(data = dat_list), venn_args)
          )
          venny_detail <- NULL
          if (
            is.list(venny_result) &&
              !inherits(venny_result, c("gg", "ggplot")) &&
              !is.null(venny_result[["venn"]])
          ) {
            venny_detail <- venny_result
            p <- venny_result[["venn"]]
          } else {
            p <- venny_result
          }
          if (!inherits(p, c("gg", "ggplot"))) {
            log_message(
              "{.fun venny::venny} must return a ggplot object or a list containing {.arg venn}",
              message_type = "error"
            )
          }
          p <- p + labs(x = sp, title = title, subtitle = subtitle)
          if (!is.null(venny_detail)) {
            attr(p, "venny_detail") <- venny_detail
            venny_details[[sp]] <- venny_detail
          }
        } else {
          venn <- ggVennDiagram::Venn(dat_list)
          data <- ggVennDiagram::process_data(venn)
          dat_venn_region <- ggVennDiagram::venn_region(data)
          idname <- dat_venn_region[["name"]][
            dat_venn_region[["name"]] %in% stat.by
          ]
          names(idname) <- dat_venn_region[["id"]][
            dat_venn_region[["name"]] %in% stat.by
          ]
          idcomb <- strsplit(dat_venn_region[["id"]], split = "")
          colorcomb <- lapply(
            idcomb,
            function(x) colors[idname[as.character(x)]]
          )
          dat_venn_region[["colors"]] <- sapply(
            colorcomb,
            function(x) blendcolors(x, mode = "blend")
          )
          dat_venn_region[["label"]] <- paste0(
            dat_venn_region[["count"]],
            "\n",
            round(
              dat_venn_region[["count"]] /
                sum(dat_venn_region[["count"]]) * 100,
              1
            ),
            "%"
          )
          dat_venn_setedge <- ggVennDiagram::venn_setedge(data)
          dat_venn_setedge[["colors"]] <- colors[stat.by[as.numeric(
            dat_venn_setedge[["id"]]
          )]]

          venn_regionedge_data <- ggVennDiagram::venn_regionedge(data)
          venn_regionedge_data[["colors"]] <- dat_venn_region[["colors"]][
            match(
              venn_regionedge_data[["id"]],
              dat_venn_region[["id"]]
            )
          ]

          p <- ggplot() +
            geom_polygon(
              data = venn_regionedge_data,
              aes(X, Y, fill = colors, group = id),
              alpha = alpha
            ) +
            geom_path(
              data = dat_venn_setedge,
              aes(X, Y, group = id),
              color = "black",
              linewidth = 1,
              show.legend = FALSE
            ) +
            ggrepel::geom_text_repel(
              data = ggVennDiagram::venn_setlabel(data),
              aes(
                X,
                Y,
                label = paste0(
                  name,
                  "\n(",
                  count,
                  ")"
                )
              ),
              fontface = "bold",
              colour = label.fg,
              size = label.size + 0.5,
              bg.color = label.bg,
              bg.r = label.bg.r,
              point.size = NA,
              max.overlaps = 100,
              force = 0,
              min.segment.length = 0,
              segment.colour = "black"
            ) +
            ggrepel::geom_text_repel(
              data = ggVennDiagram::venn_regionlabel(data),
              aes(X, Y, label = count),
              colour = label.fg,
              size = label.size,
              bg.color = label.bg,
              bg.r = label.bg.r,
              point.size = NA,
              max.overlaps = 100,
              force = 0,
              min.segment.length = 0,
              segment.colour = "black"
            ) +
            scale_fill_identity() +
            coord_fixed(ratio = 1, clip = "off") +
            theme(
              plot.title = element_text(hjust = 0.5),
              plot.background = element_blank(),
              panel.background = element_blank(),
              axis.title.y = element_blank(),
              axis.text = element_blank(),
              axis.ticks = element_blank()
            )
          p <- p + labs(x = sp, title = title, subtitle = subtitle)
        }
      }

      if (plot_type == "upset") {
        for (n in seq_len(nrow(dat_use))) {
          dat_use[["intersection"]][n] <- list(
            stat.by[unlist(dat_use[
              n,
              stat.by
            ])]
          )
        }
        dat_use <- dat_use[
          sapply(dat_use[["intersection"]], length) > 0,
          ,
          drop = FALSE
        ]
        p <- ggplot(
          dat_use,
          aes(x = intersection)
        ) +
          geom_bar(
            aes(fill = after_stat(count)),
            color = "black",
            width = 0.5,
            show.legend = FALSE
          ) +
          ggrepel::geom_text_repel(
            aes(label = after_stat(count)),
            stat = "count",
            colour = label.fg,
            size = label.size,
            bg.color = label.bg,
            bg.r = label.bg.r,
            point.size = NA,
            max.overlaps = 100,
            force = 0,
            min.segment.length = 0,
            segment.colour = "black"
          ) +
          labs(
            title = title,
            subtitle = subtitle,
            x = sp,
            y = "Intersection size"
          ) +
          ggupset::scale_x_upset(sets = stat.by, n_intersections = 20) +
          scale_fill_gradientn(
            colors = palette_colors(palette = "material-indigo")
          ) +
          theme_this(
            aspect.ratio = 0.6,
            panel.grid.major = grid_major_element
          ) +
          ggupset::theme_combmatrix(
            combmatrix.label.text = element_text(size = 12, color = "black"),
            combmatrix.label.extra_spacing = 6
          )
        p <- p + labs(title = title, subtitle = subtitle)
      }

      if (plot_type == "sankey") {
        colors <- palette_colors(
          c(
            unique(
              unlist(
                lapply(
                  dat_all[, stat.by, drop = FALSE],
                  levels
                )
              )
            ),
            NA
          ),
          palette = palette,
          palcolor = palcolor,
          NA_keep = TRUE,
          NA_color = NA_color
        )

        legend_list <- list()
        for (l in stat.by) {
          df <- data.frame(
            factor(levels(dat_use[[l]]), levels = levels(dat_use[[l]]))
          )
          colnames(df) <- l

          legend_list[[l]] <- get_legend(
            ggplot(data = df) +
              geom_col(
                aes(x = 1, y = 1, fill = .data[[l]]),
                color = "black"
              ) +
              scale_fill_manual(
                values = colors[levels(dat_use[[l]])]
              ) +
              guides(
                fill = guide_legend(
                  title.hjust = 0,
                  title.vjust = 0,
                  order = 1,
                  override.aes = list(size = 4, color = "black", alpha = 1)
                )
              ) +
              theme_this(
                legend.position = "bottom",
                legend.direction = legend.direction
              )
          )

          if (any(is.na(dat_use[[l]]))) {
            raw_levels <- levels(dat_use[[l]])
            dat_use[[l]] <- as.character(dat_use[[l]])
            dat_use[[l]][is.na(dat_use[[l]])] <- "NA"
            dat_use[[l]] <- factor(dat_use[[l]], levels = c(raw_levels, "NA"))
          }
        }

        if (legend.direction == "vertical") {
          legend <- do.call(cbind, legend_list)
        } else {
          legend <- do.call(rbind, legend_list)
        }

        dat <- suppressWarnings(
          make_long(
            dat_use,
            dplyr::all_of(stat.by)
          )
        )
        dat$node <- factor(dat$node, levels = rev(names(colors)))
        p0 <- ggplot(
          dat,
          aes(
            x = x,
            next_x = next_x,
            node = node,
            next_node = next_node,
            fill = node
          )
        ) +
          geom_sankey(
            color = "black",
            flow.alpha = alpha,
            show.legend = FALSE,
            na.rm = FALSE
          ) +
          scale_fill_manual(values = colors, drop = FALSE) +
          scale_x_discrete(expand = c(0, 0.2)) +
          theme_void() +
          theme(axis.text.x = element_text())
        gtable <- as_grob(p0)
        gtable <- add_grob(
          gtable = gtable,
          grob = legend,
          position = legend.position
        )
        p <- patchwork::wrap_plots(gtable)
      }

      if (plot_type == "chord") {
        colors <- palette_colors(
          c(
            unique(
              unlist(
                lapply(
                  dat_all[, stat.by, drop = FALSE],
                  levels
                )
              )
            ),
            NA
          ),
          palette = palette,
          palcolor = palcolor,
          NA_keep = TRUE,
          NA_color = NA_color
        )
        M <- table(
          dat_use[[stat.by[1]]],
          dat_use[[stat.by[2]]],
          useNA = "ifany"
        )
        m <- matrix(M, ncol = ncol(M), dimnames = dimnames(M))
        colnames(m)[is.na(colnames(m))] <- "NA"
        circlize::chordDiagram(
          m,
          grid.col = colors,
          transparency = 0.2,
          link.lwd = 1,
          link.lty = 1,
          link.border = 1
        )
        circlize::circos.clear()
        p <- grDevices::recordPlot()
      }

      plist[[sp]] <- p
    }
  }
  if (isTRUE(combine) && plot_type == "chord") {
    plot <- grDevices::recordPlot()
    if (chord_use_temp) {
      grDevices::dev.off()
      unlink(temp)
    }
    return(plot)
  }
  if (isTRUE(combine) && plot_type != "chord") {
    if (length(plist) > 1) {
      plot <- patchwork::wrap_plots(
        plotlist = plist,
        nrow = nrow,
        ncol = ncol,
        byrow = byrow
      )
      if (
        identical(plot_type, "venn") &&
          identical(venn_engine, "venny") &&
          length(venny_details) > 0
      ) {
        attr(plot, "venny_detail") <- venny_details
      }
    } else {
      plot <- plist[[1]]
    }
    return(plot)
  } else {
    return(plist)
  }
}

stat_value_plot <- function(
  data,
  label,
  score,
  group = NULL,
  facet = NULL,
  plot_type = c("bar", "dot", "comparison"),
  top_n = Inf,
  top_by = NULL,
  complete_groups = identical(plot_type, "comparison"),
  score_cutoff = NULL,
  sort_by = c("absolute", "score"),
  palette = "RdBu",
  palcolor = NULL,
  limits = NULL,
  midpoint = 0,
  bar_fill = NULL,
  bar_width = 0.72,
  point_size = c(2.5, 7),
  alpha = 1,
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  legend_title = "Score",
  character_width = 50,
  flip = TRUE,
  theme_use = "theme_this",
  theme_args = list(),
  x_text_angle = 45,
  grid_major_element = element_line(
    colour = "grey80",
    linetype = 2,
    linewidth = 0.3
  ),
  legend.position = "right",
  legend.direction = "vertical",
  return_data = FALSE
) {
  plot_type <- match.arg(plot_type)
  sort_by <- match.arg(sort_by)
  if (!is.data.frame(data)) {
    log_message("{.arg data} must be a data frame", message_type = "error")
  }
  columns <- unique(c(label, score, group, facet, top_by))
  columns <- columns[!is.na(columns) & nzchar(columns)]
  missing_columns <- setdiff(columns, colnames(data))
  if (length(missing_columns) > 0L) {
    log_message(
      "Columns not found in {.arg data}: {.val {missing_columns}}",
      message_type = "error"
    )
  }
  if (length(label) != 1L || length(score) != 1L) {
    log_message(
      "{.arg label} and {.arg score} must each name one column",
      message_type = "error"
    )
  }
  if (plot_type %in% c("dot", "comparison") && is.null(group)) {
    log_message(
      "{.arg group} is required for grouped dot plots",
      message_type = "error"
    )
  }

  df <- data
  df[[score]] <- suppressWarnings(as.numeric(df[[score]]))
  keep <- is.finite(df[[score]]) &
    !is.na(df[[label]]) &
    nzchar(as.character(df[[label]]))
  if (!is.null(score_cutoff)) {
    keep <- keep & abs(df[[score]]) >= abs(score_cutoff)
  }
  df <- df[keep, , drop = FALSE]
  if (nrow(df) == 0L) {
    log_message(
      "No finite scores remain after filtering",
      message_type = "error"
    )
  }

  top_by <- top_by %||% unique(c(facet, group))
  top_by <- top_by[!is.na(top_by) & nzchar(top_by)]
  rank_value <- if (identical(sort_by, "absolute")) {
    abs(df[[score]])
  } else {
    df[[score]]
  }
  df[[".stat_value_rank"]] <- rank_value
  df[[".stat_value_row"]] <- seq_len(nrow(df))
  selected <- stat_value_top_rows(df, top_n = top_n, top_by = top_by)

  if (
    isTRUE(complete_groups) &&
      plot_type %in% c("dot", "comparison") &&
      !is.null(group)
  ) {
    selected_keys <- unique(stat_value_key(selected, c(facet, label)))
    df <- df[
      stat_value_key(df, c(facet, label)) %in% selected_keys,
      ,
      drop = FALSE
    ]
  } else {
    df <- selected
  }
  df <- df[
    order(df[[".stat_value_rank"]], decreasing = TRUE, na.last = NA),
    ,
    drop = FALSE
  ]

  label_text <- as.character(df[[label]])
  label_text <- vapply(
    label_text,
    function(x) {
      if (is.na(x)) {
        return(NA_character_)
      }
      paste(base::strwrap(x, width = character_width), collapse = "\n")
    },
    character(1),
    USE.NAMES = FALSE
  )
  df[[".stat_value_label"]] <- factor(
    label_text,
    levels = unique(rev(label_text))
  )
  df[[".stat_value_abs"]] <- abs(df[[score]])

  if (is.null(limits)) {
    score_limit <- max(abs(df[[score]] - midpoint), na.rm = TRUE)
    if (!is.finite(score_limit) || score_limit <= 0) {
      score_limit <- 1
    }
    limits <- midpoint + c(-score_limit, score_limit)
  }
  colors <- palette_colors(
    palette = palette,
    palcolor = palcolor,
    matched = FALSE
  )
  if (length(colors) < 3L) {
    colors <- grDevices::colorRampPalette(colors)(3L)
  }
  scale_colors <- colors[round(seq(1, length(colors), length.out = 3L))]

  theme_fun <- if (is.function(theme_use)) {
    theme_use
  } else {
    get(theme_use, mode = "function", inherits = TRUE)
  }
  facet_columns <- unique(c(
    facet,
    if (identical(plot_type, "bar")) group else NULL
  ))
  facet_columns <- facet_columns[!is.na(facet_columns) & nzchar(facet_columns)]

  if (identical(plot_type, "bar")) {
    p <- ggplot(
      df,
      aes(x = .data[[".stat_value_label"]], y = .data[[score]])
    )
    if (is.null(bar_fill)) {
      p <- p +
        geom_col(
          aes(fill = .data[[score]]),
          width = bar_width,
          color = "black",
          alpha = alpha
        ) +
        scale_fill_gradient2(
          low = scale_colors[[1]],
          mid = scale_colors[[2]],
          high = scale_colors[[3]],
          midpoint = midpoint,
          limits = limits,
          name = legend_title
        )
    } else {
      p <- p + geom_col(
        width = bar_width,
        fill = bar_fill,
        color = "black",
        alpha = alpha
      )
    }
    p <- p +
      geom_hline(yintercept = midpoint, color = "grey75", linewidth = 0.35) +
      labs(title = title, x = xlab, y = ylab %||% legend_title)
    if (isTRUE(flip)) p <- p + coord_flip()
  } else {
    p <- ggplot(
      df,
      aes(x = .data[[group]], y = .data[[".stat_value_label"]])
    ) +
      geom_point(
        aes(size = .data[[".stat_value_abs"]], fill = .data[[score]]),
        shape = 21,
        color = "black",
        alpha = alpha
      ) +
      scale_size_continuous(
        name = paste0("|", legend_title, "|"),
        range = point_size
      ) +
      scale_fill_gradient2(
        low = scale_colors[[1]],
        mid = scale_colors[[2]],
        high = scale_colors[[3]],
        midpoint = midpoint,
        limits = limits,
        name = legend_title
      ) +
      labs(title = title, x = xlab %||% group, y = ylab)
  }
  if (length(facet_columns) > 0L) {
    p <- p +
      facet_wrap(
        stats::as.formula(paste("~", paste(facet_columns, collapse = "+"))),
        scales = if (identical(plot_type, "bar") && isTRUE(flip)) {
          "free_x"
        } else {
          "free_y"
        }
      )
  }
  p <- p +
    do.call(theme_fun, theme_args) +
    theme(
      legend.position = legend.position,
      legend.direction = legend.direction,
      panel.grid.major = grid_major_element,
      axis.text.x = element_text(
        angle = if (identical(plot_type, "bar") && isTRUE(flip)) 0 else x_text_angle,
        hjust = 1,
        vjust = 1
      )
    )

  df[[".stat_value_rank"]] <- NULL
  df[[".stat_value_row"]] <- NULL
  df[[".stat_value_label"]] <- NULL
  df[[".stat_value_abs"]] <- NULL
  rownames(df) <- NULL
  if (isTRUE(return_data)) {
    return(list(plot = p, data = df))
  }
  p
}

stat_value_top_rows <- function(data, top_n, top_by) {
  if (is.null(top_n) || length(top_n) == 0L || is.infinite(top_n)) {
    return(data)
  }
  if (length(top_n) != 1L || is.na(top_n) || top_n < 1L) {
    log_message(
      "{.arg top_n} must be a positive number or {.val Inf}",
      message_type = "error"
    )
  }
  split_index <- if (length(top_by) == 0L) {
    list(All = seq_len(nrow(data)))
  } else {
    split(seq_len(nrow(data)), stat_value_key(data, top_by), drop = TRUE)
  }
  keep <- unlist(
    lapply(split_index, function(index) {
      index <- index[order(
        data[[".stat_value_rank"]][index],
        decreasing = TRUE,
        na.last = NA
      )]
      utils::head(index, as.integer(top_n))
    }),
    use.names = FALSE
  )
  data[sort(unique(keep)), , drop = FALSE]
}

stat_value_key <- function(data, columns) {
  columns <- columns[!is.na(columns) & nzchar(columns)]
  if (length(columns) == 0L) {
    return(rep("All", nrow(data)))
  }
  do.call(paste, c(lapply(data[columns], as.character), sep = "\r"))
}

#' @title Slim unused data in the plot
#'
#' @description
#' Remove unused columns from the data in a ggplot or patchwork object.
#' This function keeps only the columns that are actually used in the plot
#' (e.g., in mappings, aesthetics, or facets), which can significantly reduce
#' the object size when the original data contains many unused columns.
#'
#' @md
#' @param p A `ggplot` object or a `patchwork` object.
#'
#' @return A `ggplot` or `patchwork` object with unused data columns removed.
#'
#' @export
#'
#' @examples
#' library(ggplot2)
#' p <- ggplot(
#'   data = mtcars,
#'   aes(x = mpg, y = wt, colour = cyl)
#' ) +
#'   geom_point()
#' object.size(p)
#' colnames(p$data)
#'
#' p_slim <- slim_data(p)
#' object.size(p_slim)
#' colnames(p_slim$data)
slim_data <- function(p) {
  UseMethod(generic = "slim_data", object = p)
}

#' @export
#' @rdname slim_data
#' @method slim_data ggplot
slim_data.ggplot <- function(p) {
  vars <- get_vars(p)
  if (length(vars) > 0) {
    p$data <- p$data[, intersect(colnames(p$data), vars), drop = FALSE]
    for (i in seq_along(p$layers)) {
      if (length(p$layers[[i]]$data) > 0) {
        ids <- intersect(colnames(p$layers[[i]]$data), vars)
        p$layers[[i]]$data <- p$layers[[i]]$data[, ids, drop = FALSE]
      }
    }
  }
  return(p)
}

#' @export
#' @rdname slim_data
#' @method slim_data patchwork
slim_data.patchwork <- function(p) {
  for (i in seq_along(p$patches$plots)) {
    p$patches$plots[[i]] <- slim_data(p$patches$plots[[i]])
  }
  p <- slim_data.ggplot(p)
  return(p)
}

#' @export
#' @method slim_data default
slim_data.default <- function(p) {
  return(p)
}

#' @title Chinese traditional colors system
#'
#' @description
#' A color system based on Chinese traditional colors with 1058 colors.
#'
#' @md
#' @return
#' A `ChineseColors` object.
#' Detailed information can be found in [print.ChineseColors()].
#'
#' @importFrom grDevices rgb colorRampPalette col2rgb rgb2hsv
#' @export
#'
#' @seealso
#' [chinese_colors] for the dataset of Chinese traditional colors.
#' [get_chinese_palettes] for getting Chinese color palettes.
#' [visual_colors] for visualizing any color vector.
#' [get_colors] for searching colors in dataset and palettes.
#'
#' @examples
#' cc <- ChineseColors()
#' cc
#'
#' # Get a color by pinyin
#' get_colors("pinlan")
#'
#' # By number
#' get_colors(44)
#'
#' # By hex code
#' get_colors("#2B73AF")
#'
#' # Multiple colors
#' get_colors("pinlan", "piao")
#' get_colors(91:100)
#'
#' # Chinese names
#' cc$visual_colors(
#'   loc_range = c(1, 180),
#'   num_per_row = 30,
#'   title = "Chinese Traditional Colors",
#'   name_type = "chinese"
#' )
#'
#' # pinyin as names
#' cc$visual_colors(
#'   loc_range = c(1, 120),
#'   title = "Chinese Traditional Colors",
#'   name_type = "pinyin"
#' )
#'
#' # rgb as names
#' cc$visual_colors(
#'   loc_range = c(1, 120),
#'   title = "Colors with RGB values",
#'   name_type = "rgb"
#' )
#'
#' # hex as names
#' cc$visual_colors(
#'   loc_range = c(1, 120),
#'   title = "Colors with hex codes",
#'   name_type = "hex"
#' )
ChineseColors <- function() {
  colors_df <- thisplot::chinese_colors
  category_ranges <- list()
  unique_categories <- unique(colors_df$category)
  for (cat in unique_categories) {
    cat_indices <- which(colors_df$category == cat)
    if (length(cat_indices) > 0) {
      category_ranges[[cat]] <- cat_indices
    }
  }

  color_ranges <- list()
  for (cat in names(category_ranges)) {
    cat_indices <- category_ranges[[cat]]
    n_cat <- length(cat_indices)

    if (n_cat <= 20) {
      color_ranges[[paste0(cat, "1")]] <- cat_indices
    } else if (n_cat <= 50) {
      mid <- ceiling(n_cat / 2)
      color_ranges[[paste0(cat, "1")]] <- cat_indices[1:mid]
      color_ranges[[paste0(cat, "2")]] <- cat_indices[(mid + 1):n_cat]
    } else if (n_cat <= 100) {
      third1 <- ceiling(n_cat / 3)
      third2 <- ceiling(2 * n_cat / 3)
      color_ranges[[paste0(cat, "1")]] <- cat_indices[1:third1]
      color_ranges[[paste0(cat, "2")]] <- cat_indices[(third1 + 1):third2]
      color_ranges[[paste0(cat, "3")]] <- cat_indices[(third2 + 1):n_cat]
    } else {
      quarter1 <- ceiling(n_cat / 4)
      quarter2 <- ceiling(2 * n_cat / 4)
      quarter3 <- ceiling(3 * n_cat / 4)
      color_ranges[[paste0(cat, "1")]] <- cat_indices[1:quarter1]
      color_ranges[[paste0(cat, "2")]] <- cat_indices[(quarter1 + 1):quarter2]
      color_ranges[[paste0(cat, "3")]] <- cat_indices[(quarter2 + 1):quarter3]
      color_ranges[[paste0(cat, "4")]] <- cat_indices[(quarter3 + 1):n_cat]
    }
  }

  sub_palettes <- lapply(
    color_ranges, function(range) {
      valid_range <- range[range >= 1 & range <= nrow(colors_df)]
      if (length(valid_range) == 0) {
        return(character(0))
      }
      indices <- valid_range[valid_range <= nrow(colors_df)]
      colors_df$hex[indices]
    }
  )

  main_palettes <- list()
  for (cat in unique_categories) {
    cat_sub_pals <- sub_palettes[grepl(paste0("^", cat), names(sub_palettes))]
    if (length(cat_sub_pals) > 0) {
      main_palettes[[cat]] <- unlist(cat_sub_pals, use.names = FALSE)
    }
  }

  visual_colors <- function(
    loc_range = c(1, nrow(colors_df)),
    num_per_row = 30,
    title = NULL,
    name_type = c("pinyin", "chinese", "rgb", "hex")
  ) {
    name_type <- match.arg(name_type)

    start_idx <- if (loc_range[1] == 0) 1 else max(1, loc_range[1])
    end_idx <- min(nrow(colors_df), loc_range[2])
    if (loc_range[2] == 0) {
      end_idx <- nrow(colors_df)
    }
    indices <- start_idx:end_idx

    colors_subset <- colors_df[indices, ]
    color_col <- colors_subset$hex
    rgb_col <- colors_subset$rgb

    if (name_type == "chinese") {
      display_name <- colors_subset$name_ch
    } else if (name_type == "pinyin") {
      display_name <- colors_subset$name
    } else if (name_type == "rgb") {
      display_name <- rgb_col
    } else if (name_type == "hex") {
      display_name <- color_col
    } else {
      display_name <- colors_subset$name
    }

    return(
      thisplot::visual_colors(
        colors = color_col,
        names = display_name,
        num_per_row = num_per_row,
        title = title,
        label_mode = name_type
      )
    )
  }

  cc_obj <- list(
    colors = colors_df
  )

  for (cat in names(main_palettes)) {
    cc_obj[[cat]] <- main_palettes[[cat]]
  }

  for (sub_name in names(sub_palettes)) {
    cc_obj[[sub_name]] <- sub_palettes[[sub_name]]
  }

  cc_obj$visual_colors <- visual_colors

  class(cc_obj) <- "ChineseColors"
  return(cc_obj)
}

#' @title Print `ChineseColors` object
#'
#' @md
#' @param x A `ChineseColors` object.
#' @param ... Additional arguments.
#'
#' @return
#' Details of the `ChineseColors` object.
#'
#' @method print ChineseColors
#' @export
print.ChineseColors <- function(x, ...) {
  cli::cli_h3("Chinese Traditional Colors System")
  cli::cli_h3("Total {nrow(x$colors)} colors")
  cli::cli_ul(
    c(
      "blue: {length(x$blue)} colors",
      "cyan: {length(x$cyan)} colors",
      "gray_brown: {length(x$gray_brown)} colors",
      "green: {length(x$green)} colors",
      "orange: {length(x$orange)} colors",
      "purple: {length(x$purple)} colors",
      "red: {length(x$red)} colors",
      "yellow: {length(x$yellow)} colors"
    )
  )
  cli::cli_h3("Methods:")
  cli::cli_ul(
    "visual_colors(loc_range, num_per_row, title, name_type)"
  )
  cli::cli_h3("See also:")
  cli::cli_ul("[get_colors()] for searching colors")
}

#' @title Set the panel width/height of a plot to a fixed value
#'
#' @description
#' The ggplot object, when stored, can only specify the height and width of the entire plot, not the panel.
#' The latter is obviously more important to control the final result of a plot.
#' This function can set the panel width/height of plot to a fixed value and rasterize it.
#'
#' @md
#' @inheritParams thisutils::log_message
#' @param x A ggplot object, a grob object, or a combined plot made by patchwork or cowplot package.
#' @param panel_index Specify the panel to be fixed.
#' If `NULL`, will fix all panels.
#' @param respect Whether row heights and column widths should respect each other.
#' @param width The desired width of the fixed panels.
#' @param height The desired height of the fixed panels.
#' @param margin The margin to add around each panel, in inches.
#' Default is `1`.
#' @param padding The padding to add around each panel, in inches.
#' Default is `0`.
#' @param units The units in which `height`, `width` and `margin` are given.
#' Can be `"mm"`, `"cm"`, `"in"`, etc. See [grid::unit].
#' @param raster Whether to rasterize the panel.
#' @param dpi Plot resolution.
#' @param return_grob Whether to return a grob object instead of a wrapped `patchwork` object.
#' Default is `FALSE`.
#' @param save `NULL` or the file name used to save the plot.
#' @param bg_color The background color of the plot.
#' @param ... Additional arguments passed to other functions.
#'
#' @return If `return_grob` is `TRUE`, returns a gtable object.
#' Otherwise, returns a patchwork object with fixed panel sizes.
#' The returned object has a `size` attribute containing width, height, and units.
#'
#' @export
#'
#' @examples
#' library(ggplot2)
#' p <- ggplot(
#'   data = mtcars, aes(x = mpg, y = wt, colour = cyl)
#' ) +
#'   geom_point() +
#'   facet_wrap(~gear, nrow = 2)
#' # fix the size of panel
#' panel_fix(
#'   p,
#'   width = 5,
#'   height = 3,
#'   units = "cm"
#' )
#' # rasterize the panel
#' panel_fix(
#'   p,
#'   width = 5,
#'   height = 3,
#'   units = "cm",
#'   raster = TRUE,
#'   dpi = 90
#' )
#'
#' # `panel_fix` will build and render the plot when input a ggplot object.
#' # so after `panel_fix`, the size of the object will be changed.
#' object.size(p)
#' object.size(
#'   panel_fix(
#'     p,
#'     width = 5,
#'     height = 3,
#'     units = "cm"
#'   )
#' )
panel_fix <- function(
  x = NULL,
  panel_index = NULL,
  respect = NULL,
  width = NULL,
  height = NULL,
  margin = 1,
  padding = 0,
  units = "in",
  raster = FALSE,
  dpi = 300,
  return_grob = FALSE,
  bg_color = "white",
  save = NULL,
  verbose = FALSE,
  ...
) {
  if (!inherits(x, "gtable")) {
    tryCatch(
      {
        gtable <- as_gtable(x)
      },
      error = function(error) {
        log_message(
          error, "\nCannot convert the x to a gtable object",
          message_type = "error"
        )
      }
    )
  } else {
    gtable <- x
  }
  args <- as.list(match.call())[-1]
  depth <- args[["depth"]]
  if (is.null(depth)) {
    depth <- 1
  }

  if (is.null(panel_index)) {
    non_zero <- grep(
      pattern = "zeroGrob",
      vapply(gtable$grobs, as.character, character(1)),
      invert = TRUE
    )
    panel_index <- grep(
      pattern = "panel|full",
      gtable[["layout"]][["name"]]
    )
    panel_index <- intersect(panel_index, non_zero)
  }
  if (length(panel_index) == 0 && length(gtable$grobs) == 1) {
    panel_index <- 1
  }
  add_margin <- TRUE
  for (i in panel_index) {
    geom_index <- grep(
      pattern = "GeomDrawGrob",
      names(gtable$grobs[[i]][["children"]])
    )
    if (length(geom_index) > 0) {
      log_message(
        "panel {.val {i}} is detected as generated by plot_grid",
        verbose = verbose
      )
      for (j in geom_index) {
        subgrob <- gtable$grobs[[i]][["children"]][[j]][["children"]][[1]][["children"]][[1]]

        if (length(subgrob$grobs[[1]][["children"]]) > 0 &&
          all(sapply(subgrob$grobs[[1]][["children"]], function(x) inherits(x, "recordedGrob")))) {
          subgrob <- panel_fix_overall(
            x = subgrob$grobs[[1]][["children"]],
            width = width,
            height = height,
            margin = padding,
            units = units,
            raster = raster,
            dpi = dpi,
            return_grob = TRUE
          )
        } else {
          subgrob <- panel_fix(
            x = subgrob,
            width = width,
            height = height,
            margin = padding,
            units = units,
            raster = raster,
            dpi = dpi,
            return_grob = TRUE,
            verbose = verbose,
            depth = depth + 1
          )
        }
        gtable$grobs[[i]][["children"]][[j]][["children"]][[1]][["children"]][[1]] <- subgrob
      }
      sum_width <- grid::convertWidth(
        sum(subgrob[["widths"]]),
        unitTo = units,
        valueOnly = TRUE
      ) / as.numeric(gtable$grobs[[i]][["children"]][[j]]$vp$width)
      sum_height <- grid::convertHeight(
        sum(subgrob[["heights"]]),
        unitTo = units,
        valueOnly = TRUE
      ) / as.numeric(gtable$grobs[[i]][["children"]][[j]]$vp$height)
      gtable <- panel_fix_overall(
        gtable,
        panel_index = i,
        width = sum_width,
        height = sum_height,
        margin = ifelse(depth == 1, margin, 0),
        units = units,
        raster = FALSE,
        return_grob = TRUE
      )
    } else if (gtable$grobs[[i]]$name == "layout" || inherits(x, "patchwork")) {
      log_message(
        "panel {.val {i}} is detected as generated by patchwork",
        verbose = verbose
      )
      # if (i == panel_index[1] && length(panel_index) > 1 && isTRUE(verbose)) {
      #   log_message("More than 2 panels detected. panel_fix may not work as expected.")
      # }
      subgrob <- gtable$grobs[[i]]
      if (length(subgrob[["children"]]) > 0 &&
        all(sapply(subgrob[["children"]], function(x) inherits(x, "recordedGrob")))) {
        subgrob <- panel_fix_overall(
          subgrob[["children"]],
          width = width,
          height = height,
          margin = 0,
          units = units,
          raster = raster,
          dpi = dpi,
          return_grob = TRUE
        )
      } else {
        subgrob <- panel_fix(
          subgrob,
          width = width,
          height = height,
          margin = 0,
          units = units,
          raster = raster,
          dpi = dpi,
          return_grob = TRUE,
          verbose = verbose,
          depth = depth + 1
        )
      }
      gtable$grobs[[i]] <- subgrob
      layout <- gtable$layout
      layout[["rowranges"]] <- lapply(
        seq_len(nrow(layout)),
        function(n) layout$t[n]:layout$b[n]
      )
      layout[["colranges"]] <- lapply(
        seq_len(nrow(layout)),
        function(n) layout$l[n]:layout$r[n]
      )
      p_row <- c(layout$t[i], layout$b[i])
      p_col <- c(layout$l[i], layout$r[i])
      background_index <- grep(
        pattern = "background", layout$name
      )
      background_index <- background_index[order(layout$z[background_index], decreasing = TRUE)]
      for (bgi in background_index) {
        if (all(p_row %in% layout[["rowranges"]][[bgi]]) && all(p_col %in% layout[["colranges"]][[bgi]])) {
          p_background_index <- bgi
          break
        }
      }
      gtable <- gtable::gtable_add_rows(
        gtable,
        heights = grid::unit(padding, units),
        pos = layout$t[p_background_index] - 1
      )
      gtable <- gtable::gtable_add_rows(
        gtable,
        heights = grid::unit(padding, units),
        pos = layout$b[p_background_index]
      )
      gtable <- gtable::gtable_add_cols(
        gtable,
        widths = grid::unit(padding, units),
        pos = layout$l[p_background_index] - 1
      )
      gtable <- gtable::gtable_add_cols(
        gtable,
        widths = grid::unit(padding, units),
        pos = layout$r[p_background_index]
      )
      sum_width <- grid::convertWidth(
        sum(subgrob[["widths"]]),
        unitTo = units,
        valueOnly = TRUE
      )
      sum_height <- grid::convertHeight(
        sum(subgrob[["heights"]]),
        unitTo = units,
        valueOnly = TRUE
      )

      gtable <- panel_fix_overall(
        gtable,
        panel_index = i,
        width = sum_width,
        height = sum_height,
        margin = ifelse(depth == 1 & add_margin, margin, 0),
        units = units,
        raster = FALSE,
        respect = TRUE,
        return_grob = TRUE
      )
      if (depth == 1 & add_margin) {
        add_margin <- FALSE
      }
    } else {
      gtable <- panel_fix_overall(
        gtable,
        panel_index = i,
        width = width,
        height = height,
        margin = margin,
        units = units,
        raster = raster,
        dpi = dpi,
        return_grob = TRUE
      )
    }
  }

  if (!is.null(respect)) {
    gtable$respect <- respect
  }

  if (isTRUE(return_grob)) {
    return(gtable)
  } else {
    p <- patchwork::wrap_plots(gtable) +
      theme(
        plot.background = element_rect(
          fill = bg_color, color = bg_color
        )
      )
    if (units != "null") {
      plot_width <- grid::convertWidth(
        sum(gtable[["widths"]]),
        unitTo = units,
        valueOnly = TRUE
      )
      plot_height <- grid::convertHeight(
        sum(gtable[["heights"]]),
        unitTo = units,
        valueOnly = TRUE
      )
      attr(p, "size") <- list(
        width = plot_width,
        height = plot_height,
        units = units
      )
    }

    if (!is.null(save) && is.character(save) && nchar(save) > 0) {
      if (units == "null") {
        log_message(
          "{.arg units} can not be 'null' if want to save the plot",
          message_type = "error"
        )
      }
      filename <- normalizePath(save)
      log_message(
        "Save the plot to the file: {.file {filename}}",
        verbose = verbose
      )
      if (!dir.exists(dirname(filename))) {
        dir.create(dirname(filename), recursive = TRUE, showWarnings = FALSE)
      }
      ggplot2::ggsave(
        plot = p,
        filename = filename,
        width = plot_width,
        height = plot_height,
        units = units,
        dpi = dpi,
        limitsize = FALSE
      )
    }
    return(p)
  }
}

#' @rdname panel_fix
#' @export
panel_fix_overall <- function(
  x,
  panel_index = NULL,
  respect = NULL,
  width = NULL,
  height = NULL,
  margin = 1,
  units = "in",
  raster = FALSE,
  dpi = 300,
  return_grob = FALSE,
  bg_color = "white",
  save = NULL,
  verbose = TRUE
) {
  if (!inherits(x, "gtable")) {
    if (inherits(x, "gTree")) {
      x <- x[["children"]]
    }
    tryCatch(
      {
        gtable <- as_gtable(x)
      },
      error = function(error) {
        log_message(
          error, "\nCannot convert the x to a gtable object",
          message_type = "error"
        )
      }
    )
  } else {
    gtable <- x
  }

  if (is.null(panel_index)) {
    non_zero <- grep(
      pattern = "zeroGrob",
      vapply(
        gtable$grobs, as.character, character(1)
      ), invert = TRUE
    )
    panel_index <- grep("panel|full", gtable[["layout"]][["name"]])
    panel_index <- intersect(panel_index, non_zero)
  }
  if (length(panel_index) == 0 && length(gtable$grobs) == 1) {
    panel_index <- 1
  }
  if (!length(width) %in% c(0, 1, length(panel_index)) || !length(height) %in% c(0, 1, length(panel_index))) {
    log_message(
      "The length of 'width' and 'height' must be 1 or the length of panels.",
      message_type = "error"
    )
  }

  if (inherits(x, "gList")) {
    panel_index <- 1
    panel_index_h <- panel_index_w <- list(1)
    w_comp <- h_comp <- list(grid::unit(1, "null"))
    w <- h <- list(grid::unit(1, "null"))
  } else if (length(panel_index) > 0) {
    panel_index_w <- panel_index_h <- list()
    w_comp <- h_comp <- list()
    w <- h <- list()
    for (i in seq_along(panel_index)) {
      index <- panel_index[i]
      panel_index_h[[i]] <- sort(
        unique(c(
          gtable[["layout"]][["t"]][index],
          gtable[["layout"]][["b"]][index]
        ))
      )
      panel_index_w[[i]] <- sort(
        unique(c(
          gtable[["layout"]][["l"]][index],
          gtable[["layout"]][["r"]][index]
        ))
      )
      w_comp[[i]] <- gtable[["widths"]][seq(min(panel_index_w[[i]]), max(panel_index_w[[i]]))]
      h_comp[[i]] <- gtable[["heights"]][seq(min(panel_index_h[[i]]), max(panel_index_h[[i]]))]

      if (length(w_comp[[i]]) == 1) {
        w[[i]] <- w_comp[[i]]
      } else if (length(w_comp[[i]]) > 1 && any(grid::unitType(w_comp[[i]]) == "null")) {
        w[[i]] <- grid::unit(1, units = "null")
      } else {
        w[[i]] <- sum(w_comp[[i]])
      }
      if (length(h_comp[[i]]) == 1) {
        h[[i]] <- h_comp[[i]]
      } else if (length(h_comp[[i]]) > 1 && any(grid::unitType(h_comp[[i]]) == "null")) {
        h[[i]] <- grid::unit(1, units = "null")
      } else {
        h[[i]] <- sum(h_comp[[i]])
      }
    }
  } else {
    log_message(
      "No panel detected",
      message_type = "error"
    )
  }

  if (units != "null") {
    raw_w <- sapply(
      w, function(x) {
        grid::convertWidth(x, unitTo = units, valueOnly = TRUE)
      }
    )
    raw_h <- sapply(
      h, function(x) {
        grid::convertHeight(x, unitTo = units, valueOnly = TRUE)
      }
    )
    for (i in seq_along(w)) {
      if (grid::unitType(w[[i]]) == "null" || grid::convertUnit(w[[i]], unitTo = "cm", valueOnly = TRUE) < 1e-10) {
        raw_w[i] <- 0
      }
    }
    for (i in seq_along(h)) {
      if (grid::unitType(h[[i]]) == "null" || grid::convertUnit(h[[i]], unitTo = "cm", valueOnly = TRUE) < 1e-10) {
        raw_h[i] <- 0
      }
    }
    if (isTRUE(gtable$respect)) {
      raw_aspect <- sapply(h, as.vector) / sapply(w, as.vector)
    } else {
      if (all(raw_w != 0) && all(raw_h != 0)) {
        raw_aspect <- raw_h / raw_w
      } else {
        raw_aspect <- grid::convertHeight(
          grid::unit(1, "npc"), "cm",
          valueOnly = TRUE
        ) / grid::convertWidth(grid::unit(1, "npc"), "cm", valueOnly = TRUE)
      }
    }

    if (is.null(width) && is.null(height)) {
      width <- raw_w
      height <- raw_h
      if (all(width == 0) && all(height == 0)) {
        width <- grid::convertWidth(
          grid::unit(1, "npc"), units,
          valueOnly = TRUE
        )
        height <- grid::convertHeight(
          grid::unit(1, "npc"), units,
          valueOnly = TRUE
        )
        if (isTRUE(gtable$respect)) {
          if (raw_aspect <= 1) {
            height <- width * raw_aspect
          } else {
            width <- height / raw_aspect
          }
        }
      }
    }

    for (i in seq_along(raw_aspect)) {
      if (is.finite(raw_aspect[i]) && raw_aspect[i] != 0) {
        if (is.null(width[i]) || is.na(width[i]) || width[i] == 0) {
          width[i] <- height[i] / raw_aspect[i]
        }
        if (is.null(height[i]) || is.na(height[i]) || height[i] == 0) {
          height[i] <- width[i] * raw_aspect[i]
        }
      }
    }

    for (i in seq_along(width)) {
      if (inherits(width[i], "unit")) {
        width[i] <- grid::convertWidth(
          width[i],
          unitTo = units,
          valueOnly = TRUE
        )
      }
    }
    for (i in seq_along(height)) {
      if (inherits(height[i], "unit")) {
        height[i] <- grid::convertHeight(
          height[i],
          unitTo = units,
          valueOnly = TRUE
        )
      }
    }
  }

  if (length(width) == 1) {
    width <- rep(width, length(panel_index))
  }
  if (length(height) == 1) {
    height <- rep(height, length(panel_index))
  }
  for (i in seq_along(panel_index)) {
    if (!is.null(width)) {
      width_unit <- width[i] / length(w_comp[[i]])
      gtable[["widths"]][seq(min(panel_index_w[[i]]), max(panel_index_w[[i]]))] <- rep(
        grid::unit(width_unit, units = units), length(w_comp[[i]])
      )
    }
    if (!is.null(height)) {
      height_unit <- height[i] / length(h_comp[[i]])
      gtable[["heights"]][seq(min(panel_index_h[[i]]), max(panel_index_h[[i]]))] <- rep(
        grid::unit(height_unit, units = units), length(h_comp[[i]])
      )
    }
  }
  gtable <- gtable::gtable_add_padding(
    gtable,
    padding = grid::unit(margin, units = units)
  )

  if (isTRUE(raster)) {
    for (i in seq_along(panel_index)) {
      index <- panel_index[i]
      g <- g_new <- gtable$grobs[[index]]
      vp <- g$vp
      children_order <- g$childrenOrder
      if (is.null(g$vp)) {
        g$vp <- grid::viewport()
      }

      for (j in seq_along(g[["children"]])) {
        child <- g[["children"]][[j]]
        child_nm <- names(g[["children"]])[j]
        if (!is.null(child$vp) ||
          any(grepl("(text)|(label)", child_nm)) ||
          any(grepl("(text)|(segments)|(legend)", class(child$list[[1]])))) {
          zero <- ggplot2::zeroGrob()
          zero$name <- g[["children"]][[j]]$name
          g[["children"]][[j]] <- zero
        } else if (inherits(child$list[[1]], "grob") || is.null(child$list[[1]])) {
          g_new[["children"]][[j]] <- ggplot2::zeroGrob()
        }
      }
      temp <- tempfile(fileext = "png")
      ragg::agg_png(
        temp,
        width = width[i],
        height = height[i],
        bg = "transparent",
        res = dpi,
        units = units
      )
      grid::grid.draw(g)
      grDevices::dev.off()
      g_ras <- grid::rasterGrob(
        png::readPNG(temp, native = TRUE)
      )
      unlink(temp)
      g <- grid::addGrob(g_new, g_ras)
      g$vp <- vp
      g$childrenOrder <- c(g_ras$name, children_order)
      gtable$grobs[[index]] <- g
    }
  }

  if (!is.null(respect)) {
    gtable$respect <- respect
  }

  if (isTRUE(return_grob)) {
    return(gtable)
  } else {
    p <- patchwork::wrap_plots(gtable) +
      theme(
        plot.background = element_rect(
          fill = bg_color, color = bg_color
        )
      )
    if (units != "null") {
      plot_width <- grid::convertWidth(
        sum(gtable[["widths"]]),
        unitTo = units,
        valueOnly = TRUE
      )
      plot_height <- grid::convertHeight(
        sum(gtable[["heights"]]),
        unitTo = units,
        valueOnly = TRUE
      )
      attr(p, "size") <- list(
        width = plot_width,
        height = plot_height,
        units = units
      )
    }

    if (!is.null(save) && is.character(save) && nchar(save) > 0) {
      if (units == "null") {
        log_message(
          "{.arg units} can not be 'null' if want to save the plot",
          message_type = "error"
        )
      }
      filename <- normalizePath(save)
      log_message(
        "Save plot to: {.file {filename}}",
        verbose = verbose
      )
      if (!dir.exists(dirname(filename))) {
        dir.create(
          dirname(filename),
          recursive = TRUE,
          showWarnings = FALSE
        )
      }
      ggplot2::ggsave(
        plot = p,
        filename = filename,
        width = plot_width,
        height = plot_height,
        units = units,
        dpi = dpi,
        limitsize = FALSE
      )
    }
    return(p)
  }
}

#' @title Get colors from Chinese colors dataset or palettes
#'
#' @description
#' Search for colors in the Chinese colors dataset and all available palettes.
#' This function can search by palette names, color names (pinyin or Chinese), numbers, or hex codes.
#' It automatically searches in all palettes and reports which palette(s) contain the found colors.
#'
#' @md
#' @param ... One or more search values.
#' Can be palette names, color names (pinyin or Chinese), numbers, or hex codes.
#' If NULL, using all Chinese colors.
#' @param palettes Optional. A named list of palettes to search in.
#' If `NULL` (default), searches in all available palettes.
#'
#' @return A data frame with class `colors` containing matching color information.
#' The result is automatically printed using [print.colors()].
#'
#' @export
#'
#' @seealso
#' [chinese_colors] for the dataset of Chinese traditional colors.
#' [get_chinese_palettes] for getting Chinese color palettes.
#' [ChineseColors] for the ChineseColors object.
#'
#' @examples
#' get_colors("Paired")
#'
#' get_colors("#FF7F00")
#'
#' get_colors("pinlan")
#' get_colors(44)
#' get_colors("#2B73AF")
#'
#' get_colors("cyan", palettes = "ChineseSet64")
get_colors <- function(..., palettes = NULL) {
  args <- list(...)

  colors_df <- thisplot::chinese_colors
  available_columns <- colnames(colors_df)

  all_palettes <- c(
    thisplot::palette_list,
    get_chinese_palettes()
  )
  specified_palette_names <- character(0)
  if (!is.null(palettes)) {
    if (is.character(palettes)) {
      all_palettes_list <- c(
        thisplot::palette_list,
        get_chinese_palettes()
      )
      specified_palette_names <- palettes
      all_palettes <- all_palettes_list[names(all_palettes_list) %in% palettes]
      if (length(all_palettes) == 0) {
        log_message(
          "No matching palettes found for: {.val {palettes}}",
          message_type = "error"
        )
      }
    } else {
      log_message(
        "Invalid palettes argument: {.val {palettes}}",
        message_type = "error"
      )
    }
  }

  if (length(args) == 0) {
    if (length(specified_palette_names) > 0) {
      palette_colors_list <- character(0)
      for (pal_name in specified_palette_names) {
        if (pal_name %in% names(all_palettes)) {
          palette_colors_list <- c(palette_colors_list, all_palettes[[pal_name]])
        }
      }
      palette_colors_list <- unique(palette_colors_list)

      if (length(palette_colors_list) > 0) {
        palette_hex_upper <- toupper(palette_colors_list)
        colors_hex_upper <- toupper(colors_df$hex)
        palette_color_indices <- which(colors_hex_upper %in% palette_hex_upper)

        if (length(palette_color_indices) > 0) {
          result <- colors_df[palette_color_indices, , drop = FALSE]
        } else {
          result <- data.frame(
            name = palette_colors_list,
            rgb = hex_to_rgb(palette_colors_list),
            hex = palette_colors_list,
            stringsAsFactors = FALSE
          )
        }
      } else {
        result <- colors_df[integer(0), , drop = FALSE]
      }
    } else {
      result <- colors_df
    }
    class(result) <- c("colors", "data.frame")
    return(result)
  }

  search_values <- unlist(args, use.names = FALSE)
  all_matches <- integer(0)
  found_hex_codes <- character(0)

  palette_names_found <- character(0)
  palette_names_not_found <- character(0)
  palette_colors_found <- character(0)
  remaining_search_values <- character(0)

  looks_like_palette_name <- function(s) {
    if (grepl("^[A-Z][A-Z0-9]*$", s)) {
      return(TRUE)
    }
    if (grepl("^[A-Z][a-zA-Z0-9]*$", s)) {
      return(TRUE)
    }
    return(FALSE)
  }

  for (val in search_values) {
    val_char <- as.character(val)
    if (val_char %in% names(all_palettes)) {
      palette_names_found <- c(palette_names_found, val_char)
      palette_colors_found <- c(palette_colors_found, all_palettes[[val_char]])
    } else {
      is_hex <- grepl("^#[0-9A-Fa-f]{6}$", val_char)
      is_number <- !is.na(suppressWarnings(as.numeric(val_char)))
      if (!is_hex && !is_number && looks_like_palette_name(val_char)) {
        palette_names_not_found <- c(palette_names_not_found, val_char)
      }
      remaining_search_values <- c(remaining_search_values, val_char)
    }
  }

  if (length(palette_names_found) > 0) {
    palette_colors_found <- unique(palette_colors_found)
    if (length(remaining_search_values) == 0) {
      if (length(palette_colors_found) > 0) {
        palette_hex_upper <- toupper(palette_colors_found)
        colors_hex_upper <- toupper(colors_df$hex)
        hex_matches <- which(colors_hex_upper %in% palette_hex_upper)
        if (length(hex_matches) > 0) {
          result <- colors_df[hex_matches, , drop = FALSE]
        } else {
          result <- data.frame(
            name = palette_colors_found,
            rgb = hex_to_rgb(palette_colors_found),
            hex = palette_colors_found,
            stringsAsFactors = FALSE
          )
        }
        class(result) <- c("colors", "data.frame")
        cli::cli_h3("Found palette{?s}: {.val {palette_names_found}}")
        return(result)
      }
    }
    if (length(palette_colors_found) > 0) {
      palette_hex_upper <- toupper(palette_colors_found)
      colors_hex_upper <- toupper(colors_df$hex)
      palette_color_indices <- which(colors_hex_upper %in% palette_hex_upper)
      colors_df <- colors_df[palette_color_indices, , drop = FALSE]
      cli::cli_h3("Searching in palette{?s}: {.val {palette_names_found}}")
    }
  }

  if (length(specified_palette_names) > 0 && length(remaining_search_values) > 0) {
    cli::cli_h3("Searching in palette{?s}: {.val {specified_palette_names}}")
  }

  if (length(remaining_search_values) > 0) {
    search_values <- remaining_search_values
  } else {
    search_values <- character(0)
  }

  for (val in search_values) {
    val_char <- as.character(val)

    for (col in available_columns) {
      if (col == "num") {
        val_num <- suppressWarnings(as.numeric(val_char))
        if (!is.na(val_num)) {
          matches <- which(colors_df[[col]] == val_num)
          all_matches <- c(all_matches, matches)
        }
      } else {
        matches <- which(colors_df[[col]] == val_char)
        all_matches <- c(all_matches, matches)
      }
    }
  }

  idx <- unique(all_matches)

  if (length(search_values) > 1 && length(idx) > 0) {
    ordered_idx <- integer(0)
    for (val in search_values) {
      val_char <- as.character(val)
      for (col in available_columns) {
        if (col == "num") {
          val_num <- suppressWarnings(as.numeric(val_char))
          if (!is.na(val_num)) {
            match_idx <- which(colors_df[[col]] == val_num)
          } else {
            match_idx <- integer(0)
          }
        } else {
          match_idx <- which(colors_df[[col]] == val_char)
        }
        if (length(match_idx) > 0) {
          ordered_idx <- c(ordered_idx, match_idx)
        }
      }
    }
    idx <- unique(ordered_idx)
  }

  palette_matches <- list()
  if (length(search_values) > 0) {
    if (length(idx) > 0) {
      found_hex_codes <- unique(colors_df$hex[idx])
    }

    for (val in search_values) {
      val_char <- as.character(val)
      if (grepl("^#[0-9A-Fa-f]{6}$", val_char)) {
        found_hex_codes <- c(found_hex_codes, toupper(val_char))
      }
    }
    found_hex_codes <- unique(found_hex_codes)

    for (pal_name in names(all_palettes)) {
      pal_colors <- all_palettes[[pal_name]]
      if (is.null(pal_colors) || length(pal_colors) == 0) {
        next
      }

      pal_colors_upper <- toupper(pal_colors)

      for (hex_code in found_hex_codes) {
        hex_upper <- toupper(hex_code)
        matches <- which(pal_colors_upper == hex_upper)
        if (length(matches) > 0) {
          if (!hex_code %in% names(palette_matches)) {
            palette_matches[[hex_code]] <- list()
          }
          palette_matches[[hex_code]][[pal_name]] <- matches
        }
      }
    }
  }

  if (length(palette_matches) > 0) {
    cli::cli_h3("Found in:")
    unique_hex_codes <- unique(names(palette_matches))
    for (hex_code in unique_hex_codes) {
      pal_info <- palette_matches[[hex_code]]
      pal_names <- unique(names(pal_info))
      cli::cli_text("{.strong {hex_code}}: {.val {pal_names}}")
    }

    if (length(idx) == 0) {
      palette_hex_codes <- unique(names(palette_matches))
      result <- data.frame(
        name = palette_hex_codes,
        rgb = hex_to_rgb(palette_hex_codes),
        hex = palette_hex_codes,
        stringsAsFactors = FALSE
      )
      if (length(palette_hex_codes) == 1) {
        result <- result[1, , drop = FALSE]
        result$hex <- palette_hex_codes[1]
      } else {
        result <- result[rep(1, length(palette_hex_codes)), , drop = FALSE]
        result$hex <- palette_hex_codes
      }
      class(result) <- c("colors", "data.frame")
      return(result)
    }
  }

  if (length(idx) == 0 && length(search_values) > 0) {
    palette_like_values <- character(0)
    for (val in search_values) {
      val_char <- as.character(val)
      is_hex <- grepl("^#[0-9A-Fa-f]{6}$", val_char)
      is_number <- !is.na(suppressWarnings(as.numeric(val_char)))
      if (!is_hex && !is_number && looks_like_palette_name(val_char)) {
        palette_like_values <- c(palette_like_values, val_char)
      }
    }

    if (length(palette_like_values) > 0 &&
      length(palette_like_values) == length(search_values)) {
      available_palette_names <- names(all_palettes)
      suggestions <- character(0)
      for (not_found in palette_like_values) {
        prefix_matches <- available_palette_names[grepl(
          paste0("^", not_found), available_palette_names,
          ignore.case = TRUE
        )]
        if (length(prefix_matches) > 0) {
          suggestions <- c(
            suggestions, prefix_matches[1:min(3, length(prefix_matches))]
          )
        }
      }
      suggestions <- unique(suggestions)

      if (length(suggestions) > 0) {
        log_message(
          "No matching palettes found for: {.val {palette_like_values}}. ",
          "Did you mean: {.val {suggestions}}?",
          message_type = "error"
        )
      } else {
        log_message(
          "No matching palettes found for: {.val {palette_like_values}}. ",
          "Available palettes include: {.val {available_palette_names}}. ",
          "Use {.code show_palettes(return_palettes = TRUE)} to see all available palettes.",
          message_type = "error"
        )
      }
    } else {
      log_message(
        "No matching color{?s} found for: {.val {search_values}}",
        message_type = "error"
      )
    }
  }

  if (length(idx) > 0) {
    result <- colors_df[idx, , drop = FALSE]

    if (length(specified_palette_names) > 0) {
      if (length(palette_matches) > 0) {
        palette_hex_codes <- unique(names(palette_matches))
        result_hex_upper <- toupper(result$hex)
        palette_hex_upper <- toupper(palette_hex_codes)
        keep_indices <- which(result_hex_upper %in% palette_hex_upper)
        if (length(keep_indices) > 0) {
          result <- result[keep_indices, , drop = FALSE]
        } else {
          result <- result[integer(0), , drop = FALSE]
        }
      } else {
        result <- result[integer(0), , drop = FALSE]
      }
    }
  } else {
    result <- colors_df[integer(0), , drop = FALSE]
  }

  class(result) <- c("colors", "data.frame")
  return(result)
}

#' @title Print colors object
#'
#' @md
#' @param x A colors object (data frame with color information).
#' @param ... Additional arguments passed to print.
#'
#' @return
#' Details of the colors objec.
#'
#' @method print colors
#' @export
print.colors <- function(x, ...) {
  has_color <- cli::num_ansi_colors() > 1 && "hex" %in% colnames(x)

  display_width <- function(s) {
    if (is.na(s) || length(s) == 0) {
      return(0)
    }
    plain_s <- cli::ansi_strip(s)
    nchar(plain_s, type = "width")
  }

  if (has_color) {
    col_names <- colnames(x)

    col_widths <- vapply(col_names, function(nm) {
      name_width <- display_width(nm)
      data_widths <- vapply(x[[nm]], function(val) {
        if (is.na(val)) {
          return(2)
        }
        display_width(format(val))
      }, numeric(1))
      max(name_width, max(data_widths, na.rm = TRUE), na.rm = TRUE)
    }, numeric(1))

    header_parts <- vapply(seq_along(col_names), function(i) {
      nm <- col_names[i]
      width <- as.integer(col_widths[i])
      current_width <- display_width(nm)
      padding <- max(0, width - current_width)
      paste0(nm, strrep(" ", padding + 2))
    }, character(1))
    cat(paste(header_parts, collapse = ""), "\n")

    for (i in seq_len(nrow(x))) {
      hex_val <- x$hex[i]

      if (!is.na(hex_val) && nchar(hex_val) > 0) {
        row_style <- cli::make_ansi_style(hex_val)
      } else {
        row_style <- function(x) x
      }

      row_parts <- vapply(
        seq_along(col_names), function(j) {
          col_name <- col_names[j]
          val <- x[[col_name]][i]
          formatted_val <- format(val)

          width <- as.integer(col_widths[j])
          current_width <- display_width(formatted_val)
          padding <- max(0, width - current_width)

          cell_content <- paste0(formatted_val, strrep(" ", padding + 2))
          return(cell_content)
        }, character(1)
      )

      row_text <- paste(row_parts, collapse = "")
      styled_row <- row_style(row_text)
      cat(styled_row, "\n")
    }
  } else {
    print.data.frame(x, ...)
  }

  invisible(x)
}

#' @title Return the first part of a colors object
#'
#' @description
#' Returns the first part of a colors object, similar to [head()] for data frames.
#'
#' @md
#' @param x A colors object (data frame with color information).
#' @param n Number of rows to return. Default is `6`.
#' @param ... Additional arguments passed to [head()].
#'
#' @return
#' A colors object with the first `n` rows.
#'
#' @method head colors
#' @importFrom utils head
#'
#' @export
#' @examples
#' head(get_colors())
#'
#' head(get_colors(), n = 10)
head.colors <- function(x, n = 6L, ...) {
  if (nrow(x) == 0) {
    return(x)
  }
  result <- x[seq_len(min(n, nrow(x))), , drop = FALSE]
  class(result) <- c("colors", "data.frame")
  return(result)
}

hex_to_rgb <- function(hex_colors) {
  vapply(hex_colors, function(hex) {
    rgb_vals <- grDevices::col2rgb(hex)
    paste0(
      "(", rgb_vals[1], ", ", rgb_vals[2], ", ", rgb_vals[3], ")"
    )
  }, character(1), USE.NAMES = FALSE)
}

#' @import ggplot2 thisutils
NULL

utils::globalVariables(
  c(
    ".",
    ".data",
    "..r",
    "count",
    "Freq",
    "group",
    "groupn",
    "id",
    "intersection",
    "label",
    "name",
    "next_node",
    ".plot_group",
    ".plot_label",
    ".plot_shape",
    ".plot_split",
    ".plot_x",
    ".plot_y",
    "next_x",
    "node",
    "value",
    "X",
    "Y",
    "x",
    "y"
  )
)

#' @title Convert color names to hexadecimal format
#'
#' @description
#' Convert color names to hexadecimal RGB color codes.
#'
#' @md
#' @param cname A character vector of color names.
#'
#' @return A character vector of hexadecimal color codes.
#'
#' @export
col2hex <- function(cname) {
  col_mat <- grDevices::col2rgb(cname)
  grDevices::rgb(
    red = col_mat[1, ] / 255,
    green = col_mat[2, ] / 255,
    blue = col_mat[3, ] / 255
  )
}

#' @title Convert a color with specified alpha level
#'
#' @md
#' @param colors Color vectors.
#' @param alpha Alpha level in `[0,1]`.
#'
#' @return A character vector of hexadecimal color codes with the specified alpha level.
#'
#' @export
#'
#' @examples
#' colors <- c("red", "blue", "green")
#' adjcolors(colors, 0.5)
#' ggplot2::alpha(colors, 0.5)
#'
#' show_palettes(
#'   list(
#'     "raw" = colors,
#'     "adjcolors" = adjcolors(colors, 0.5),
#'     "ggplot2::alpha" = ggplot2::alpha(colors, 0.5)
#'   )
#' )
adjcolors <- function(colors, alpha) {
  color_df <- as.data.frame(
    grDevices::col2rgb(colors) / 255
  )
  colors_out <- sapply(
    color_df, function(color) {
      color_rgb <- RGBA2RGB(list(color, alpha))
      grDevices::rgb(color_rgb[1], color_rgb[2], color_rgb[3])
    }
  )
  colors_out
}

#' @title Blends a list of colors using the specified blend mode
#'
#' @md
#' @param colors Color vectors.
#' @param mode Blend mode.
#' One of `"blend"`, `"average"`, `"screen"`, or `"multiply"`.
#'
#' @return A character vector of hexadecimal color codes representing the blended color.
#'
#' @export
#'
#' @examples
#' blend <- c(
#'   "red",
#'   "green",
#'   blendcolors(c("red", "green"),
#'     mode = "blend"
#'   )
#' )
#' average <- c(
#'   "red",
#'   "green",
#'   blendcolors(c("red", "green"),
#'     mode = "average"
#'   )
#' )
#' screen <- c(
#'   "red",
#'   "green",
#'   blendcolors(c("red", "green"),
#'     mode = "screen"
#'   )
#' )
#' multiply <- c(
#'   "red",
#'   "green",
#'   blendcolors(c("red", "green"),
#'     mode = "multiply"
#'   )
#' )
#' show_palettes(
#'   list(
#'     "blend" = blend,
#'     "average" = average,
#'     "screen" = screen,
#'     "multiply" = multiply
#'   )
#' )
blendcolors <- function(
  colors,
  mode = c("blend", "average", "screen", "multiply")
) {
  mode <- match.arg(mode)
  colors <- colors[!is.na(colors)]
  if (length(colors) == 0) {
    return(NA)
  }
  if (length(colors) == 1) {
    return(colors)
  }
  rgb <- as.list(
    as.data.frame(
      grDevices::col2rgb(colors) / 255
    )
  )
  Clist <- lapply(
    rgb, function(x) {
      list(x, 1)
    }
  )
  blend_color <- BlendRGBList(Clist, mode = mode)
  blend_color <- grDevices::rgb(
    blend_color[1], blend_color[2], blend_color[3]
  )
  return(blend_color)
}

#' @title Convert RGBA color to RGB with background
#'
#' @description
#' Convert an RGBA (Red, Green, Blue, Alpha) color to RGB by compositing
#' it with a background color based on the alpha channel.
#'
#' @md
#' @param RGBA A list containing RGB values and alpha channel.
#' @param BackGround The background RGB color to composite with.
#' Default is `c(1, 1, 1)` (white).
#'
#' @return A numeric vector of RGB values.
#'
#' @export
RGBA2RGB <- function(RGBA, BackGround = c(1, 1, 1)) {
  A <- RGBA[[length(RGBA)]]
  RGB <- RGBA[[-length(RGBA)]] * A + BackGround * (1 - A)
  return(RGB)
}

#' @title Blend two colors using a specified mode
#'
#' @description
#' Blend two colors with alpha channels using one of several blending modes:
#' blend, average, screen, or multiply.
#'
#' @md
#' @param C1 A list containing the first color RGB values and alpha channel.
#' @param C2 A list containing the second color RGB values and alpha channel.
#' @param mode The blending mode to use.
#' One of `"blend"`, `"average"`, `"screen"`, or `"multiply"`.
#' Default is `"blend"`.
#'
#' @return A list containing the blended RGB values and alpha channel.
#'
#' @export
Blend2Color <- function(C1, C2, mode = "blend") {
  c1 <- C1[[1]]
  c1a <- C1[[2]]
  c2 <- C2[[1]]
  c2a <- C2[[2]]
  A <- 1 - (1 - c1a) * (1 - c2a)
  if (A < 1.0e-6) {
    return(list(c(0, 0, 0), 1))
  }
  if (mode == "blend") {
    out <- (c1 * c1a + c2 * c2a * (1 - c1a)) / A
    A <- 1
  }
  if (mode == "average") {
    out <- (c1 + c2) / 2
    out[out > 1] <- 1
  }
  if (mode == "screen") {
    out <- 1 - (1 - c1) * (1 - c2)
  }
  if (mode == "multiply") {
    out <- c1 * c2
  }
  return(list(out, A))
}

#' @title Blend a list of colors
#'
#' @description
#' Blend multiple colors with alpha channels into a single color using a specified blending mode.
#'
#' @md
#' @param Clist A list of colors, where each color is a list containing
#' RGB values and alpha channel.
#' @param mode The blending mode to use.
#' One of `"blend"`, `"average"`, `"screen"`, or `"multiply"`.
#' Default is `"blend"`.
#' @param RGB_BackGround The background RGB color to composite with.
#' Default is `c(1, 1, 1)` (white).
#'
#' @return A numeric vector of RGB values.
#'
#' @export
BlendRGBList <- function(
  Clist,
  mode = "blend",
  RGB_BackGround = c(1, 1, 1)
) {
  n <- length(Clist)
  clist_use <- Clist
  while (n != 1) {
    temp <- clist_use
    clist_use <- list()
    for (C in temp[1:(length(temp) - 1)]) {
      c1 <- C[[1]]
      a1 <- C[[2]]
      c2 <- temp[[length(temp)]][[1]]
      a2 <- temp[[length(temp)]][[2]]
      clist_use <- append(
        clist_use,
        list(
          Blend2Color(
            C1 = list(c1, a1 * (1 - 1 / n)),
            C2 = list(c2, a2 * 1 / n),
            mode = mode
          )
        )
      )
    }
    n <- length(clist_use)
  }
  result <- list(clist_use[[1]][[1]], clist_use[[1]][[2]])
  result <- RGBA2RGB(result, BackGround = RGB_BackGround)
  return(result)
}

#' @title Simple random color selection
#'
#' @md
#' @param n The number of colors to return. Default is `10`.
#' @param palette The name of the palette to use.
#' Default is `NULL`, colors will be selected from ChineseColors.
#' Otherwise, colors will be selected from the specified palette.
#' Available palette names can be queried with [show_palettes].
#'
#' @return A character vector of hexadecimal color codes.
#'
#' @export
#'
#' @examples
#' simple_colors()
#'
#' show_palettes(simple_colors(n = 5))
#'
#' # Get colors from a specific palette
#' simple_colors(n = 10, palette = "Paired")
#' simple_colors(n = 10, palette = "ChineseBlue")
#' simple_colors(n = 10, palette = "Spectral")
simple_colors <- function(n = 10, palette = NULL) {
  if (!is.numeric(n) || n < 1) {
    log_message(
      "{.arg n} must be a positive integer",
      message_type = "error"
    )
  }
  n <- as.integer(n)

  if (is.null(palette)) {
    cc_obj <- tryCatch(
      ChineseColors(),
      error = function(e) {
        log_message(
          "Failed to initialize {.fn ChineseColors}: {.val {conditionMessage(e)}}",
          message_type = "warning"
        )
        NULL
      }
    )

    if (is.null(cc_obj)) {
      log_message(
        "Cannot access ChineseColors. Please specify a palette name",
        message_type = "error"
      )
    }

    all_colors <- cc_obj$colors$hex
    if (length(all_colors) == 0) {
      log_message(
        "No colors available in ChineseColors.",
        message_type = "error"
      )
    }

    if (n > length(all_colors)) {
      log_message(
        "Requested {.val {n}} colors but only {.val {length(all_colors)}} available. Returning all colors",
        message_type = "warning"
      )
      return(sample(all_colors, length(all_colors)))
    }

    return(sample(all_colors, n))
  } else {
    palette_list <- c(
      thisplot::palette_list,
      get_chinese_palettes()
    )

    if (!palette %in% names(palette_list)) {
      log_message(
        "The palette {.val {palette}} is invalid.\n",
        "Check the available palette names with {.fn show_palettes}",
        message_type = "error"
      )
    }

    palette_colors <- palette_list[[palette]]
    if (length(palette_colors) == 0) {
      log_message(
        "The palette {.val {palette}} is empty.",
        message_type = "error"
      )
    }

    if (n > length(palette_colors)) {
      log_message(
        "Requested {.val {n}} colors but only {.val {length(palette_colors)}} available in palette {.val {palette}}. Returning all colors",
        message_type = "warning"
      )
      return(sample(palette_colors, length(palette_colors)))
    }

    return(sample(palette_colors, n))
  }
}

get_namespace_fun <- thisutils::get_namespace_fun

#' @title Get used vars in a ggplot object
#'
#' @md
#' @inheritParams thisutils::log_message
#' @param p A `ggplot` object.
#' @param reverse Whether to return unused vars.
#' Default is `FALSE`.
#'
#' @return A character vector of variable names.
#' If `reverse` is `FALSE`, returns used variables;
#' if `TRUE`, returns unused variables.
#'
#' @export
#' @examples
#' library(ggplot2)
#' p <- ggplot(
#'   data = mtcars,
#'   aes(x = mpg, y = wt, colour = cyl)
#' ) +
#'   geom_point()
#' get_vars(p)
#' get_vars(p, reverse = TRUE)
get_vars <- function(
  p,
  reverse = FALSE,
  verbose = TRUE
) {
  mappings <- c(
    as.character(p$mapping),
    unlist(
      lapply(p$layers, function(x) as.character(x$mapping))
    ),
    unlist(
      lapply(p$layers, function(x) names(p$layers[[1]]$aes_params))
    ),
    names(
      p$facet$params$facets
    ), names(p$facet$params$rows), names(p$facet$params$cols)
  )
  vars <- unique(
    unlist(
      strsplit(
        gsub(
          "[~\\[\\]\\\"\\(\\)]", " ", unique(mappings),
          perl = TRUE
        ), " "
      )
    )
  )
  vars_used <- intersect(
    unique(
      c(
        colnames(p$data), unlist(lapply(p$layers, function(x) colnames(x$data)))
      )
    ), vars
  )

  vars_notused <- setdiff(colnames(p$data), vars_used)
  log_message(
    "vars_used: {.val {vars_used}}\n",
    "vars_notused: {.val {vars_notused}}",
    verbose = verbose
  )
  if (reverse) {
    return(vars_notused)
  } else {
    return(vars_used)
  }
}

#' @title Build a patchwork gtable
#'
#' @description
#' Build a gtable from a patchwork object by arranging multiple plots
#' according to the layout specification.
#'
#' @md
#' @param x A patchwork object.
#' @param guides How to handle guides.
#' Default is `"auto"`.
#' @param table_rows The number of rows in the table grid.
#' Default is `18`.
#' @param table_cols The number of columns in the table grid.
#' Default is `15`.
#' @param panel_row The row index for panels.
#' Default is `10`.
#' @param panel_col The column index for panels.
#' Default is `8`.
#'
#' @return A gtable object.
#'
#' @export
build_patchwork <- function(
  x,
  guides = "auto",
  table_rows = 18,
  table_cols = 15,
  panel_row = 10,
  panel_col = 8
) {
  x$layout <- utils::modifyList(
    get_namespace_fun("patchwork", "default_layout"),
    x$layout[!vapply(x$layout, is.null, logical(1))]
  )

  if (guides == "collect" && x$layout$guides != "keep") {
    guides <- "collect"
  } else {
    guides <- x$layout$guides
  }
  plot_table_fun <- get_namespace_fun(
    "patchwork", "plot_table"
  )
  gt <- lapply(
    x$plots,
    plot_table_fun,
    guides = guides
  )
  fixed_asp <- vapply(
    gt, function(x) isTRUE(x$respect), logical(1)
  )
  guide_grobs <- unlist(
    lapply(gt, `[[`, "collected_guides"),
    recursive = FALSE
  )
  simplify_gt_fun <- get_namespace_fun(
    "patchwork", "simplify_gt"
  )
  gt <- lapply(
    gt,
    simplify_gt_fun
  )
  add_insets_fun <- get_namespace_fun(
    "patchwork", "add_insets"
  )
  gt <- add_insets_fun(gt)
  if (is.null(x$layout$design)) {
    if (is.null(x$layout$ncol) && !is.null(x$layout$widths) && length(x$layout$widths) > 1) {
      x$layout$ncol <- length(x$layout$widths)
    }
    if (is.null(x$layout$nrow) && !is.null(x$layout$heights) && length(x$layout$heights) > 1) {
      x$layout$nrow <- length(x$layout$heights)
    }
    dims <- ggplot2::wrap_dims(
      length(gt),
      nrow = x$layout$nrow,
      ncol = x$layout$ncol
    )
    create_design_fun <- get_namespace_fun(
      "patchwork", "create_design"
    )
    x$layout$design <- create_design_fun(
      dims[2],
      dims[1],
      x$layout$byrow
    )
  } else {
    dims <- c(
      max(x$layout$design$b),
      max(x$layout$design$r)
    )
  }

  gt_new <- gtable::gtable(
    grid::unit(rep(0, table_cols * dims[2]), "null"),
    grid::unit(rep(0, table_rows * dims[1]), "null")
  )
  design <- as.data.frame(unclass(x$layout$design))
  if (nrow(design) < length(gt)) {
    log_message(
      "Too few patch areas to hold all plots. Dropping plots",
      message_type = "warning"
    )
    gt <- gt[seq_len(nrow(design))]
    fixed_asp <- fixed_asp[seq_len(nrow(design))]
  } else {
    design <- design[seq_along(gt), ]
  }
  if (any(design$t < 1)) design$t[design$t < 1] <- 1
  if (any(design$l < 1)) design$l[design$l < 1] <- 1
  if (any(design$b > dims[1])) design$b[design$b > dims[1]] <- dims[1]
  if (any(design$r > dims[2])) design$r[design$r > dims[2]] <- dims[2]
  max_z <- lapply(gt, function(x) max(x$layout$z))
  max_z <- c(0, cumsum(max_z))
  gt_new$layout <- do.call(
    rbind,
    lapply(
      seq_along(gt), function(i) {
        loc <- design[i, ]
        lay <- gt[[i]]$layout
        lay$name <- paste0(lay$name, "-", i)
        lay$t <- lay$t +
          ifelse(
            lay$t <= panel_row, (loc$t - 1) * table_rows,
            (loc$b - 1) * table_rows
          )
        lay$l <- lay$l +
          ifelse(
            lay$l <= panel_col,
            (loc$l - 1) * table_cols,
            (loc$r - 1) * table_cols
          )
        lay$b <- lay$b +
          ifelse(lay$b < panel_row,
            (loc$t - 1) * table_rows,
            (loc$b - 1) * table_rows
          )
        lay$r <- lay$r +
          ifelse(lay$r < panel_col,
            (loc$l - 1) * table_cols,
            (loc$r - 1) * table_cols
          )
        lay$z <- lay$z + max_z[i]
        lay
      }
    )
  )
  table_dims_fun <- get_namespace_fun("patchwork", "table_dims")
  table_dimensions <- table_dims_fun(
    lapply(gt, `[[`, "widths"),
    lapply(gt, `[[`, "heights"),
    design,
    dims[2],
    dims[1]
  )
  set_grob_sizes_fun <- get_namespace_fun("patchwork", "set_grob_sizes")
  gt_new$grobs <- set_grob_sizes_fun(
    gt,
    table_dimensions$widths,
    table_dimensions$heights, design
  )
  gt_new$widths <- table_dimensions$widths
  gt_new$heights <- table_dimensions$heights
  widths <- rep(x$layout$widths, length.out = dims[2])
  heights <- rep(x$layout$heights, length.out = dims[1])
  set_panel_dimensions_fun <- get_namespace_fun("patchwork", "set_panel_dimensions")
  gt_new <- set_panel_dimensions_fun(
    gt_new,
    gt,
    widths,
    heights,
    fixed_asp,
    design
  )
  if (x$layout$guides == "collect") {
    collapse_guides_fun <- get_namespace_fun("patchwork", "collapse_guides")
    guide_grobs <- collapse_guides_fun(guide_grobs)
    if (length(guide_grobs) != 0) {
      theme <- x$annotation$theme
      if (!attr(theme, "complete")) {
        theme <- ggplot2::theme_get() + theme
      }
      assemble_guides_fun <- get_namespace_fun(
        "patchwork", "assemble_guides"
      )
      guide_grobs <- assemble_guides_fun(guide_grobs, theme)
      attach_guides_fun <- get_namespace_fun(
        "patchwork", "attach_guides"
      )
      gt_new <- attach_guides_fun(gt_new, guide_grobs, theme)
    }
  } else {
    gt_new$collected_guides <- guide_grobs
  }

  class(gt_new) <- c("gtable_patchwork", class(gt_new))
  gt_new
}

#' @title Convert a patchwork object to a grob
#'
#' @description
#' Convert a patchwork object to a gtable grob by processing annotations
#' and building the patchwork layout.
#'
#' @md
#' @param x A patchwork object.
#' @param ... Additional arguments passed to other functions.
#'
#' @return A gtable object.
#'
#' @export
patchwork_grob <- function(x, ...) {
  annotation <- utils::modifyList(
    get_namespace_fun("patchwork", "default_annotation"),
    x$patches$annotation[!vapply(x$patches$annotation, is.null, logical(1))]
  )
  recurse_tags_fun <- get_namespace_fun("patchwork", "recurse_tags")
  x <- recurse_tags_fun(
    x,
    annotation$tag_levels,
    annotation$tag_prefix,
    annotation$tag_suffix,
    annotation$tag_sep
  )$patches
  get_patches_fun <- get_namespace_fun("patchwork", "get_patches")
  plot <- get_patches_fun(x)
  gtable <- build_patchwork(plot)
  annotate_table_fun <- get_namespace_fun("patchwork", "annotate_table")
  gtable <- annotate_table_fun(gtable, annotation)
  class(gtable) <- setdiff(class(gtable), "gtable_patchwork")
  gtable
}

#' @title Convert a plot object to a grob
#'
#' @description
#' Convert various plot objects (gList, patchwork, ggplot) to a grob object.
#'
#' @md
#' @param plot A plot object (gList, patchwork, or ggplot).
#' @param ... Additional arguments passed to other functions.
#'
#' @return A grob object.
#'
#' @export
as_grob <- function(plot, ...) {
  if (inherits(plot, "gList")) {
    grid::grobTree(plot)
  } else if (inherits(plot, "patchwork")) {
    patchwork_grob(plot, ...)
  } else if (inherits(plot, "ggplot")) {
    ggplot2::ggplotGrob(plot)
  } else {
    log_message(
      "Cannot convert object of {.cls {class(plot)}} into a grob",
      message_type = "warning"
    )
  }
}

#' @title Convert a plot object to a gtable
#'
#' @description
#' Convert various plot objects (gtable, grob, patchwork, ggplot) to a gtable object.
#'
#' @md
#' @param plot A plot object (gtable, grob, patchwork, or ggplot).
#' @param ... Additional arguments passed to other functions.
#'
#' @return A gtable object.
#'
#' @export
as_gtable <- function(plot, ...) {
  if (inherits(plot, "gtable")) {
    return(plot)
  }
  if (inherits(plot, "grob")) {
    u <- grid::unit(1, "null")
    gt <- gtable::gtable_col(NULL, list(plot), u, u)
    gt$layout$clip <- "inherit"
    return(gt)
  } else {
    grob <- as_grob(plot, ...)
    if (inherits(grob, "gtable")) {
      return(grob)
    } else if (inherits(grob, "grob")) {
      u <- grid::unit(1, "null")
      gt <- gtable::gtable_col(NULL, list(grob), u, u)
      gt$layout$clip <- "inherit"
      return(gt)
    } else {
      log_message(
        "Cannot convert object of {.cls {class(grob)}} into a gtable",
        message_type = "warning"
      )
      return(NULL)
    }
  }
}

#' @title Extract legend from a plot
#'
#' @description
#' Extract the legend grob from a plot object.
#'
#' @md
#' @param plot A plot object.
#'
#' @return The legend grob.
#'
#' @export
get_legend <- function(plot) {
  plot <- as_gtable(plot)
  grob_names <- plot$layout$name
  grobs <- plot$grobs
  grob_index <- which(
    grepl(
      "guide-box-bottom",
      grob_names
    )
  )
  grob_index <- grob_index[1]
  matched_grobs <- grobs[[grob_index]]
  matched_grobs
}

#' @title Add a grob to a gtable
#'
#' @description
#' Add a grob to a gtable at a specified position (top, bottom, left, or right).
#'
#' @md
#' @param gtable A gtable object.
#' @param grob A grob or gtable object to add.
#' @param position The position to add the grob.
#' One of `"top"`, `"bottom"`, `"left"`, `"right"`, or `"none"`.
#' @param space The space to allocate for the grob.
#' If `NULL`, will be calculated automatically.
#' @param clip The clipping mode.
#' Default is `"on"`.
#'
#' @return A gtable object with the grob added.
#'
#' @export
add_grob <- function(
  gtable,
  grob,
  position = c("top", "bottom", "left", "right", "none"),
  space = NULL,
  clip = "on"
) {
  position <- match.arg(position)
  if (position == "none" || is.null(grob)) {
    return(gtable)
  }

  if (is.null(space)) {
    if (gtable::is.gtable(grob)) {
      if (position %in% c("top", "bottom")) {
        space <- sum(grob$heights)
      } else {
        space <- sum(grob$widths)
      }
    } else if (grid::is.grob(grob)) {
      if (position %in% c("top", "bottom")) {
        space <- grid::grobHeight(grob)
      } else {
        space <- grid::grobWidth(grob)
      }
    }
  }

  if (position == "top") {
    gtable <- gtable::gtable_add_rows(
      gtable,
      space,
      0
    )
    gtable <- gtable::gtable_add_grob(
      gtable, grob,
      t = 1,
      l = mean(
        gtable$layout[grepl(pattern = "panel", x = gtable$layout$name), "l"]
      ),
      clip = clip
    )
  }
  if (position == "bottom") {
    gtable <- gtable::gtable_add_rows(
      gtable,
      space,
      -1
    )
    gtable <- gtable::gtable_add_grob(
      gtable, grob,
      t = dim(gtable)[1],
      l = mean(
        gtable$layout[grepl(pattern = "panel", x = gtable$layout$name), "l"]
      ), clip = clip
    )
  }
  if (position == "left") {
    gtable <- gtable::gtable_add_cols(
      gtable,
      space,
      0
    )
    gtable <- gtable::gtable_add_grob(
      gtable,
      grob,
      t = mean(
        gtable$layout[grep("panel", gtable$layout$name), "t"]
      ),
      l = 1,
      clip = clip
    )
  }
  if (position == "right") {
    gtable <- gtable::gtable_add_cols(
      gtable,
      space,
      -1
    )
    gtable <- gtable::gtable_add_grob(
      gtable,
      grob,
      t = mean(
        gtable$layout[grep("panel", gtable$layout$name), "t"]
      ),
      l = dim(gtable)[2],
      clip = clip
    )
  }
  return(gtable)
}
#' @title Normalize values to a unit sum
#'
#' @param x A numeric vector.
#'
#' @return A numeric vector scaled to sum to 1.
#' @keywords internal
normalize_to_sum <- function(x) {
  if (sum(x) == 0) 0 * x else x / sum(x)
}

#' @title Convert proportions to percentages
#'
#' @param x A numeric vector.
#'
#' @return A numeric vector expressed as percentages.
#'
#' @export
#' @examples
#' to_percent(c(1, 1, 2))
#' to_percent(c(0, 0))
to_percent <- function(x) {
  100 * round(normalize_to_sum(x), 3)
}

#' @title Compute fixed heatmap device size
#'
#' @md
#' @param width Optional user-provided target width. If `NULL`, width is
#' estimated from the drawn heatmap layout.
#' @param width_sum Numeric baseline width used when heatmap body size is in
#' `npc` units.
#' @param height Optional user-provided target height. If `NULL`, height is
#' estimated from the drawn heatmap layout.
#' @param height_sum Numeric baseline height used when heatmap body size is in
#' `npc` units.
#' @param units Output unit string passed to grid conversion helpers, such as
#' `"in"`, `"cm"`, or `"mm"`.
#' @param ht_list A `ComplexHeatmap::Heatmap` or
#' `ComplexHeatmap::HeatmapList` object.
#' @param legend_list A list of `ComplexHeatmap::Legend` objects.
#'
#' @return A list with two converted units: `ht_width` and `ht_height`.
#'
#' @export
#'
#' @examples
#' mat <- matrix(rnorm(100), nrow = 10)
#' ht <- ComplexHeatmap::Heatmap(mat, name = "expr")
#' lgd <- list(ComplexHeatmap::Legend(title = "expr", at = c(-2, 0, 2)))
#' out <- heatmap_fixsize(
#'   width = NULL,
#'   width_sum = 6,
#'   height = NULL,
#'   height_sum = 4,
#'   units = "in",
#'   ht_list = ht,
#'   legend_list = lgd
#' )
#' out
heatmap_fixsize <- function(
  width,
  width_sum,
  height,
  height_sum,
  units,
  ht_list,
  legend_list
) {
  unit_value_or_zero <- function(u, units, type = c("width", "height")) {
    type <- match.arg(type)
    if (length(u) == 0) {
      return(0)
    }
    if (type == "width") {
      return(grid::convertWidth(u, units, valueOnly = TRUE))
    }
    grid::convertHeight(u, units, valueOnly = TRUE)
  }

  ht <- ComplexHeatmap::draw(
    ht_list,
    annotation_legend_list = legend_list
  )
  ht_width <- ht@ht_list_param$width
  ht_height <- ht@ht_list_param$height

  if (grid::unitType(ht_width) == "npc") {
    ht_width <- grid::unit(width_sum, units = units)
  }
  if (grid::unitType(ht_height) == "npc") {
    ht_height <- grid::unit(height_sum, units = units)
  }
  if (is.null(width)) {
    legend_size <- ht@annotation_legend_param$size
    ht_width <- max(
      grid::convertWidth(
        ht@layout$max_left_component_width,
        units,
        valueOnly = TRUE
      ) +
        grid::convertWidth(
          ht@layout$max_right_component_width,
          units,
          valueOnly = TRUE
        ) +
        grid::convertWidth(
          sum(ht@layout$max_title_component_width),
          units,
          valueOnly = TRUE
        ) +
        grid::convertWidth(
          grid::unit(
            unit_value_or_zero(
              if (length(legend_size) >= 1) legend_size[1] else legend_size,
              units,
              type = "width"
            ),
            units
          ),
          units,
          valueOnly = TRUE
        ) +
        grid::convertWidth(
          grid::unit(1, "in"),
          units,
          valueOnly = TRUE
        ),
      grid::convertWidth(
        grid::unit(0.95, "npc"),
        units,
        valueOnly = TRUE
      )
    )
    ht_width <- grid::unit(ht_width, units)
  }
  if (is.null(height)) {
    legend_size <- ht@annotation_legend_param$size
    ht_height <- max(
      grid::convertHeight(
        ht@layout$max_top_component_height,
        units,
        valueOnly = TRUE
      ) +
        grid::convertHeight(
          ht@layout$max_bottom_component_height,
          units,
          valueOnly = TRUE
        ) +
        grid::convertHeight(
          sum(ht@layout$max_title_component_height),
          units,
          valueOnly = TRUE
        ) +
        grid::convertHeight(
          grid::unit(1, "in"),
          units,
          valueOnly = TRUE
        ),
      unit_value_or_zero(
        if (length(legend_size) >= 2) legend_size[2] else legend_size,
        units,
        type = "height"
      ),
      grid::convertHeight(
        grid::unit(0.95, "npc"),
        units,
        valueOnly = TRUE
      )
    )
    ht_height <- grid::unit(ht_height, units)
  }
  ht_width <- grid::convertUnit(ht_width, unitTo = units)
  ht_height <- grid::convertUnit(ht_height, unitTo = units)

  list(
    ht_width = ht_width,
    ht_height = ht_height
  )
}

#' @title Estimate heatmap render size
#'
#' @md
#' @param width Numeric vector of heatmap body widths.
#' Interpretation depends on `flip`.
#' @param height Numeric vector of heatmap body heights.
#' Interpretation depends on `flip`.
#' @param units Unit string for numeric-to-grid conversion (for example, `"in"`, `"cm"`, `"mm"`).
#' @param ha_top_list A list of top annotations (typically `ComplexHeatmap::HeatmapAnnotation` objects).
#' @param ha_left Optional left annotation.
#' @param ha_right Optional right annotation.
#' @param ht_list A `ComplexHeatmap::Heatmap` or
#' `ComplexHeatmap::HeatmapList` object.
#' @param legend_list A list containing legend objects; `NULL` entries are ignored.
#' @param flip Logical; when `TRUE`, width/height interpretation follows the flipped layout branch.
#'
#' @return A list with numeric `width_sum` and `height_sum` in `units`.
#'
#' @export
#'
#' @examples
#' mat <- matrix(rnorm(100), nrow = 10)
#' ht <- ComplexHeatmap::Heatmap(mat, name = "expr")
#' size <- heatmap_rendersize(
#'   width = c(4),
#'   height = c(3),
#'   units = "in",
#'   ha_top_list = list(),
#'   ha_left = NULL,
#'   ha_right = NULL,
#'   ht_list = ht,
#'   legend_list = list(),
#'   flip = FALSE
#' )
#' size
heatmap_rendersize <- function(
  width,
  height,
  units,
  ha_top_list,
  ha_left,
  ha_right,
  ht_list,
  legend_list,
  flip
) {
  legend_list <- legend_list[!vapply(legend_list, is.null, logical(1))]

  width_annotation <- height_annotation <- 0
  if (isTRUE(flip)) {
    width_sum <- width[1] %||%
      grid::convertWidth(
        grid::unit(1, "in"),
        units,
        valueOnly = TRUE
      )
    height_sum <- sum(
      height %||% grid::convertHeight(
        grid::unit(1, "in"),
        units,
        valueOnly = TRUE
      )
    )
    if (length(ha_top_list) > 0) {
      width_annotation <- grid::convertWidth(
        grid::unit(width_annotation, units) +
          ComplexHeatmap::width.HeatmapAnnotation(ha_top_list[[1]]),
        units,
        valueOnly = TRUE
      )
    }
    if (!is.null(ha_left)) {
      height_annotation <- grid::convertHeight(
        grid::unit(height_annotation, units) +
          ComplexHeatmap::height.HeatmapAnnotation(ha_left),
        units,
        valueOnly = TRUE
      )
    }
    if (!is.null(ha_right)) {
      height_annotation <- grid::convertHeight(
        grid::unit(height_annotation, units) +
          ComplexHeatmap::height.HeatmapAnnotation(ha_right),
        units,
        valueOnly = TRUE
      )
    }
  } else {
    width_sum <- sum(
      width %||% grid::convertWidth(
        grid::unit(1, "in"),
        units,
        valueOnly = TRUE
      )
    )
    height_sum <- height[1] %||%
      grid::convertHeight(grid::unit(1, "in"), units, valueOnly = TRUE)
    if (length(ha_top_list) > 0) {
      height_annotation <- grid::convertHeight(
        grid::unit(height_annotation, units) +
          ComplexHeatmap::height.HeatmapAnnotation(ha_top_list[[1]]),
        units,
        valueOnly = TRUE
      )
    }
    if (!is.null(ha_left)) {
      width_annotation <- grid::convertWidth(
        grid::unit(width_annotation, units) +
          ComplexHeatmap::width.HeatmapAnnotation(ha_left),
        units,
        valueOnly = TRUE
      )
    }
    if (!is.null(ha_right)) {
      width_annotation <- grid::convertWidth(
        grid::unit(width_annotation, units) +
          ComplexHeatmap::width.HeatmapAnnotation(ha_right),
        units,
        valueOnly = TRUE
      )
    }
  }
  dend_width <- name_width <- NULL
  dend_height <- name_height <- NULL
  if (inherits(ht_list, "HeatmapList")) {
    for (nm in names(ht_list@ht_list)) {
      ht <- ht_list@ht_list[[nm]]
      dend_width <- max(ht@row_dend_param$width, dend_width)
      dend_height <- max(ht@column_dend_param$height, dend_height)
      name_width <- max(ht@row_names_param$max_width, name_width)
      name_height <- max(ht@column_names_param$max_height, name_height)
    }
  } else if (inherits(ht_list, "Heatmap")) {
    ht <- ht_list
    dend_width <- max(ht@row_dend_param$width, dend_width)
    dend_height <- max(ht@column_dend_param$height, dend_height)
    name_width <- max(ht@row_names_param$max_width, name_width)
    name_height <- max(ht@column_names_param$max_height, name_height)
  } else {
    log_message(
      "{.arg ht_list} is not a class of HeatmapList or Heatmap",
      message_type = "error"
    )
  }

  lgd_width <- numeric(0)
  if (length(legend_list) > 0) {
    legend_widths <- lapply(legend_list, function(x) {
      out <- tryCatch(
        ComplexHeatmap::width.Legends(x),
        error = function(e) NULL
      )
      if (is.null(out) || length(out) == 0) {
        return(NULL)
      }
      out
    })
    legend_widths <- legend_widths[!vapply(legend_widths, is.null, logical(1))]
    if (length(legend_widths) > 0) {
      lgd_width <- vapply(
        legend_widths,
        function(w) grid::convertWidth(w, unitTo = units, valueOnly = TRUE),
        numeric(1)
      )
    }
  }
  width_sum <- grid::convertWidth(
    grid::unit(width_sum, units) +
      grid::unit(width_annotation, units) +
      dend_width +
      name_width,
    units,
    valueOnly = TRUE
  ) +
    sum(lgd_width)
  height_sum <- max(
    grid::convertHeight(
      grid::unit(height_sum, units) +
        grid::unit(height_annotation, units) +
        dend_height +
        name_height,
      units,
      valueOnly = TRUE
    ),
    grid::convertHeight(
      grid::unit(0.95, "npc"),
      units,
      valueOnly = TRUE
    )
  )
  return(
    list(
      width_sum = width_sum,
      height_sum = height_sum
    )
  )
}

#' @title Cluster within group
#'
#' @param mat A matrix of data
#' @param factor A factor
#'
#' @return A dendrogram with ordered leaves
#'
#' @export
#'
#' @examples
#' mat <- matrix(rnorm(100), 10, 10)
#' factor <- factor(rep(1:2, each = 5))
#' dend <- cluster_within_group2(mat, factor)
#' dend
#' plot(dend)
cluster_within_group2 <- function(mat, factor) {
  if (!is.factor(factor)) {
    factor <- factor(factor, levels = unique(factor))
  }
  dend_list <- list()
  order_list <- list()
  for (le in unique(levels(factor))) {
    m <- mat[, factor == le, drop = FALSE]
    if (ncol(m) == 1) {
      order_list[[le]] <- which(factor == le)
      dend_list[[le]] <- structure(
        which(factor == le),
        class = "dendrogram",
        leaf = TRUE,
        label = 1,
        members = 1
      )
    } else if (ncol(m) > 1) {
      hc1 <- stats::hclust(
        stats::as.dist(
          proxyC::dist(Matrix::t(m))
        )
      )
      dend_list[[le]] <- stats::as.dendrogram(hc1)
      dend_order <- stats::order.dendrogram(dend_list[[le]])
      order_list[[le]] <- which(factor == le)[dend_order]
      dendextend::order.dendrogram(dend_list[[le]]) <- order_list[[le]]
    }
    attr(dend_list[[le]], ".class_label") <- le
  }
  parent <- stats::as.dendrogram(
    stats::hclust(
      stats::as.dist(
        proxyC::dist(
          Matrix::t(sapply(
            order_list,
            function(x) rowMeans(mat[, x, drop = FALSE])
          ))
        )
      )
    )
  )
  dend_list <- lapply(dend_list, function(dend) {
    stats::dendrapply(
      dend,
      function(node) {
        if (is.null(attr(node, "height"))) {
          attr(node, "height") <- 0
        }
        node
      }
    )
  })

  dend <- ComplexHeatmap::merge_dendrogram(parent, dend_list)
  dendextend::order.dendrogram(dend) <- unlist(
    order_list[stats::order.dendrogram(parent)]
  )
  dend
}

#' @title Normalize drawable objects to grobs
#'
#' @param obj A drawable object, such as a ggplot, patchwork, or grid grob.
#'
#' @return A grid grob object. If the input is not drawable, returns a null grob.
#' @export
#'
#' @examples
#' library(ggplot2)
#' p <- ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg)) +
#'   ggplot2::geom_point()
#' g <- normalize_drawable(p)
#' p + g
normalize_drawable <- function(obj) {
  x <- obj
  while (is.list(x) && length(x) > 0) {
    x <- x[[1]]
  }
  if (is.null(x)) {
    return(grid::nullGrob())
  }
  if (inherits(x, "patchwork")) {
    return(patchwork::patchworkGrob(x))
  }
  if (ggplot2::is.ggplot(x)) {
    x <- x + ggplot2::theme_void() + ggplot2::theme(
      plot.title = ggplot2::element_blank(),
      plot.subtitle = ggplot2::element_blank(),
      legend.position = "none",
      axis.title = ggplot2::element_blank(),
      axis.text = ggplot2::element_blank(),
      axis.ticks = ggplot2::element_blank(),
      strip.text = ggplot2::element_blank(),
      strip.background = ggplot2::element_blank()
    )
    return(ggplot2::ggplotGrob(x))
  }
  if (inherits(x, c("grob", "gTree"))) {
    return(x)
  }
  grid::nullGrob()
}

#' @title Build graphics callback list for anno_customize
#'
#' @md
#' @param subplots A named list of subplot objects.
#' @param prefix Prefix used to generate stable grob names.
#'
#' @return A named list of callback functions accepted by [ComplexHeatmap::anno_customize].
#'
#' @export
annotation_graphics <- function(subplots, prefix) {
  graphics <- list()
  for (nm in names(subplots)) {
    graphics[[nm]] <- local({
      p0 <- subplots[[nm]]
      g_name <- paste0(prefix, "-", nm)
      function(x, y, w, h) {
        g <- tryCatch(
          normalize_drawable(p0),
          error = function(e) grid::nullGrob()
        )
        g$name <- g_name
        tryCatch(
          grid::grid.draw(g),
          error = function(e) {
            grid::grid.draw(grid::nullGrob())
          }
        )
      }
    })
  }
  graphics
}

#' @title Build block panel function
#'
#' @md
#' @param subplot A single drawable subplot object.
#' @param name A name assigned to the generated grob.
#' @param border Whether to draw a rectangle border around the block.
#' @param border_color Border color when `border = TRUE`. Default is `"black"`.
#' @param border_size Border line width when `border = TRUE`. Default is `1`.
#'
#' @return A function with signature `(index, levels)` suitable for `panel_fun` in [ComplexHeatmap::anno_block].
#'
#' @export
#'
#' @examples
#' library(ggplot2)
#' p <- ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg)) +
#'   ggplot2::geom_point()
#' panel_fun <- annotation_block_graphics(
#'   subplot = p, name = "demo-block"
#' )
#' ComplexHeatmap::anno_block(panel_fun = panel_fun)
annotation_block_graphics <- function(
  subplot,
  name,
  border = TRUE,
  border_color = "black",
  border_size = 1
) {
  p0 <- subplot
  g_name <- name
  function(index, levels) {
    g <- tryCatch(
      normalize_drawable(p0),
      error = function(e) grid::nullGrob()
    )
    g$name <- g_name
    tryCatch(
      grid::grid.draw(g),
      error = function(e) {
        grid::grid.draw(grid::nullGrob())
      }
    )
    if (isTRUE(border)) {
      grid::grid.rect(
        gp = grid::gpar(fill = "transparent", col = border_color, lwd = border_size)
      )
    }
  }
}

#' @title Build block fill panel function
#'
#' @md
#' @inheritParams palette_colors
#' @param levels Character/factor levels used as keys for fill mapping.
#' @param fill_values Optional named fill vector. Names should match `levels`.
#' @param border Whether to draw block border.
#' @param border_color Border color when `border = TRUE`. Default is `"black"`.
#' @param border_size Border line width when `border = TRUE`. Default is `1`.
#'
#' @return A function with signature `(index, levels)` suitable for `panel_fun` in [ComplexHeatmap::anno_block].
#'
#' @export
#'
#' @examples
#' library(ggplot2)
#' lv <- c("A", "B", "C")
#' panel_fun <- annotation_block_fill_graphics(
#'   levels = lv,
#'   fill_values = c(A = "#1b9e77", B = "#d95f02", C = "#7570b3")
#' )
#' ComplexHeatmap::anno_block(panel_fun = panel_fun)
annotation_block_fill_graphics <- function(
  levels,
  palette = NULL,
  palcolor = NULL,
  fill_values = NULL,
  border = TRUE,
  border_color = "black",
  border_size = 1
) {
  if (is.null(fill_values)) {
    fill_values <- palette_colors(
      levels,
      type = "discrete",
      palette = palette,
      palcolor = palcolor,
      matched = TRUE
    )
  }
  names(fill_values) <- as.character(levels)

  function(index, levels) {
    level_key <- as.character(levels[1])
    fill <- unname(fill_values[[level_key]])
    if (is.null(fill) || length(fill) == 0 || is.na(fill)) {
      fill <- "transparent"
    }
    grid::grid.rect(
      gp = grid::gpar(
        fill = fill, col = if (isTRUE(border)) border_color else NA, lwd = border_size
      )
    )
  }
}

#' @title Build HeatmapAnnotation with safe parameter merge
#'
#' @md
#' @inheritParams annotation_block_graphics
#' @param annotations Named list of annotation components (for example, `anno_simple`, `anno_block`, `anno_customize`).
#' @param which Annotation direction (`"row"` or `"column"`).
#' @param show_annotation_name Whether to show annotation names.
#' @param annotation_name_side Side for annotation names.
#' @param params Additional user parameters; duplicated keys are ignored if already set explicitly.
#'
#' @return A [ComplexHeatmap::HeatmapAnnotation] object.
#'
#' @export
#'
#' @examples
#' anno <- list(
#'   group = ComplexHeatmap::anno_simple(
#'     x = c("A", "B", "A"),
#'     col = c(A = "#1b9e77", B = "#d95f02"),
#'     which = "column"
#'   )
#' )
#' ha <- build_heatmap_annotation(
#'   annotations = anno,
#'   which = "column",
#'   show_annotation_name = TRUE,
#'   annotation_name_side = "left",
#'   params = list(gap = grid::unit(1, "mm"))
#' )
#' ha
build_heatmap_annotation <- function(
  annotations,
  which,
  show_annotation_name = TRUE,
  annotation_name_side = NULL,
  border = NULL,
  params = NULL
) {
  anno_args <- c(
    annotations,
    which = which,
    show_annotation_name = show_annotation_name
  )
  if (!is.null(annotation_name_side)) {
    anno_args[["annotation_name_side"]] <- annotation_name_side
  }
  if (!is.null(border)) {
    anno_args[["border"]] <- border
  }
  if (!is.null(params) && length(params) > 0) {
    anno_args <- c(
      anno_args,
      params[setdiff(names(params), names(anno_args))]
    )
  }
  do.call(
    ComplexHeatmap::HeatmapAnnotation,
    args = anno_args
  )
}

#' @title Visualize colors in HTML widget
#'
#' @description
#' Display a grid of color swatches with optional names or color codes.
#'
#' @md
#' @param colors A character vector of hex color codes.
#' @param names Optional. A character vector of names for each color.
#' Default is `NULL`, which means hex color codes will be displayed.
#' You can pass any labels (e.g., RGB values, custom names) via this parameter.
#' @param num_per_row Number of colors per row.
#' Default is `30`.
#' @param title Optional title for the visualization.
#' Default is `NULL`.
#' @param label_mode Label layout mode. Default is `"auto"`, which detects
#' the label type from `names`. Use `"chinese"`, `"pinyin"`, `"rgb"`,
#' or `"hex"` to force a layout optimized for that label style.
#'
#' @return
#' An HTML widget.
#'
#' @export
#'
#' @examples
#' # Visualize a simple color palette
#' visual_colors(
#'   colors = c("#FF0000", "#00FF00", "#0000FF"),
#'   names = c("Red", "Green", "Blue")
#' )
#'
#' visual_colors(
#'   colors = c("#FF0000", "#00FF00"),
#'   names = c("(255, 0, 0)", "(0, 255, 0)")
#' )
#'
#' visual_colors(thisplot::palette_list$Paired)
#'
#' # Use with ChineseColors
#' cc <- ChineseColors()
#' visual_colors(
#'   colors = cc$blue[1:60],
#'   title = "Chinese Blue Colors"
#' )
visual_colors <- function(
  colors,
  names = NULL,
  num_per_row = 30,
  title = NULL,
  label_mode = c("auto", "chinese", "pinyin", "rgb", "hex")
) {
  label_mode <- match.arg(label_mode)

  if (is.null(colors) || length(colors) == 0) {
    log_message(
      "No colors provided",
      message_type = "error"
    )
  }

  if (is.null(names)) {
    names <- colors
  }

  if (length(names) != length(colors)) {
    if (length(names) < length(colors)) {
      names <- c(names, colors[(length(names) + 1):length(colors)])
    } else {
      names <- names[seq_along(colors)]
    }
  }

  n_colors <- length(colors)
  display_names <- as.character(names)

  luminance <- function(hex_color) {
    rgb <- grDevices::col2rgb(hex_color) / 255
    0.299 * rgb[1, ] + 0.587 * rgb[2, ] + 0.114 * rgb[3, ]
  }

  text_color_for <- function(hex_color) {
    ifelse(luminance(hex_color) > 0.5, "#000000", "#FFFFFF")
  }

  is_chinese <- function(text) {
    if (is.null(text) || length(text) == 0 || is.na(text)) {
      return(FALSE)
    }
    grepl("[\u4e00-\u9fff]", text, perl = TRUE)
  }

  is_hex_label <- function(text) {
    if (is.null(text) || length(text) == 0 || is.na(text)) {
      return(FALSE)
    }
    grepl("^#[A-Fa-f0-9]{6}$", text)
  }

  is_rgb_label <- function(text) {
    if (is.null(text) || length(text) == 0 || is.na(text)) {
      return(FALSE)
    }
    grepl("^\\(\\s*\\d{1,3}\\s*,\\s*\\d{1,3}\\s*,\\s*\\d{1,3}\\s*\\)$", text)
  }

  detect_label_mode <- function(texts) {
    valid_texts <- texts[!is.na(texts) & nzchar(texts)]
    if (length(valid_texts) == 0) {
      return("hex")
    }
    if (all(vapply(valid_texts, is_chinese, logical(1)))) {
      return("chinese")
    }
    if (all(vapply(valid_texts, is_hex_label, logical(1)))) {
      return("hex")
    }
    if (all(vapply(valid_texts, is_rgb_label, logical(1)))) {
      return("rgb")
    }
    if (all(grepl("^[A-Za-z]+$", valid_texts))) {
      return("pinyin")
    }
    "pinyin"
  }

  get_label_layout <- function(mode, texts) {
    switch(
      mode,
      chinese = list(
        cell_width = 21,
        cell_height = 50,
        font_size = 8.5,
        line_height = 1.08,
        letter_spacing = "0.2px",
        rotate = FALSE,
        white_space = "normal",
        font_weight = 500
      ),
      pinyin = list(
        cell_width = 21,
        cell_height = 50,
        font_size = 9,
        line_height = 1,
        letter_spacing = "0px",
        rotate = FALSE,
        white_space = "nowrap",
        font_weight = 500
      ),
      rgb = list(
        cell_width = 21,
        cell_height = 50,
        font_size = 8.8,
        line_height = 1,
        letter_spacing = "0px",
        rotate = FALSE,
        white_space = "nowrap",
        font_weight = 500
      ),
      hex = list(
        cell_width = 21,
        cell_height = 50,
        font_size = 9,
        line_height = 1,
        letter_spacing = "0px",
        rotate = FALSE,
        white_space = "nowrap",
        font_weight = 600
      ),
      list(
        cell_width = 21,
        cell_height = 50,
        font_size = 8.8,
        line_height = 1,
        letter_spacing = "0px",
        rotate = FALSE,
        white_space = "nowrap",
        font_weight = 500
      )
    )
  }

  layout_mode <- if (identical(label_mode, "auto")) {
    detect_label_mode(display_names)
  } else {
    label_mode
  }
  label_layout <- get_label_layout(layout_mode, display_names)
  cell_width_px <- label_layout$cell_width
  cell_height_px <- label_layout$cell_height

  format_text_display <- function(text) {
    if (identical(layout_mode, "chinese") && is_chinese(text)) {
      chars <- strsplit(text, "", fixed = TRUE)[[1]]
      if (length(chars) == 1) {
        return(chars[1])
      }
      htmltools::HTML(paste(chars, collapse = "<br>"))
    } else {
      text
    }
  }

  get_text_style <- function(text, text_color) {
    base_style <- paste0(
      "color:", text_color, ";",
      "font-size:", label_layout$font_size, "px;",
      "font-family:'Noto Sans SC','PingFang SC','Microsoft YaHei',sans-serif;",
      "letter-spacing:", label_layout$letter_spacing, ";",
      "line-height:", label_layout$line_height, ";",
      "font-weight:", label_layout$font_weight, ";"
    )

    if (identical(layout_mode, "chinese") && is_chinese(text)) {
      paste0(
        base_style,
        "display:block;",
        "text-align:center;",
        "width:100%;",
        "white-space:normal;"
      )
    } else {
      paste0(
        base_style,
        "display:block;",
        "white-space:", label_layout$white_space, ";",
        "writing-mode:vertical-rl;",
        "text-orientation:mixed;",
        "transform:rotate(180deg);",
        "transform-origin:center center;",
        "text-align:center;"
      )
    }
  }

  idx_list <- split(
    seq_len(n_colors), ceiling(seq_len(n_colors) / num_per_row)
  )
  rows_html <- lapply(
    idx_list, function(idxs) {
      cells <- lapply(idxs, function(i) {
        text_color <- text_color_for(colors[i])
        text_style <- get_text_style(display_names[i], text_color)
        text_content <- format_text_display(display_names[i])
        container_base <- paste0(
          "display:flex;",
          "align-items:center;",
          "justify-content:center;",
          "width:100%;",
          "height:100%;",
          "box-sizing:border-box;",
          "padding:2px 0;",
          "overflow:hidden;"
        )
        container_style <- container_base

        htmltools::tags$td(
          style = paste0(
            "padding:2px 2px;",
            "width:", cell_width_px, "px;",
            "min-width:", cell_width_px, "px;",
            "max-width:", cell_width_px, "px;",
            "height:", cell_height_px, "px;",
            "min-height:", cell_height_px, "px;",
            "border-radius:3px;",
            "background:", colors[i], ";",
            "box-shadow:0 1px 2px rgba(0,0,0,0.2);",
            "text-align:center;",
            "vertical-align:middle;",
            "position:relative;",
            "overflow:hidden;",
            "box-sizing:border-box;"
          ),
          htmltools::tags$div(
            style = container_style,
            htmltools::tags$div(
              style = text_style,
              text_content
            )
          )
        )
      })

      if (length(cells) < num_per_row) {
        empties <- replicate(
          num_per_row - length(cells),
          htmltools::tags$td(
            style = paste0(
              "padding:2px 2px;",
              "width:", cell_width_px, "px;",
              "min-width:", cell_width_px, "px;",
              "max-width:", cell_width_px, "px;",
              "height:", cell_height_px, "px;",
              "min-height:", cell_height_px, "px;",
              "border-radius:3px;",
              "background:transparent;",
              "text-align:center;",
              "box-sizing:border-box;"
            ),
            ""
          ),
          simplify = FALSE
        )
        cells <- c(cells, empties)
      }

      htmltools::tags$tr(cells)
    }
  )

  header <- NULL
  if (!is.null(title)) {
    header <- htmltools::tags$div(
      style = paste0(
        "font-family:'Noto Sans SC','PingFang SC','Microsoft YaHei',sans-serif;",
        "font-size:13px;font-weight:600;",
        "color:inherit;margin-bottom:3px;"
      ),
      title
    )
  }

  table_html <- htmltools::tags$div(
    style = paste0(
      "background:transparent;",
      "padding:12px;",
      "border-radius:12px;",
      "border:1px solid rgba(0,0,0,0.1);",
      "box-shadow:none;",
      "display:inline-block;",
      "overflow-x:auto;"
    ),
    header,
    htmltools::tags$table(
      style = "border-collapse:separate;border-spacing:3px;",
      htmltools::tags$tbody(rows_html)
    )
  )

  if (check_ci_env()) {
    htmltools::browsable(table_html)
  } else {
    invisible(table_html)
  }
}

#' @title Drop unused data in the plot
#'
#' @description
#' Drop unused data points from a ggplot or patchwork object while preserving
#' the plot structure. This function keeps only a single row of data for each
#' unique combination of used variables, which can significantly reduce the
#' object size when the original data contains many rows that are not displayed
#' in the plot (e.g., due to scale limits or filtering).
#'
#' @md
#' @param p A `ggplot` object or a `patchwork` object.
#'
#' @return A `ggplot` or `patchwork` object with unused data points removed.
#'
#' @export
#'
#' @examples
#' library(ggplot2)
#' library(patchwork)
#' p <- ggplot(
#'   data = mtcars,
#'   aes(x = mpg, y = wt, colour = cyl)
#' ) +
#'   geom_point() +
#'   scale_x_continuous(limits = c(10, 30)) +
#'   scale_y_continuous(limits = c(1, 6))
#' object.size(p)
#'
#' p_drop <- drop_data(p)
#' object.size(p_drop)
#'
#' p / p_drop
drop_data <- function(p) {
  UseMethod(generic = "drop_data", object = p)
}

plot_clone <- function(plot) {
  p <- plot
  p@scales <- plot@scales$clone()
  p
}

#' @export
#' @rdname drop_data
#' @method drop_data ggplot
drop_data.ggplot <- function(p) {
  p <- plot_clone(p)

  # fix the scales for x/y axis and 'fill', 'color', 'shape',...
  for (i in seq_along(p$scales$scales)) {
    if (inherits(p$scales$scales[[i]], "ScaleDiscrete")) {
      p$scales$scales[[i]][["drop"]] <- FALSE
    }
    if (inherits(p$scales$scales[[i]], "ScaleContinuous")) {
      limits <- p$scales$scales[[i]]$get_limits()
      if (p$scales$scales[[i]]$aesthetics[1] == "x") {
        p$coordinates$limits$x <- limits
      }
      if (p$scales$scales[[i]]$aesthetics[1] == "y") {
        p$coordinates$limits$y <- limits
      }
    }
  }

  vars <- get_vars(p, verbose = FALSE)
  if (length(p$data) > 0) {
    vars_modified <- names(
      which(
        sapply(
          p$data[, intersect(colnames(p$data), vars), drop = FALSE], class
        ) == "character"
      )
    )
    for (v in vars_modified) {
      p$data[[v]] <- as.factor(p$data[[v]])
    }
    p$data <- p$data[1, , drop = FALSE]
  }

  for (i in seq_along(p$layers)) {
    if (length(p$layers[[i]]$data) > 0) {
      vars_modified <- names(
        which(
          sapply(
            p$layers[[i]]$data[, intersect(colnames(p$layers[[i]]$data), vars), drop = FALSE], class
          ) == "character"
        )
      )
      for (v in vars_modified) {
        p$layers[[i]]$data[[v]] <- as.factor(p$layers[[i]]$data[[v]])
      }
      p$layers[[i]]$data <- p$layers[[i]]$data[1, , drop = FALSE]
    }
  }

  return(p)
}

#' @export
#' @rdname drop_data
#' @method drop_data patchwork
drop_data.patchwork <- function(p) {
  for (i in seq_along(p$patches$plots)) {
    p$patches$plots[[i]] <- drop_data(p$patches$plots[[i]])
  }
  drop_data.ggplot(p)
}

#' @export
#' @rdname drop_data
#' @method drop_data default
drop_data.default <- function(p) {
  p
}

#' @title Data Dimensional Plot
#'
#' @description
#' Visualize observations on precomputed two-dimensional coordinates.
#'
#' @md
#' @param data A data frame containing coordinates and optional grouping columns.
#' @param x Name of the column used as the x-axis coordinate.
#' @param y Name of the column used as the y-axis coordinate.
#' @param group.by Name of the column used to color observations.
#' Default is `NULL`.
#' @param split.by Name of the column used to split the plot into facets.
#' Default is `NULL`.
#' @param shape.by Name of the column used to map point shapes.
#' Default is `NULL`.
#' @param show_na Whether to keep missing values in `group.by`, `split.by`,
#' or `shape.by` as an explicit `"NA"` level. Default is `FALSE`.
#' @param palette Color palette name. Available palettes can be found in
#' [show_palettes]. Default is `"Chinese"`.
#' @param palcolor Custom colors used to create a color palette.
#' Default is `NULL`.
#' @param NA_color Color used for missing values.
#' @param pt.size Point size.
#' @param pt.alpha Point transparency.
#' @param shape.values Point shapes used when `shape.by` is set.
#' @param add_mark Whether to add marks around groups. Default is `FALSE`.
#' @param mark_type Type of mark to add around groups.
#' One of `"hull"`, `"ellipse"`, `"rect"`, or `"circle"`.
#' Default is `"hull"`. `"ellipse"` uses [ggplot2::stat_ellipse]
#' with `type = "norm"` and `level = 0.95`; the other mark types use ggforce marks.
#' @param mark_expand Expansion of the mark around each group.
#' This is used for `"hull"`, `"rect"`, and `"circle"` marks.
#' Default is `grid::unit(3, "mm")`.
#' @param mark_alpha Transparency of the mark.
#' Default is `0.1`.
#' @param mark_linetype Line type of the mark border.
#' Default is `1`.
#' @param label Whether to label group centers.
#' @param label.size Size of labels.
#' @param label.fg Foreground color of labels.
#' @param label.bg Background color of labels.
#' @param label.bg.r Background ratio of labels.
#' @param label_repel Whether labels repel from their group centers.
#' @param label_repulsion Repulsion force for labels.
#' @param label_point_size Size of center points for repelled labels.
#' @param label_point_color Color of center points for repelled labels.
#' @param label_segment_color Color of label segments.
#' @param add_origin Whether to add dashed x = 0 and y = 0 reference lines.
#' @param origin.color Color of origin reference lines.
#' @param origin.linetype Line type of origin reference lines.
#' @param origin.linewidth Line width of origin reference lines.
#' @param aspect.ratio Aspect ratio passed to `theme()`.
#' @param title Plot title.
#' @param subtitle Plot subtitle.
#' @param xlab Label for the x axis.
#' @param ylab Label for the y axis.
#' @param legend.position Position of the legend.
#' @param legend.direction Direction of the legend.
#' @param legend.title Title of the group legend.
#' @param theme_use Theme function applied to the plot.
#' @param theme_args A named list of arguments passed to `theme_use`.
#' @param seed Random seed.
#'
#' @return A ggplot object.
#'
#' @export
#'
#' @examples
#' set.seed(1)
#' plot_data <- data.frame(
#'   x = c(rnorm(40, -1), rnorm(40, 1)),
#'   y = c(rnorm(40, 0), rnorm(40, 1)),
#'   group = rep(c("A", "B"), each = 40)
#' )
#' DimDataPlot(
#'   plot_data,
#'   x = "x",
#'   y = "y",
#'   group.by = "group"
#' )
#'
#' DimDataPlot(
#'   plot_data,
#'   x = "x",
#'   y = "y",
#'   group.by = "group",
#'   add_origin = TRUE,
#'   mark_type = "ellipse"
#' )
#'
#' DimDataPlot(
#'   plot_data,
#'   x = "x",
#'   y = "y",
#'   group.by = "group",
#'   mark_type = "circle",
#'   mark_alpha = 0.3,
#'   mark_expand = grid::unit(1, "mm"),
#'   mark_linetype = 2
#' )
DimDataPlot <- function(
    data,
    x,
    y,
    group.by = NULL,
    split.by = NULL,
    shape.by = NULL,
    show_na = FALSE,
    palette = "Chinese",
    palcolor = NULL,
    NA_color = "grey80",
    pt.size = 2,
    pt.alpha = 1,
    shape.values = NULL,
    add_mark = FALSE,
    mark_type = c("hull", "ellipse", "rect", "circle"),
    mark_expand = grid::unit(3, "mm"),
    mark_alpha = 0.1,
    mark_linetype = 1,
    label = FALSE,
    label.size = 4,
    label.fg = "black",
    label.bg = "white",
    label.bg.r = 0.1,
    label_repel = FALSE,
    label_repulsion = 20,
    label_point_size = 1,
    label_point_color = "black",
    label_segment_color = "black",
    add_origin = FALSE,
    origin.color = "grey30",
    origin.linetype = "dashed",
    origin.linewidth = 0.4,
    aspect.ratio = 1,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    legend.title = NULL,
    theme_use = "theme_this",
    theme_args = list(),
    seed = 11) {
  set.seed(seed)
  mark_type <- match.arg(mark_type)

  data <- as.data.frame(data)
  column_args <- list(
    x = x,
    y = y,
    group.by = group.by,
    split.by = split.by,
    shape.by = shape.by
  )
  optional_args <- c("group.by", "split.by", "shape.by")
  for (arg in names(column_args)) {
    column <- column_args[[arg]]
    if (is.null(column) && arg %in% optional_args) {
      next
    }
    if (!is.character(column) || length(column) != 1L || is.na(column)) {
      log_message(
        "{.arg {arg}} must be one column name.",
        message_type = "error"
      )
    }
    if (!column %in% colnames(data)) {
      log_message(
        "{.val {column}} is not in {.arg data}.",
        message_type = "error"
      )
    }
  }

  has_group <- !is.null(group.by)
  has_split <- !is.null(split.by)
  has_shape <- !is.null(shape.by)

  plot_data <- data
  plot_data[[".plot_x"]] <- suppressWarnings(as.numeric(plot_data[[x]]))
  plot_data[[".plot_y"]] <- suppressWarnings(as.numeric(plot_data[[y]]))
  keep <- is.finite(plot_data[[".plot_x"]]) &
    is.finite(plot_data[[".plot_y"]])
  if (any(!keep)) {
    log_message(
      "Rows with non-finite coordinate values were removed.",
      message_type = "warning"
    )
  }
  plot_data <- plot_data[keep, , drop = FALSE]
  if (nrow(plot_data) == 0) {
    log_message(
      "No rows remain after removing non-finite coordinates.",
      message_type = "error"
    )
  }

  if (has_group) {
    group_info <- dim_data_plot_factor(
      plot_data[[group.by]],
      show_na = show_na,
      arg = "group.by"
    )
    plot_data <- plot_data[group_info[["keep"]], , drop = FALSE]
    plot_data[[".plot_group"]] <- group_info[["value"]]
  } else {
    plot_data[[".plot_group"]] <- factor("All", levels = "All")
  }
  if (nrow(plot_data) == 0) {
    log_message(
      "No rows remain after removing missing group values.",
      message_type = "error"
    )
  }

  if (has_split) {
    split_info <- dim_data_plot_factor(
      plot_data[[split.by]],
      show_na = show_na,
      arg = "split.by"
    )
    plot_data <- plot_data[split_info[["keep"]], , drop = FALSE]
    plot_data[[".plot_split"]] <- split_info[["value"]]
  } else {
    plot_data[[".plot_split"]] <- factor("All", levels = "All")
  }
  if (nrow(plot_data) == 0) {
    log_message(
      "No rows remain after removing missing split values.",
      message_type = "error"
    )
  }

  if (has_shape) {
    shape_info <- dim_data_plot_factor(
      plot_data[[shape.by]],
      show_na = show_na,
      arg = "shape.by"
    )
    plot_data <- plot_data[shape_info[["keep"]], , drop = FALSE]
    plot_data[[".plot_shape"]] <- shape_info[["value"]]
  }
  if (nrow(plot_data) == 0) {
    log_message(
      "No rows remain after removing missing shape values.",
      message_type = "error"
    )
  }

  group_levels <- levels(plot_data[[".plot_group"]])
  colors <- if (has_group) {
    palette_colors(
      group_levels,
      palette = palette,
      palcolor = palcolor,
      NA_keep = TRUE
    )
  } else {
    stats::setNames("#1F1F1F", group_levels)
  }
  colors <- colors[group_levels]

  plot_data <- plot_data[
    order(plot_data[[".plot_group"]], plot_data[[".plot_split"]]),
    ,
    drop = FALSE
  ]

  mark_layers <- NULL
  if (isTRUE(add_mark)) {
    check_r("concaveman", verbose = FALSE)
    if (identical(mark_type, "ellipse")) {
      mark_layers <- list(
        ggplot2::stat_ellipse(
          data = plot_data,
          mapping = ggplot2::aes(
            x = .data[[".plot_x"]],
            y = .data[[".plot_y"]],
            group = .data[[".plot_group"]],
            fill = .data[[".plot_group"]]
          ),
          geom = "polygon",
          type = "norm",
          level = 0.95,
          alpha = mark_alpha,
          color = NA,
          show.legend = FALSE,
          inherit.aes = FALSE
        ),
        ggplot2::stat_ellipse(
          data = plot_data,
          mapping = ggplot2::aes(
            x = .data[[".plot_x"]],
            y = .data[[".plot_y"]],
            color = .data[[".plot_group"]]
          ),
          geom = "path",
          type = "norm",
          level = 0.95,
          linetype = mark_linetype,
          show.legend = FALSE,
          inherit.aes = FALSE
        )
      )
    } else {
      mark_fun <- switch(
        EXPR = mark_type,
        "hull" = ggforce::geom_mark_hull,
        "rect" = ggforce::geom_mark_rect,
        "circle" = ggforce::geom_mark_circle
      )
      mark_layers <- list(
        do.call(
          mark_fun,
          list(
            data = plot_data,
            mapping = ggplot2::aes(
              x = .data[[".plot_x"]],
              y = .data[[".plot_y"]],
              color = .data[[".plot_group"]],
              fill = .data[[".plot_group"]]
            ),
            expand = mark_expand,
            alpha = mark_alpha,
            linetype = mark_linetype,
            show.legend = FALSE,
            inherit.aes = FALSE
          )
        )
      )
    }
  }

  fill_scale <- NULL
  if (isTRUE(add_mark)) {
    fill_scale <- list(
      ggplot2::scale_fill_manual(values = colors, guide = "none")
    )
  }

  origin_layers <- NULL
  if (isTRUE(add_origin)) {
    origin_layers <- list(
      ggplot2::geom_vline(
        xintercept = 0,
        color = origin.color,
        linetype = origin.linetype,
        linewidth = origin.linewidth
      ),
      ggplot2::geom_hline(
        yintercept = 0,
        color = origin.color,
        linetype = origin.linetype,
        linewidth = origin.linewidth
      )
    )
  }

  point_mapping <- ggplot2::aes(
    x = .data[[".plot_x"]],
    y = .data[[".plot_y"]],
    color = .data[[".plot_group"]]
  )
  if (has_shape) {
    point_mapping <- ggplot2::aes(
      x = .data[[".plot_x"]],
      y = .data[[".plot_y"]],
      color = .data[[".plot_group"]],
      shape = .data[[".plot_shape"]]
    )
  }

  legend_title_use <- legend.title %||% group.by %||% ""
  p <- ggplot2::ggplot(plot_data) +
    mark_layers +
    fill_scale +
    origin_layers +
    ggplot2::geom_point(
      mapping = point_mapping,
      size = pt.size,
      alpha = pt.alpha
    ) +
    ggplot2::scale_color_manual(
      name = legend_title_use,
      values = colors,
      na.value = NA_color,
      guide = if (has_group) {
        ggplot2::guide_legend(
          title.hjust = 0,
          order = 1,
          override.aes = list(size = 4, alpha = 1)
        )
      } else {
        "none"
      }
    ) +
    ggplot2::labs(
      title = title,
      subtitle = subtitle,
      x = xlab %||% x,
      y = ylab %||% y
    ) +
    do.call(theme_use, theme_args) +
    ggplot2::theme(
      aspect.ratio = aspect.ratio,
      legend.position = legend.position,
      legend.direction = legend.direction
    )

  if (has_shape) {
    shape_levels <- levels(plot_data[[".plot_shape"]])
    shape_values <- dim_data_plot_shapes(shape_levels, shape.values)
    p <- p +
      ggplot2::scale_shape_manual(
        name = if (identical(shape.by, group.by)) {
          legend_title_use
        } else {
          shape.by
        },
        values = shape_values
      )
  }

  if (has_split) {
    p <- p + ggplot2::facet_grid(. ~ .plot_split)
  }

  if (isTRUE(label) && has_group) {
    label_data <- stats::aggregate(
      plot_data[, c(".plot_x", ".plot_y"), drop = FALSE],
      by = list(
        .plot_group = plot_data[[".plot_group"]],
        .plot_split = plot_data[[".plot_split"]]
      ),
      FUN = stats::median
    )
    label_data[[".plot_label"]] <- as.character(label_data[[".plot_group"]])

    if (isTRUE(label_repel)) {
      p <- p +
        ggplot2::geom_point(
          data = label_data,
          mapping = ggplot2::aes(
            x = .data[[".plot_x"]],
            y = .data[[".plot_y"]]
          ),
          shape = "circle",
          color = label_point_color,
          size = label_point_size,
          inherit.aes = FALSE
        ) +
        ggrepel::geom_text_repel(
          data = label_data,
          mapping = ggplot2::aes(
            x = .data[[".plot_x"]],
            y = .data[[".plot_y"]],
            label = .data[[".plot_label"]]
          ),
          fontface = "bold",
          min.segment.length = 0,
          segment.color = label_segment_color,
          point.size = label_point_size,
          max.overlaps = 100,
          force = label_repulsion,
          color = label.fg,
          bg.color = label.bg,
          bg.r = label.bg.r,
          size = label.size,
          inherit.aes = FALSE
        )
    } else {
      p <- p +
        ggrepel::geom_text_repel(
          data = label_data,
          mapping = ggplot2::aes(
            x = .data[[".plot_x"]],
            y = .data[[".plot_y"]],
            label = .data[[".plot_label"]]
          ),
          fontface = "bold",
          min.segment.length = 0,
          point.size = NA,
          max.overlaps = 100,
          force = 0,
          color = label.fg,
          bg.color = label.bg,
          bg.r = label.bg.r,
          size = label.size,
          inherit.aes = FALSE
        )
    }
  }

  attr(p, "data") <- plot_data
  p
}

dim_data_plot_factor <- function(x, show_na = FALSE, arg = "group") {
  keep <- rep(TRUE, length(x))
  if (isTRUE(show_na)) {
    x_chr <- as.character(x)
    levels_use <- if (is.factor(x)) {
      levels(x)
    } else {
      unique(x_chr[!is.na(x_chr)])
    }
    levels_use <- unique(c(levels_use, "NA"))
    x_chr[is.na(x_chr)] <- "NA"
    value <- factor(x_chr, levels = levels_use)
  } else {
    keep <- !is.na(x)
    if (any(!keep)) {
      log_message(
        "Rows with missing {.arg {arg}} values were removed.",
        message_type = "warning"
      )
    }
    x <- x[keep]
    value <- if (is.factor(x)) {
      factor(as.character(x), levels = levels(x))
    } else {
      factor(as.character(x), levels = unique(as.character(x)))
    }
  }
  list(value = value, keep = keep)
}

dim_data_plot_shapes <- function(levels_use, shape.values = NULL) {
  if (is.null(shape.values)) {
    shape.values <- c(16, 17, 15, 18, 3, 7, 8, 0, 1, 2, 5, 6)
  }
  if (!is.null(names(shape.values)) && all(levels_use %in% names(shape.values))) {
    return(shape.values[levels_use])
  }
  shape.values <- rep(shape.values, length.out = length(levels_use))
  stats::setNames(shape.values, levels_use)
}

#' @title Graph Plot
#'
#' @description
#' A function to plot a graph with nodes and edges.
#'
#' @md
#' @param node A data frame representing the nodes of the graph.
#' @param edge A matrix representing the edges of the graph.
#' @param transition A matrix representing the transitions between nodes.
#' @param node_coord A character vector specifying the names of the columns in `node` that represent the x and y coordinates.
#' @param node_group A character vector specifying the name of the column in `node` that represents the grouping of the nodes.
#' @param node_palette A character vector specifying the name of the color palette for node groups.
#' @param node_palcolor A character vector specifying the names of the colors for each node group.
#' @param node_size A numeric value or column name of `node` specifying the size of the nodes.
#' @param node_alpha A numeric value or column name of `node` specifying the transparency of the nodes.
#' @param node_highlight A character vector specifying the names of nodes to highlight.
#' @param node_highlight_color A character vector specifying the color for highlighting nodes.
#' @param label Whether to show labels for the nodes.
#' @param label.size The size of the labels.
#' @param label.fg A character vector specifying the foreground color of the labels.
#' @param label.bg A character vector specifying the background color of the labels.
#' @param label.bg.r The background color transparency of the labels.
#' @param label_insitu Whether to display the node group labels in situ or as numeric values.
#' @param label_repel Whether to use force-directed label repulsion.
#' @param label_repulsion The repulsion force for labels.
#' @param label_point_size The size of the label points.
#' @param label_point_color A character vector specifying the color of the label points.
#' @param label_segment_color A character vector specifying the color for the label segments.
#' @param edge_threshold The threshold for removing edges.
#' @param use_triangular A character vector specifying which part of the edge matrix to use (upper, lower, both).
#' @param edge_line A character vector specifying the type of line for edges (straight, curved).
#' @param edge_line_curvature The curvature of curved edges.
#' @param edge_line_angle The angle of curved edges.
#' @param edge_color A character vector specifying the color of the edges.
#' @param edge_size A numeric vector specifying the range of edge sizes.
#' @param edge_alpha The transparency of the edges.
#' @param edge_shorten The length of the edge shorten.
#' @param edge_offset The length of the edge offset.
#' @param edge_highlight A character vector specifying the names of edges to highlight.
#' @param edge_highlight_color A character vector specifying the color for highlighting edges.
#' @param transition_threshold The threshold for removing transitions.
#' @param transition_line A character vector specifying the type of line for transitions (straight, curved).
#' @param transition_line_curvature The curvature of curved transitions.
#' @param transition_line_angle The angle of curved transitions.
#' @param transition_color A character vector specifying the color of the transitions.
#' @param transition_size A numeric vector specifying the range of transition sizes.
#' @param transition_alpha The transparency of the transitions.
#' @param transition_arrow_type A character vector specifying the type of arrow for transitions (closed, open).
#' @param transition_arrow_angle The angle of the transition arrow.
#' @param transition_arrow_length The length of the transition arrow.
#' @param transition_shorten The length of the transition shorten.
#' @param transition_offset The length of the transition offset.
#' @param transition_highlight A character vector specifying the names of transitions to highlight.
#' @param transition_highlight_color A character vector specifying the color for highlighting transitions.
#' @param aspect.ratio Aspect ratio passed to `theme()`.
#' @param title Plot title.
#' @param subtitle Plot subtitle.
#' @param xlab Label for the x axis.
#' @param ylab Label for the y axis.
#' @param legend.position Legend position passed to `theme()`.
#' @param legend.direction Legend direction passed to `theme()`.
#' @param theme_use Theme function applied to the plot.
#' @param theme_args A named list of arguments passed to `theme_use`.
#' @param return_layer Whether to return the plot layers as a list.
#' Defaults is `FALSE`.
#'
#' @export
#' 
#' @examples
#' node <- data.frame(
#'   x = c(0.2, 0.3, 0.5, 0.3),
#'   y = c(0.2, 0, 0.6, 0.7),
#'   group = c("A", "B", "C", "B"),
#'   row.names = c("n1", "n2", "n3", "n4")
#' )
#' edge <- matrix(
#'   c(
#'     0, 0.4, 0.2, 0.1,
#'     0.4, 0, 0.3, 0.2,
#'     0.2, 0.3, 0 , 0.5,
#'     0.2, 0.2, 0.5, 0.3
#'   ),
#'   nrow = 4,
#'   byrow = TRUE,
#'   dimnames = list(rownames(node), rownames(node))
#' )
#' GraphPlot(
#'   node = node,
#'   edge = edge
#' )
#'
#' GraphPlot(
#'   node = node,
#'   edge = edge,
#'   node_group = "group",
#'   node_highlight = "n1",
#'   label = TRUE,
#'   label_repel = TRUE
#' )
GraphPlot <- function(
    node,
    edge,
    transition = NULL,
    node_coord = c("x", "y"),
    node_group = NULL,
    node_palette = "Chinese",
    node_palcolor = NULL,
    node_size = 4,
    node_alpha = 1,
    node_highlight = NULL,
    node_highlight_color = "red",
    label = FALSE,
    label.size = 3.5,
    label.fg = "white",
    label.bg = "black",
    label.bg.r = 0.1,
    label_insitu = FALSE,
    label_repel = FALSE,
    label_repulsion = 20,
    label_point_size = 1,
    label_point_color = "black",
    label_segment_color = "black",
    edge_threshold = 0.01,
    use_triangular = c("upper", "lower", "both"),
    edge_line = c("straight", "curved"),
    edge_line_curvature = 0.3,
    edge_line_angle = 90,
    edge_color = "grey30",
    edge_size = c(0.2, 1),
    edge_alpha = 0.5,
    edge_shorten = 0,
    edge_offset = 0,
    edge_highlight = NULL,
    edge_highlight_color = "red",
    transition_threshold = 0.01,
    transition_line = c("straight", "curved"),
    transition_line_curvature = 0.3,
    transition_line_angle = 90,
    transition_color = "black",
    transition_size = c(0.2, 1),
    transition_alpha = 1,
    transition_arrow_type = "closed",
    transition_arrow_angle = 20,
    transition_arrow_length = grid::unit(0.02, "npc"),
    transition_shorten = 0.05,
    transition_offset = 0,
    transition_highlight = NULL,
    transition_highlight_color = "red",
    aspect.ratio = 1,
    title = NULL,
    subtitle = NULL,
    xlab = NULL,
    ylab = NULL,
    legend.position = "right",
    legend.direction = "vertical",
    theme_use = "theme_this",
    theme_args = list(),
    return_layer = FALSE) {
  use_triangular <- match.arg(use_triangular)
  edge_line <- match.arg(edge_line)
  transition_line <- match.arg(transition_line)
  if (!is.data.frame(node)) {
    log_message(
      "'node' must be a data.frame object.",
      message_type = "error"
    )
  }
  if (!is.matrix(edge)) {
    log_message(
      "'edge' must be a matrix object.",
      message_type = "error"
    )
  }
  if (!identical(nrow(edge), ncol(edge))) {
    log_message(
      "nrow and ncol is not identical in edge matrix",
      message_type = "error"
    )
  }
  if (!identical(nrow(edge), nrow(node))) {
    log_message(
      "nrow is not identical between edge and node.",
      message_type = "error"
    )
  }
  if (!identical(rownames(edge), rownames(node))) {
    log_message(
      "rownames of node is not identical with edge matrix. They will correspond according to the order.",
      message_type = "warning"
    )
    colnames(edge) <- rownames(edge) <- rownames(node) <- rownames(node) %||%
      colnames(edge) %||%
      rownames(edge)
  }
  if (!all(node_coord %in% colnames(node))) {
    log_message(
      "Cannot find the node_coord ",
      paste(node_coord[!node_coord %in% colnames(node)], collapse = ","),
      " in the node column",
      message_type = "error"
    )
  }
  if (!is.null(transition)) {
    if (!identical(nrow(transition), nrow(node))) {
      log_message(
        "nrow is not identical between transition and node.",
        message_type = "error"
      )
    }
    if (!identical(rownames(transition), rownames(node))) {
      log_message(
        "rownames of node is not identical with transition matrix. They will correspond according to the order.",
        message_type = "warning"
      )
      colnames(transition) <- rownames(transition) <- rownames(
        node
      ) <- rownames(node) %||% colnames(transition) %||% rownames(transition)
    }
  }
  if (identical(theme_use, "theme_blank")) {
    theme_args[["xlab"]] <- xlab
    theme_args[["ylab"]] <- ylab
  }

  node <- as.data.frame(node)
  node[["x"]] <- node[[node_coord[1]]]
  node[["y"]] <- node[[node_coord[2]]]
  node[["node_name"]] <- rownames(node)
  node_group <- node_group %||% "node_name"
  node_size <- node_size %||% 5
  node_alpha <- node_alpha %||% 1
  if (!node_group %in% colnames(node)) {
    node[["node_group"]] <- node_group
  } else {
    node[["node_group"]] <- node[[node_group]]
  }
  if (!is.factor(node[["node_group"]])) {
    node[["node_group"]] <- factor(
      node[["node_group"]],
      levels = unique(node[["node_group"]])
    )
  }
  if (!node_size %in% colnames(node)) {
    if (!is.numeric(node_size)) {
      node_size <- 5
    }
    node[["node_size"]] <- node_size
    scale_size <- scale_size_identity()
  } else {
    node[["node_size"]] <- node[[node_size]]
    if (is.numeric(node[[node_size]])) {
      scale_size <- scale_size_continuous(name = node_size)
    } else {
      scale_size <- scale_size_discrete()
    }
  }
  if (!node_alpha %in% colnames(node)) {
    if (!is.numeric(node_alpha)) {
      node_alpha <- 1
    }
    node[["node_alpha"]] <- node_alpha
    scale_alpha <- scale_alpha_identity()
  } else {
    node[["node_alpha"]] <- node[[node_alpha]]
    if (is.numeric(node[[node_alpha]])) {
      scale_alpha <- scale_alpha_continuous()
    } else {
      scale_alpha <- scale_alpha_discrete()
    }
  }

  if (isTRUE(label) && isFALSE(label_insitu)) {
    label_use <- paste0(
      1:nlevels(node[["node_group"]]),
      ": ",
      levels(node[["node_group"]])
    )
  } else {
    label_use <- levels(node[["node_group"]])
  }
  global_size <- sqrt(
    max(node[["x"]], na.rm = TRUE)^2 + max(node[["y"]], na.rm = TRUE)^2
  )

  edge[edge <= edge_threshold] <- NA
  if (use_triangular == "upper") {
    edge[lower.tri(edge)] <- NA
  } else if (use_triangular == "lower") {
    edge[upper.tri(edge)] <- NA
  }
  edge_df <- reshape2::melt(
    edge,
    na.rm = TRUE,
    stringsAsFactors = FALSE
  )
  if (nrow(edge_df) == 0) {
    edge_layer <- NULL
  } else {
    colnames(edge_df) <- c("from", "to", "size")
    edge_df[, "from"] <- as.character(edge_df[, "from"])
    edge_df[, "to"] <- as.character(edge_df[, "to"])
    edge_df[, "size"] <- as.numeric(edge_df[, "size"])
    edge_df[, "x"] <- node[edge_df[, "from"], "x"]
    edge_df[, "y"] <- node[edge_df[, "from"], "y"]
    edge_df[, "xend"] <- node[edge_df[, "to"], "x"]
    edge_df[, "yend"] <- node[edge_df[, "to"], "y"]
    rownames(edge_df) <- edge_df[, "edge_name"] <- paste0(
      edge_df[, "from"],
      "-",
      edge_df[, "to"]
    )
    edge_df <- segements_df(
      edge_df,
      global_size * edge_shorten,
      global_size * edge_shorten,
      global_size * edge_offset
    )

    linetype <- ifelse(is.null(transition), 1, 2)
    if (edge_line == "straight") {
      edge_layer <- list(
        geom_segment(
          data = edge_df,
          mapping = aes(
            x = .data[["x"]],
            y = .data[["y"]],
            xend = .data[["xend"]],
            yend = .data[["yend"]],
            linewidth = .data[["size"]]
          ),
          lineend = "round",
          linejoin = "mitre",
          linetype = linetype,
          color = edge_color,
          alpha = edge_alpha,
          inherit.aes = FALSE,
          show.legend = FALSE
        )
      )
      if (!is.null(edge_highlight)) {
        edge_df_highlight <- edge_df[
          edge_df[["edge_name"]] %in% edge_highlight, ,
          drop = FALSE
        ]
        edge_layer <- c(
          edge_layer,
          list(
            geom_segment(
              data = edge_df_highlight,
              mapping = aes(
                x = .data[["x"]],
                y = .data[["y"]],
                xend = .data[["xend"]],
                yend = .data[["yend"]],
                linewidth = .data[["size"]]
              ),
              lineend = "round",
              linejoin = "mitre",
              linetype = linetype,
              color = edge_highlight_color,
              alpha = 1,
              inherit.aes = FALSE,
              show.legend = FALSE
            )
          )
        )
      }
    } else {
      edge_layer <- list(
        geom_curve(
          data = edge_df,
          mapping = aes(
            x = .data[["x"]],
            y = .data[["y"]],
            xend = .data[["xend"]],
            yend = .data[["yend"]],
            linewidth = .data[["size"]]
          ),
          curvature = edge_line_curvature,
          angle = edge_line_angle,
          lineend = "round",
          linetype = linetype,
          color = edge_color,
          alpha = edge_alpha,
          inherit.aes = FALSE,
          show.legend = FALSE
        )
      )
      if (!is.null(edge_highlight)) {
        edge_df_highlight <- edge_df[
          edge_df[["edge_name"]] %in% edge_highlight, ,
          drop = FALSE
        ]
        edge_layer <- c(
          edge_layer,
          list(
            geom_curve(
              data = edge_df_highlight,
              mapping = aes(
                x = .data[["x"]],
                y = .data[["y"]],
                xend = .data[["xend"]],
                yend = .data[["yend"]],
                linewidth = .data[["size"]]
              ),
              curvature = edge_line_curvature,
              angle = edge_line_angle,
              lineend = "round",
              linetype = linetype,
              color = edge_highlight_color,
              alpha = 1,
              inherit.aes = FALSE,
              show.legend = FALSE
            )
          )
        )
      }
    }
    edge_layer <- c(
      edge_layer,
      list(
        scale_linewidth_continuous(
          range = range(edge_size),
          guide = "none"
        ),
        ggnewscale::new_scale("linewidth")
      )
    )
  }

  if (!is.null(transition)) {
    trans2 <- trans1 <- as_matrix(transition)
    trans1[lower.tri(trans1)] <- 0
    trans2[upper.tri(trans2)] <- 0
    trans <- Matrix::t(trans1) - trans2
    trans[abs(trans) <= transition_threshold] <- NA
    trans_df <- reshape2::melt(
      trans,
      na.rm = TRUE,
      stringsAsFactors = FALSE
    )
    if (nrow(trans_df) == 0) {
      trans_layer <- NULL
    } else {
      trans_df <- as.data.frame(
        Matrix::t(
          apply(trans_df, 1, function(x) {
            if (as.numeric(x[3]) < 0) {
              return(c(x[c(2, 1)], -as.numeric(x[3])))
            } else {
              return(x)
            }
          })
        )
      )
      colnames(trans_df) <- c("from", "to", "size")
      trans_df[, "from"] <- as.character(trans_df[, "from"])
      trans_df[, "to"] <- as.character(trans_df[, "to"])
      trans_df[, "size"] <- as.numeric(trans_df[, "size"])
      trans_df[, "x"] <- node[trans_df[, "from"], "x"]
      trans_df[, "y"] <- node[trans_df[, "from"], "y"]
      trans_df[, "xend"] <- node[trans_df[, "to"], "x"]
      trans_df[, "yend"] <- node[trans_df[, "to"], "y"]
      rownames(trans_df) <- trans_df[, "trans_name"] <- paste0(
        trans_df[, "from"],
        "-",
        trans_df[, "to"]
      )
      trans_df <- segements_df(
        trans_df,
        global_size * transition_shorten,
        global_size * transition_shorten,
        global_size * transition_offset
      )

      if (transition_line == "straight") {
        trans_layer <- list(
          geom_segment(
            data = trans_df,
            mapping = aes(
              x = .data[["x"]],
              y = .data[["y"]],
              xend = .data[["xend"]],
              yend = .data[["yend"]],
              linewidth = .data[["size"]]
            ),
            arrow = grid::arrow(
              angle = transition_arrow_angle,
              type = transition_arrow_type,
              length = transition_arrow_length
            ),
            lineend = "round",
            linejoin = "mitre",
            color = transition_color,
            alpha = transition_alpha,
            inherit.aes = FALSE,
            show.legend = FALSE
          )
        )
        if (!is.null(transition_highlight)) {
          trans_df_highlight <- trans_df[
            trans_df[["trans_name"]] %in% transition_highlight, ,
            drop = FALSE
          ]
          trans_layer <- c(
            trans_layer,
            list(
              geom_segment(
                data = trans_df_highlight,
                mapping = aes(
                  x = .data[["x"]],
                  y = .data[["y"]],
                  xend = .data[["xend"]],
                  yend = .data[["yend"]],
                  linewidth = .data[["size"]]
                ),
                arrow = grid::arrow(
                  angle = transition_arrow_angle,
                  type = transition_arrow_type,
                  length = transition_arrow_length
                ),
                lineend = "round",
                linejoin = "mitre",
                color = transition_highlight_color,
                alpha = 1,
                inherit.aes = FALSE,
                show.legend = FALSE
              )
            )
          )
        }
      } else {
        trans_layer <- list(
          geom_curve(
            data = trans_df,
            mapping = aes(
              x = .data[["x"]],
              y = .data[["y"]],
              xend = .data[["xend"]],
              yend = .data[["yend"]],
              linewidth = .data[["size"]]
            ),
            arrow = grid::arrow(
              angle = transition_arrow_angle,
              type = transition_arrow_type,
              length = transition_arrow_length
            ),
            curvature = transition_line_curvature,
            angle = transition_line_angle,
            lineend = "round",
            color = transition_color,
            alpha = transition_alpha,
            inherit.aes = FALSE,
            show.legend = FALSE
          )
        )
        if (!is.null(edge_highlight)) {
          trans_df_highlight <- trans_df[
            trans_df[["trans_name"]] %in% transition_highlight, ,
            drop = FALSE
          ]
          trans_layer <- c(
            trans_layer,
            list(
              geom_curve(
                data = trans_df_highlight,
                mapping = aes(
                  x = .data[["x"]],
                  y = .data[["y"]],
                  xend = .data[["xend"]],
                  yend = .data[["yend"]],
                  linewidth = .data[["size"]]
                ),
                arrow = grid::arrow(
                  angle = transition_arrow_angle,
                  type = transition_arrow_type,
                  length = transition_arrow_length
                ),
                curvature = transition_line_curvature,
                angle = transition_line_angle,
                lineend = "round",
                color = transition_highlight_color,
                alpha = 1,
                inherit.aes = FALSE,
                show.legend = FALSE
              )
            )
          )
        }
      }
      trans_layer <- c(
        trans_layer,
        list(
          scale_linewidth_continuous(
            range = range(transition_size),
            guide = "none"
          ),
          ggnewscale::new_scale("linewidth")
        )
      )
    }
  } else {
    trans_layer <- NULL
  }

  node_layer <- list(
    geom_point(
      data = node,
      aes(x = .data[["x"]], y = .data[["y"]], size = .data[["node_size"]] * 1.2),
      color = "black",
      show.legend = FALSE,
      inherit.aes = FALSE
    ),
    geom_point(
      data = node,
      aes(
        x = .data[["x"]],
        y = .data[["y"]],
        size = .data[["node_size"]],
        color = .data[["node_group"]],
        alpha = .data[["node_alpha"]]
      ),
      inherit.aes = FALSE
    )
  )
  if (!is.null(node_highlight)) {
    node_highlight <- node[
      node[["node_name"]] %in% node_highlight, ,
      drop = FALSE
    ]
    node_layer <- c(
      node_layer,
      list(
        geom_point(
          data = node_highlight,
          aes(x = .data[["x"]], y = .data[["y"]], size = .data[["node_size"]] * 1.3),
          color = node_highlight_color,
          show.legend = FALSE,
          inherit.aes = FALSE
        ),
        geom_point(
          data = node_highlight,
          aes(
            x = .data[["x"]],
            y = .data[["y"]],
            size = .data[["node_size"]],
            color = .data[["node_group"]],
            alpha = .data[["node_alpha"]]
          ),
          inherit.aes = FALSE
        )
      )
    )
  }
  node_layer <- c(
    node_layer,
    list(
      scale_color_manual(
        name = node_group,
        values = palette_colors(
          node[["node_group"]],
          palette = node_palette,
          palcolor = node_palcolor
        ),
        labels = label_use,
        guide = guide_legend(
          title.hjust = 0,
          order = 1,
          override.aes = list(size = 4, alpha = 1)
        )
      ),
      scale_size,
      scale_alpha
    )
  )

  if (isTRUE(label)) {
    if (isTRUE(label_insitu)) {
      node[, "label"] <- node[["node_group"]]
    } else {
      node[, "label"] <- as.numeric(node[["node_group"]])
    }
    if (isTRUE(label_repel)) {
      label_layer <- list(
        geom_point(
          data = node,
          mapping = aes(x = .data[["x"]], y = .data[["y"]]),
          color = label_point_color,
          size = label_point_size,
          inherit.aes = FALSE
        ),
        ggrepel::geom_text_repel(
          data = node,
          aes(x = .data[["x"]], y = .data[["y"]], label = .data[["label"]]),
          fontface = "bold",
          min.segment.length = 0,
          segment.color = label_segment_color,
          point.size = label_point_size,
          max.overlaps = 100,
          force = label_repulsion,
          color = label.fg,
          bg.color = label.bg,
          bg.r = label.bg.r,
          size = label.size,
          inherit.aes = FALSE
        )
      )
    } else {
      label_layer <- list(
        ggrepel::geom_text_repel(
          data = node,
          aes(x = .data[["x"]], y = .data[["y"]], label = .data[["label"]]),
          fontface = "bold",
          min.segment.length = 0,
          segment.color = label_segment_color,
          point.size = NA,
          max.overlaps = 100,
          force = 0,
          color = label.fg,
          bg.color = label.bg,
          bg.r = label.bg.r,
          size = label.size,
          inherit.aes = FALSE
        )
      )
    }
  } else {
    label_layer <- NULL
  }

  lab_layer <- list(labs(
    title = title,
    subtitle = subtitle,
    x = xlab,
    y = ylab
  ))
  theme_layer <- list(
    do.call(theme_use, theme_args) +
      theme(
        aspect.ratio = aspect.ratio,
        legend.position = legend.position,
        legend.direction = legend.direction
      )
  )

  if (isTRUE(return_layer)) {
    return(list(
      edge_layer = edge_layer,
      trans_layer = trans_layer,
      node_layer = node_layer,
      label_layer = label_layer,
      lab_layer = lab_layer,
      theme_layer = theme_layer
    ))
  } else {
    return(
      ggplot() +
        edge_layer +
        trans_layer +
        node_layer +
        label_layer +
        lab_layer +
        theme_layer
    )
  }
}

#' @title Normalize ComplexHeatmap border settings
#'
#' @description
#' Normalize a heatmap border flag, optional colour override, and line width
#' into values accepted by `ComplexHeatmap::Heatmap` and
#' `ComplexHeatmap::Legend`.
#'
#' @param border `NULL`, a logical border flag, or a non-empty colour string.
#' @param palcolor Fallback border colour when `border` is not a colour string.
#' @param color Fallback border colour used by the `gp` and legend helpers.
#' @param size Border line width. Invalid or negative values use `1`.
#'
#' @return A list with `enabled`, `color`, `size`, `gp`, and `legend_border`.
#'
#' @export
heatmap_border_spec <- function(border = NULL, palcolor = "black", size = 1) {
  enabled <- !is.null(border) && !isFALSE(border)
  color <- if (is.character(border) && length(border) > 0L && nzchar(border[[1]])) {
    border[[1]]
  } else if (is.null(palcolor)) {
    "black"
  } else {
    palcolor
  }
  size <- suppressWarnings(as.numeric(size)[1])
  if (!is.finite(size) || size < 0) {
    size <- 1
  }
  list(
    enabled = enabled,
    color = color,
    size = size,
    gp = grid::gpar(col = if (isTRUE(enabled)) color else NA, lwd = size),
    legend_border = if (isTRUE(enabled)) color else FALSE
  )
}

#' @rdname heatmap_border_spec
#' @export
heatmap_border_enabled <- function(border) {
  heatmap_border_spec(border = border)$enabled
}

#' @rdname heatmap_border_spec
#' @export
heatmap_border_color <- function(border, palcolor = "black") {
  heatmap_border_spec(border = border, palcolor = palcolor)$color
}

#' @rdname heatmap_border_spec
#' @export
heatmap_border_size_value <- function(size) {
  heatmap_border_spec(size = size)$size
}

#' @rdname heatmap_border_spec
#' @export
heatmap_border_gp <- function(border, color, size = 1) {
  heatmap_border_spec(border = border, palcolor = color, size = size)$gp
}

#' @rdname heatmap_border_spec
#' @export
heatmap_legend_border <- function(border, color) {
  heatmap_border_spec(border = border, palcolor = color)$legend_border
}

# modified from: https://github.com/davidsjoberg/ggsankey/blob/main/R/sankey.R

# ** Support functions ----------
prepare_params <- function(...) {
  # Prepare aesthics for flow lines
  flow.aes <- list(...)
  removes <- names(flow.aes) |>
    stringr::str_extract_all(pattern = "(?<=flow.).*") |>
    unlist()
  removes2 <- names(flow.aes) |>
    stringr::str_subset(pattern = "node") |>
    unlist()
  flow.aes[c(removes, removes2)] <- NULL
  names(flow.aes) <- names(flow.aes) |>
    stringr::str_replace_all(pattern = "flow.", replacement = "")

  # Prepare aesthics for node boxes
  node.aes <- list(...)
  removes <- names(node.aes) |>
    stringr::str_extract_all(pattern = "(?<=node.).*") |>
    unlist()
  removes2 <- names(node.aes) |>
    stringr::str_subset(pattern = "flow") |>
    unlist()
  node.aes[c(removes, removes2)] <- NULL
  names(node.aes) <- names(node.aes) |>
    stringr::str_replace_all(pattern = "node.", replacement = "")

  return(list(flow.aes, node.aes))
}

find_default_space <- function(.df) {
  .df |>
    dplyr::group_by(.data$n_x) |>
    dplyr::summarise(
      n_groups = dplyr::n_distinct(.data$node),
      freq = sum(.data$freq, na.rm = TRUE)
    ) |>
    dplyr::mutate(v = .data$freq / .data$n_groups / 4) |>
    dplyr::pull(.data$v) |>
    max()
}

sigmoid <- function(x_from, x_to, y_from, y_to, smooth = 5, n = 300) {
  x <- seq(-smooth, smooth, length = n)
  y <- exp(x) / (exp(x) + 1)
  out <- data.frame(
    x = (x + smooth) / (smooth * 2) * (x_to - x_from) + x_from,
    y = y * (y_to - y_from) + y_from
  )
}

#' @title Make a long data frame for sankey plot
#'
#' @description Prepares a 'wide' data frame into a format that `geom_sankey` or `geom_alluvial` understands.
#' Useful to show flows between dimensions in dataset.
#'
#' @param .df a data frame
#' @param ... unquoted columnnames of df that you want to include in the plot.
#' @param value if each row have a weight this weight could be kept by providing column name of weight.
#'
#' @return a longer data frame
#'
#' @export
make_long <- function(.df, ..., value = NULL) {
  if ("..r" %in% names(.df)) {
    stop("The column name '..r' is not allowed")
  }
  .vars <- dplyr::quos(...)

  if (!missing(value)) {
    value_var <- dplyr::enquo(value)
    out <- .df |>
      dplyr::select(!!!.vars, value = !!value_var) |>
      dplyr::mutate(..r = dplyr::row_number()) |>
      tidyr::gather(x, node, -..r, -value) |>
      dplyr::arrange(.data$..r) |>
      dplyr::group_by(.data$..r) |>
      dplyr::mutate(
        next_x = dplyr::lead(.data$x),
        next_node = dplyr::lead(.data$node)
      ) |>
      dplyr::ungroup() |>
      dplyr::select(-..r) |>
      dplyr::relocate(value, .after = dplyr::last_col())
  } else {
    out <- .df |>
      dplyr::select(!!!.vars) |>
      dplyr::mutate(..r = dplyr::row_number()) |>
      tidyr::gather(x, node, -..r) |>
      dplyr::arrange(.data$..r) |>
      dplyr::group_by(.data$..r) |>
      dplyr::mutate(
        next_x = dplyr::lead(.data$x),
        next_node = dplyr::lead(.data$node)
      ) |>
      dplyr::ungroup() |>
      dplyr::select(-..r)
  }

  levels <- unique(out$x)

  out |>
    dplyr::mutate(dplyr::across(c(x, next_x), ~ factor(., levels = levels)))
}


#' @title Themes for sankey plot
#'
#' @description Minimal themes for sankey, alluvial and sankey bump plots
#'
#' @param base_size Base font size, given in pts.
#' @param base_family Base font family.
#' @param base_line_size Base size for line elements.
#' @param base_rect_size Base size for rect elements.
#'
#' @name theme_sankey
#' @aliases theme_alluvial
#' @aliases theme_sankey_bump
#'
#' @export
theme_sankey <-
  function(
    base_size = 11,
    base_family = "",
    base_line_size = base_size / 22,
    base_rect_size = base_size / 22
  ) {
    {
      ggplot2::theme_bw(
        base_size = base_size,
        base_family = base_family,
        base_line_size = base_line_size,
        base_rect_size = base_rect_size
      ) %+replace%
        ggplot2::theme(
          panel.border = ggplot2::element_blank(),
          panel.grid.major = ggplot2::element_blank(),
          panel.grid.minor = ggplot2::element_blank(),
          axis.line = ggplot2::element_line(
            colour = "black",
            linewidth = ggplot2::rel(1)
          ),
          legend.key = ggplot2::element_blank(),
          strip.background = ggplot2::element_rect(
            fill = "white",
            colour = "transparent",
            linewidth = ggplot2::rel(2)
          ),
          complete = TRUE,
          axis.line.y = ggplot2::element_blank(),
          axis.line.x = ggplot2::element_blank(),
          axis.text.y = ggplot2::element_blank(),
          axis.ticks.y = ggplot2::element_blank(),
          axis.ticks.x = ggplot2::element_blank()
        )
    }
  }

#' @rdname theme_sankey
#' @export
theme_alluvial <-
  function(
    base_size = 11,
    base_family = "",
    base_line_size = base_size / 22,
    base_rect_size = base_size / 22
  ) {
    {
      ggplot2::theme_bw(
        base_size = base_size,
        base_family = base_family,
        base_line_size = base_line_size,
        base_rect_size = base_rect_size
      ) %+replace%
        ggplot2::theme(
          panel.border = ggplot2::element_blank(),
          panel.grid.major = ggplot2::element_blank(),
          panel.grid.minor = ggplot2::element_blank(),
          axis.line = ggplot2::element_blank(),
          legend.key = ggplot2::element_blank(),
          strip.background = ggplot2::element_rect(
            fill = "white",
            colour = "transparent",
            linewidth = ggplot2::rel(2)
          ),
          complete = TRUE,
          axis.line.x = ggplot2::element_blank(),
          axis.ticks.x = ggplot2::element_blank()
        )
    }
  }

#' @rdname theme_sankey
#' @export
theme_sankey_bump <-
  function(
    base_size = 11,
    base_family = "",
    base_line_size = base_size / 22,
    base_rect_size = base_size / 22
  ) {
    {
      ggplot2::theme_bw(
        base_size = base_size,
        base_family = base_family,
        base_line_size = base_line_size,
        base_rect_size = base_rect_size
      ) %+replace%
        ggplot2::theme(
          panel.border = ggplot2::element_blank(),
          panel.grid.major.x = ggplot2::element_blank(),
          panel.grid.minor = ggplot2::element_blank(),
          axis.line = ggplot2::element_blank(),
          legend.key = ggplot2::element_blank(),
          strip.background = ggplot2::element_rect(
            fill = "white",
            colour = "transparent",
            linewidth = ggplot2::rel(2)
          ),
          complete = TRUE,
          axis.line.x = ggplot2::element_blank(),
          axis.ticks.x = ggplot2::element_blank(),
          panel.grid.major.y = ggplot2::element_line("gray90")
        )
    }
  }

# FLOW LAYER ---------
StatSankeyFlow <- ggplot2::ggproto(
  "StatSankeyFlow",
  ggplot2::Stat,
  extra_params = c("n_grid", "na.rm", "type", "width", "space", "smooth"),
  setup_data = function(data, params) {
    purrr::map_dfr(
      unique(data$PANEL),
      ~ {
        data <- data |> dplyr::filter(PANEL == .x)

        data <- data |>
          dplyr::mutate(
            dplyr::across(c(x, next_x), ~ as.numeric(.), .names = ("n_{.col}"))
          )

        if (!("value" %in% names(data))) {
          flow_data <- data |>
            dplyr::mutate(group = 1) |>
            dplyr::group_by(n_x, node, n_next_x, next_node) |>
            dplyr::summarise(flow_freq = dplyr::n(), .groups = "keep") |>
            dplyr::ungroup()

          data <- data |>
            dplyr::mutate(group = 1) |>
            dplyr::select(-n_next_x, -next_node, -next_x) |>
            dplyr::group_by_all() |>
            dplyr::summarise(freq = dplyr::n(), .groups = "keep") |>
            dplyr::ungroup()
        } else {
          flow_data <- data |>
            dplyr::mutate(group = 1) |>
            dplyr::group_by(n_x, node, n_next_x, next_node) |>
            dplyr::summarise(
              flow_freq = sum(value, na.rm = TRUE),
              .groups = "keep"
            ) |>
            dplyr::ungroup()

          data <- data |>
            dplyr::mutate(group = 1) |>
            dplyr::select(-n_next_x, -next_node, -next_x) |>
            dplyr::group_by_at(dplyr::vars(dplyr::everything(), -value)) |>
            dplyr::summarise(
              freq = sum(value, na.rm = TRUE),
              ,
              .groups = "keep"
            ) |>
            dplyr::ungroup()
        }

        if (is.null(params$space)) {
          params$space <- find_default_space(data)
        }

        data <- data |>
          dplyr::group_by(n_x) |>
          dplyr::mutate(
            ymax = cumsum(freq) + (dplyr::row_number() - 1) * params$space,
            ymin = ymax - freq
          ) |>
          dplyr::ungroup()

        if (params$type == "sankey") {
          data <- data |>
            dplyr::group_by(n_x) |>
            dplyr::mutate(
              ymin = ymin - max(ymax) / 2,
              ymax = ymax - max(ymax) / 2
            ) |>
            dplyr::ungroup()
        } else if (params$type == "alluvial") {
          data <- data
        }

        data <- data |>
          dplyr::mutate(
            xmin = n_x - params$width / 2,
            xmax = n_x + params$width / 2
          )

        if ("shift" %in% names(data)) {
          data <- data |>
            dplyr::mutate(dplyr::across(dplyr::contains("y"), ~ . + shift))
        }

        df <- data |>
          dplyr::left_join(flow_data, by = c("n_x", "node"))

        flows <- df |>
          dplyr::left_join(
            df |>
              dplyr::select(
                n_x,
                node,
                ymin_end = ymin,
                ymax_end = ymax,
                xmin_end = xmin,
                xmax_end = xmax
              ) |>
              dplyr::distinct(),
            by = c("n_next_x" = "n_x", "next_node" = "node")
          ) |>
          tidyr::drop_na(
            n_x,
            node,
            next_node,
            n_next_x,
            ymax_end,
            ymin_end,
            xmax_end,
            xmin_end
          ) |>
          dplyr::mutate(r = dplyr::row_number()) |>
          dplyr::arrange(n_x, -r) |>
          dplyr::select(-r) |>
          dplyr::group_by(n_x, node) |>
          dplyr::mutate(cum_flow_freq = cumsum(flow_freq) - flow_freq) |>
          dplyr::ungroup() |>
          dplyr::group_by(n_x, n_next_x, node, next_node) |>
          dplyr::mutate(
            flow_start_ymax = ymax - cum_flow_freq,
            flow_start_ymin = flow_start_ymax - flow_freq
          )

        flows <- flows |>
          dplyr::arrange(n_x, n_next_x, next_node) |>
          dplyr::group_by(n_next_x, next_node) |>
          dplyr::mutate(cum_flow_freq_end = cumsum(flow_freq) - flow_freq) |>
          dplyr::mutate(
            flow_end_ymax = ymax_end - cum_flow_freq_end,
            flow_end_ymin = flow_end_ymax - flow_freq
          ) |>
          dplyr::ungroup()

        flows <- flows |>
          dplyr::select(
            -n_x,
            -node,
            -freq,
            -ymax,
            -ymin,
            -xmin,
            -n_next_x,
            -next_node,
            -flow_freq,
            -ymin_end,
            -ymax_end,
            -xmax_end,
            -cum_flow_freq,
            -cum_flow_freq_end
          ) |>
          dplyr::mutate(group = dplyr::row_number())

        flows |>
          dplyr::mutate(smooth = params$smooth) |>
          as.data.frame()
      }
    )
  },
  compute_group = function(data, scales) {
    out1 <- sigmoid(
      data$xmax,
      data$xmin_end,
      data$flow_start_ymax,
      data$flow_end_ymax,
      smooth = data$smooth
    )
    out2 <- sigmoid(
      data$xmin_end,
      data$xmax,
      data$flow_end_ymin,
      data$flow_start_ymin,
      smooth = data$smooth
    )
    dplyr::bind_rows(out1, out2)
  }
)

# FLOW SANKEYBUMP LAYER ---------
StatSankeyBumpFlow <- ggplot2::ggproto(
  "StatSankeyBumpFlow",
  ggplot2::Stat,
  extra_params = c("na.rm", "type", "space", "smooth"),
  setup_data = function(data, params) {
    purrr::map_dfr(
      unique(data$PANEL),
      ~ {
        data <- data |> dplyr::filter(PANEL == .x)

        data <- data |>
          dplyr::mutate(nodes = paste(node, x)) |>
          dplyr::arrange(x, -value) |>
          dplyr::mutate(bbb = dplyr::row_number()) |>
          dplyr::arrange(bbb) |>
          dplyr::mutate(nodes = forcats::fct_reorder(nodes, value, mean)) |>
          dplyr::arrange(node, x) |>
          dplyr::group_by(node) |>
          dplyr::mutate(
            next_x = dplyr::lead(x),
            node = nodes,
            next_node = dplyr::lead(nodes)
          ) |>
          dplyr::ungroup() |>
          dplyr::arrange(x, node)

        data <- data |>
          dplyr::mutate(
            dplyr::across(c(x, next_x), ~ as.numeric(.), .names = ("n_{.col}"))
          )

        if (!("value" %in% names(data))) {
          flow_data <- data |>
            dplyr::mutate(group = 1) |>
            dplyr::group_by(n_x, node, n_next_x, next_node) |>
            dplyr::summarise(flow_freq = dplyr::n(), .groups = "keep") |>
            dplyr::ungroup()

          data <- data |>
            dplyr::mutate(group = 1) |>
            dplyr::select(-n_next_x, -next_node) |>
            dplyr::group_by_all() |>
            dplyr::summarise(freq = dplyr::n(), .groups = "keep") |>
            dplyr::ungroup()
        } else {
          flow_data <- data |>
            dplyr::mutate(group = 1) |>
            dplyr::group_by(n_x, node, n_next_x, next_node) |>
            dplyr::summarise(
              flow_freq = sum(value, na.rm = TRUE),
              .groups = "keep"
            ) |>
            dplyr::ungroup()

          data <- data |>
            dplyr::mutate(group = 1) |>
            dplyr::select(-n_next_x, -next_node) |>
            dplyr::group_by_at(dplyr::vars(dplyr::everything(), -value)) |>
            dplyr::summarise(
              freq = sum(value, na.rm = TRUE),
              .groups = "keep"
            ) |>
            dplyr::ungroup()
        }

        if (is.null(params$space)) {
          params$space <- find_default_space(data)
        }

        data <- data |>
          dplyr::group_by(n_x) |>
          dplyr::arrange(node) |>
          dplyr::mutate(
            ymax = cumsum(freq) + (dplyr::row_number() - 1) * params$space,
            ymin = ymax - freq
          ) |>
          dplyr::ungroup()

        if (params$type == "sankey") {
          data <- data |>
            dplyr::group_by(n_x) |>
            dplyr::mutate(
              ymin = ymin - max(ymax) / 2,
              ymax = ymax - max(ymax) / 2
            ) |>
            dplyr::ungroup()
        } else if (params$type == "alluvial") {
          data <- data
        }

        data <- data |>
          dplyr::mutate(
            xmin = n_x,
            xmax = n_x
          )

        df <- data |>
          dplyr::left_join(flow_data, by = c("n_x", "node"))

        flows <- df |>
          dplyr::left_join(
            df |>
              dplyr::select(
                n_x,
                node,
                ymin_end = ymin,
                ymax_end = ymax,
                xmin_end = xmin,
                xmax_end = xmax,
                flow_freq_end = flow_freq
              ) |>
              dplyr::distinct(),
            by = c("n_next_x" = "n_x", "next_node" = "node")
          ) |>
          tidyr::drop_na(
            n_x,
            node,
            next_node,
            n_next_x,
            ymax_end,
            ymin_end,
            xmax_end,
            xmin_end
          ) |>
          dplyr::mutate(r = dplyr::row_number()) |>
          dplyr::arrange(n_x, -r) |>
          dplyr::select(-r) |>
          dplyr::group_by(n_x, node) |>
          dplyr::mutate(cum_flow_freq = cumsum(flow_freq) - flow_freq) |>
          dplyr::ungroup() |>
          dplyr::group_by(n_x, n_next_x, node, next_node) |>
          dplyr::mutate(
            flow_start_ymax = ymax - cum_flow_freq,
            flow_start_ymin = flow_start_ymax - flow_freq
          )

        flows <- flows |>
          dplyr::arrange(n_x, n_next_x, next_node) |>
          dplyr::group_by(n_next_x, next_node) |>
          dplyr::mutate(
            cum_flow_freq_end = cumsum(flow_freq_end) - flow_freq_end
          ) |>
          dplyr::mutate(
            flow_end_ymax = ymax_end - cum_flow_freq_end,
            flow_end_ymin = flow_end_ymax - flow_freq_end
          ) |>
          dplyr::ungroup()

        flows <- flows |>
          dplyr::select(
            -n_x,
            -node,
            -freq,
            -ymax,
            -ymin,
            -xmin,
            -n_next_x,
            -next_node,
            -flow_freq,
            -ymin_end,
            -ymax_end,
            -xmax_end,
            -cum_flow_freq,
            -cum_flow_freq_end
          ) |>
          dplyr::mutate(group = dplyr::row_number())

        flows |>
          rowwise() |>
          dplyr::mutate(
            ..groupqq = stringr::str_remove(nodes, as.character(x))
          ) |>
          dplyr::ungroup() |>
          dplyr::group_by(..groupqq) |>
          dplyr::mutate(group = dplyr::cur_group_id()) |>
          dplyr::ungroup() |>
          dplyr::select(-..groupqq) |>
          dplyr::mutate(smooth = params$smooth) |>
          as.data.frame()
      }
    )
  },
  compute_group = function(data, scales) {
    out1 <- purrr::map_dfr(
      1:nrow(data),
      ~ {
        datat <- data |> dplyr::slice(.x)
        sigmoid(
          datat$xmax,
          datat$xmin_end,
          datat$flow_start_ymax,
          datat$flow_end_ymax,
          smooth = datat$smooth
        )
      }
    ) |>
      dplyr::arrange(x)
    out2 <- purrr::map_dfr(
      1:nrow(data),
      ~ {
        datat <- data |> dplyr::slice(.x)
        sigmoid(
          datat$xmin_end,
          datat$xmax,
          datat$flow_end_ymin,
          datat$flow_start_ymin,
          smooth = datat$smooth
        )
      }
    ) |>
      dplyr::arrange(-x)

    dplyr::bind_rows(out1, out2)
  }
)

# TEXT LAYER -------
StatSankeyText <- ggplot2::ggproto(
  "StatSankeyText",
  ggplot2::Stat,
  extra_params = c("n_grid", "na.rm", "type", "width", "space"),
  setup_data = function(data, params) {
    purrr::map_dfr(
      unique(data$PANEL),
      ~ {
        data <- data |> dplyr::filter(PANEL == .x)

        data <- data |>
          dplyr::mutate(dplyr::across(
            c(x, next_x),
            ~ as.numeric(.),
            .names = ("n_{.col}")
          ))

        if (!("value" %in% names(data))) {
          data <- data |>
            dplyr::mutate(group = 1) |>
            dplyr::select(-n_next_x, -next_node, -next_x) |>
            dplyr::group_by_all() |>
            dplyr::summarise(freq = dplyr::n(), .groups = "keep") |>
            dplyr::ungroup()
        } else {
          data <- data |>
            dplyr::mutate(group = 1) |>
            dplyr::select(-n_next_x, -next_node) |>
            dplyr::group_by_at(dplyr::vars(dplyr::everything(), -value)) |>
            dplyr::summarise(
              freq = sum(value, na.rm = TRUE),
              .groups = "keep"
            ) |>
            dplyr::ungroup()
        }

        if (is.null(params$space)) {
          params$space <- find_default_space(data)
        }

        data <- data |>
          dplyr::group_by(n_x) |>
          dplyr::mutate(
            ymax = cumsum(freq) + (dplyr::row_number() - 1) * params$space,
            ymin = ymax - freq
          ) |>
          dplyr::ungroup()

        if (params$type == "sankey") {
          data <- data |>
            dplyr::group_by(n_x) |>
            dplyr::mutate(
              ymin = ymin - max(ymax) / 2,
              ymax = ymax - max(ymax) / 2
            ) |>
            dplyr::ungroup()
        } else if (params$type == "alluvial") {
          data <- data
        }

        data <- data |>
          dplyr::mutate(
            xmin = n_x - params$width / 2,
            xmax = n_x + params$width / 2
          )

        data <- data |>
          dplyr::mutate(
            x = n_x,
            y = ymin + (ymax - ymin) / 2
          )

        if ("shift" %in% names(data)) {
          data <- data |>
            dplyr::mutate(dplyr::across(dplyr::contains("y"), ~ . + shift))
        }

        return(as.data.frame(data))
      }
    )
  },
  compute_group = function(data, scales) {
    data
  }
)

# NODE LAYER -------
StatSankeyNode <- ggplot2::ggproto(
  "StatSankeyNode",
  ggplot2::Stat,
  extra_params = c("n_grid", "na.rm", "type", "width", "space", "smooth"),
  setup_data = function(data, params) {
    purrr::map_dfr(
      unique(data$PANEL),
      ~ {
        data <- data |> dplyr::filter(PANEL == .x)
        data <- data |>
          dplyr::mutate(dplyr::across(
            c(x, next_x),
            ~ as.numeric(.),
            .names = ("n_{.col}")
          ))

        if (!("value" %in% names(data))) {
          data <- data |>
            dplyr::mutate(group = 1) |>
            dplyr::select(-n_next_x, -next_node, -next_x) |>
            dplyr::group_by_all() |>
            dplyr::summarise(freq = dplyr::n(), .groups = "keep") |>
            dplyr::ungroup()
        } else {
          data <- data |>
            dplyr::mutate(group = 1) |>
            dplyr::select(-n_next_x, -next_node, -next_x) |>
            dplyr::group_by_at(dplyr::vars(dplyr::everything(), -value)) |>
            dplyr::summarise(
              freq = sum(value, na.rm = TRUE),
              .groups = "keep"
            ) |>
            dplyr::ungroup()
        }

        if (is.null(params$space)) {
          params$space <- find_default_space(data)
        }

        data <- data |>
          dplyr::group_by(n_x) |>
          dplyr::mutate(
            ymax = cumsum(freq) + (dplyr::row_number() - 1) * params$space,
            ymin = ymax - freq
          ) |>
          dplyr::ungroup()

        if (params$type == "sankey") {
          data <- data |>
            dplyr::group_by(n_x) |>
            dplyr::mutate(
              ymin = ymin - max(ymax) / 2,
              ymax = ymax - max(ymax) / 2
            ) |>
            dplyr::ungroup()
        } else if (params$type == "alluvial") {
          data <- data
        }

        data <- data |>
          dplyr::mutate(
            xmin = n_x - params$width / 2,
            xmax = n_x + params$width / 2
          )

        if ("shift" %in% names(data)) {
          data <- data |>
            dplyr::mutate(dplyr::across(dplyr::contains("y"), ~ . + shift))
        }

        return(as.data.frame(data))
      }
    )
  },
  compute_group = function(data, scales) {
    data
  }
)

#' @title geom_sankey
#'
#' @description Creates a sankey plot which visualize flows between nodes.
#' Each observation needs to have a `x` aesthetic as well as a `next_x` column which declares where that observation should flow.
#' Also each observation should have a `node` and a `next_node` aesthetic which provide information about which group in the y-direction.
#' By default each row of the data frame is counted to calculate the size of flows. A manual flow value can be added with the `value` aesthetic.
#'
#' @param mapping Provide you own mapping. Both x and y need to be numeric.
#' @param data Provide you own data.
#' @param position Change position.
#' @param na.rm Remove missing values.
#' @param show.legend Show legend in plot.
#' @param space Space between nodes in the y-direction.
#' @param type Either `sankey` or `alluvial`.
#' @param width Width of nodes.
#' @param smooth How much smooth should the curve have? More means steeper curve.
#' @param inherit.aes Should the geom inherit aesthetics.
#' @param ... Other arguments to be passed to the geom.
#'
#' @section Aesthetics:
#' geom_sankey understand the following aesthetics (required aesthetics are in
#' bold):
#'
#' - **x0**
#' - **y0**
#' - **a**
#' - **b**
#' - **angle**
#' - m1
#' - m2
#' - color
#' - fill
#' - size
#' - linetype
#' - alpha
#' - lineend
#'
#' @return A ggplot layer.
#'
#' @export
#' @examples
#' dat <- data.frame(
#'   Group = c("A", "A", "B", "B", "C"),
#'   Type = c("X", "Y", "X", "Y", "X")
#' )
#' long <- make_long(dat, Group, Type)
#' ggplot2::ggplot(
#'   long,
#'   ggplot2::aes(
#'     x = x,
#'     next_x = next_x,
#'     node = node,
#'     next_node = next_node,
#'     fill = node
#'   )
#' ) +
#'   geom_sankey() +
#'   theme_sankey()
geom_sankey <- function(
  mapping = NULL,
  data = NULL,
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  space = NULL,
  type = "sankey",
  width = .1,
  smooth = 8,
  inherit.aes = TRUE,
  ...
) {
  params_list <- prepare_params(...)

  list(
    flow = ggplot2::layer(
      stat = StatSankeyFlow,
      data = data,
      mapping = mapping,
      geom = "polygon",
      position = position,
      show.legend = show.legend,
      inherit.aes = inherit.aes,
      params = purrr::flatten(
        list(
          na.rm = na.rm,
          width = width,
          space = space,
          smooth = smooth,
          type = type,
          params_list[[1]]
        )
      )
    ),
    node = ggplot2::layer(
      stat = StatSankeyNode,
      data = data,
      mapping = mapping,
      geom = ggplot2::GeomRect,
      position = position,
      show.legend = show.legend,
      inherit.aes = inherit.aes,
      params = purrr::flatten(
        list(
          na.rm = na.rm,
          width = width,
          space = space,
          smooth = smooth,
          type = type,
          params_list[[2]]
        )
      )
    )
  )
}

#' @title geom_sankey_label
#'
#' @name geom_sankey_label
#' @aliases geom_sankey_text
#'
#' @description Creates centered labels or text in nodes of your sankey plot. Needs to have the exact same aesthetics as the call to `geom_sankey` to work.
#'
#' @inheritParams geom_sankey
#'
#' @return A ggplot layer.
#'
#' @rdname geom_sankey_label
#' @export
geom_sankey_label <- function(
  mapping = NULL,
  data = NULL,
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  space = NULL,
  type = "sankey",
  width = .1,
  inherit.aes = TRUE,
  ...
) {
  # Prepare aesthetics for label
  label.aes <- list(...)

  list(
    label = ggplot2::layer(
      stat = StatSankeyText,
      data = data,
      mapping = mapping,
      geom = "label",
      position = position,
      show.legend = show.legend,
      inherit.aes = inherit.aes,
      params = purrr::flatten(
        list(
          na.rm = na.rm,
          width = width,
          space = space,
          type = type,
          label.aes
        )
      )
    )
  )
}

#' @rdname geom_sankey_label
#' @export
geom_sankey_text <- function(
  mapping = NULL,
  data = NULL,
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  space = NULL,
  type = "sankey",
  width = .1,
  inherit.aes = TRUE,
  ...
) {
  label.aes <- list(...)

  list(
    label = ggplot2::layer(
      stat = StatSankeyText,
      data = data,
      mapping = mapping,
      geom = "text",
      position = position,
      show.legend = show.legend,
      inherit.aes = inherit.aes,
      params = purrr::flatten(
        list(
          na.rm = na.rm,
          width = width,
          space = space,
          type = type,
          label.aes
        )
      )
    )
  )
}

#' @title geom_alluvial
#'
#' @description Creates an alluvial plot which visualize flows between nodes.
#' Each observation needs to have a `x` aesthetic as well as a `next_x` column which declares where that observation should flow.
#' Also each observation should have a `node` and a `next_node` aesthetic which provide information about which group in the y-direction.
#'
#' @inheritParams geom_sankey
#'
#' @return A ggplot layer.
#'
#' @export
geom_alluvial <- function(
  mapping = NULL,
  data = NULL,
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  space = 0,
  width = .1,
  smooth = 8,
  inherit.aes = TRUE,
  ...
) {
  geom_sankey(
    mapping = mapping,
    data = data,
    position = position,
    na.rm = na.rm,
    show.legend = show.legend,
    space = space,
    width = width,
    smooth = smooth,
    type = "alluvial",
    inherit.aes = inherit.aes,
    ...
  )
}

#' @title geom_alluvial_label
#' @name geom_alluvial_label
#' @aliases geom_alluvial_text
#'
#' @description Creates centered labels or text in nodes of your alluvial plot.
#' Needs to have the exact same aesthetics as the call to `geom_alluvial` to work.
#'
#' @inheritParams geom_sankey
#'
#' @details Other important arguments are;
#' `space` which provides the space between nodes in the y-direction;
#' `shift` which shifts nodes in the y-direction.
#'
#' @return A ggplot layer.
#'
#' @rdname geom_alluvial_label
#' @export
geom_alluvial_text <- function(
  mapping = NULL,
  data = NULL,
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  space = 0,
  width = .1,
  inherit.aes = TRUE,
  ...
) {
  geom_sankey_text(
    mapping = mapping,
    data = data,
    position = position,
    na.rm = na.rm,
    show.legend = show.legend,
    space = space,
    width = width,
    type = "alluvial",
    inherit.aes = inherit.aes,
    ...
  )
}

#' @rdname geom_alluvial_label
#' @export
geom_alluvial_label <- function(
  mapping = NULL,
  data = NULL,
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  space = 0,
  width = .1,
  inherit.aes = TRUE,
  ...
) {
  geom_sankey_label(
    mapping = mapping,
    data = data,
    position = position,
    na.rm = na.rm,
    show.legend = show.legend,
    space = space,
    width = width,
    type = "alluvial",
    inherit.aes = inherit.aes,
    ...
  )
}

#' @title geom_sankey_bump
#'
#' @description Creates a sankey bump plot which visualize flows between nodes.
#' Each observation needs to have a `x` aesthetic as well as a `next_x` column which declares where that observation should flow.
#' Also, each observation should have a `node` and a `next_node` aesthetic which provide information about which group in the y-direction.
#'
#' @inheritParams geom_sankey
#'
#' @details Other important arguments are;
#' `space` which provides the space between nodes in the y-direction;
#' `shift` which shifts nodes in the y-direction.
#'
#' @return A ggplot layer.
#'
#' @export
geom_sankey_bump <- function(
  mapping = NULL,
  data = NULL,
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  smooth = 8,
  type = "sankey",
  inherit.aes = TRUE,
  ...
) {
  params_list <- prepare_params(...)

  list(
    flow = ggplot2::layer(
      stat = StatSankeyBumpFlow,
      data = data,
      mapping = mapping,
      geom = "polygon",
      position = position,
      show.legend = show.legend,
      inherit.aes = inherit.aes,
      params = purrr::flatten(list(
        na.rm = na.rm,
        type = type,
        smooth = smooth,
        params_list[[1]]
      ))
    )
  )
}

#' @title The default theme for scop plot function.
#'
#' @md
#' @param aspect.ratio Aspect ratio of the panel.
#' @param base_size Base font size
#' @param ... Arguments passed to the [ggplot2::theme].
#'
#' @return A ggplot2 theme object (class `theme`, `gg`).
#'
#' @export
#'
#' @examples
#' library(ggplot2)
#' p <- ggplot(
#'   data = mtcars,
#'   aes(x = wt, y = mpg, colour = factor(cyl))
#' ) +
#'   geom_point()
#' p + theme_this()
theme_this <- function(
  aspect.ratio = NULL,
  base_size = 12,
  ...
) {
  text_size_scale <- base_size / 12
  args1 <- list(
    aspect.ratio = aspect.ratio,
    text = element_text(
      size = 12 * text_size_scale,
      color = "black"
    ),
    plot.title = element_text(
      size = 14 * text_size_scale,
      colour = "black", vjust = 1
    ),
    plot.subtitle = element_text(
      size = 13 * text_size_scale,
      hjust = 0,
      margin = margin(b = 3)
    ),
    plot.background = element_rect(
      fill = "white",
      color = "white"
    ),
    axis.line = element_blank(),
    axis.title = element_text(
      size = 13 * text_size_scale,
      colour = "black"
    ),
    axis.text = element_text(
      size = 12 * text_size_scale,
      colour = "black"
    ),
    strip.text = element_text(
      size = 12.5 * text_size_scale,
      colour = "black",
      hjust = 0.5,
      margin = margin(3, 3, 3, 3)
    ),
    strip.background = element_rect(
      fill = "transparent", linetype = 0
    ),
    strip.switch.pad.grid = grid::unit(-1, "pt"),
    strip.switch.pad.wrap = grid::unit(-1, "pt"),
    strip.placement = "outside",
    legend.title = element_text(
      size = 12 * text_size_scale,
      colour = "black",
      hjust = 0
    ),
    legend.text = element_text(
      size = 11 * text_size_scale,
      colour = "black"
    ),
    legend.key = element_rect(
      fill = "transparent",
      color = "transparent"
    ),
    legend.key.size = grid::unit(10, "pt"),
    legend.background = element_blank(),
    panel.background = element_rect(
      fill = "white",
      color = "white"
    ),
    panel.border = element_rect(
      fill = "transparent",
      colour = "black",
      linewidth = 1
    )
  )
  args2 <- as.list(match.call())[-1]
  call_envir <- parent.frame(1)
  args2 <- lapply(
    args2, function(arg) {
      if (is.symbol(arg)) {
        eval(arg, envir = call_envir)
      } else if (is.call(arg)) {
        eval(arg, envir = call_envir)
      } else {
        arg
      }
    }
  )
  for (n in names(args2)) {
    args1[[n]] <- args2[[n]]
  }
  args <- args1[names(args1) %in% methods::formalArgs(theme)]
  out <- do.call(
    what = theme,
    args = args
  )
  return(out)
}

#' @title Blank theme
#'
#' @description This function creates a theme with all elements blank except for axis lines and labels.
#' It can optionally add coordinate axes in the plot.
#'
#' @md
#' @param add_coord Whether to add coordinate arrows. Default is `TRUE`.
#' @param xlen_npc The length of the x-axis arrow in "npc".
#' @param ylen_npc The length of the y-axis arrow in "npc".
#' @param xlab The label of the x-axis.
#' @param ylab The label of the y-axis.
#' @param lab_size The size of the axis labels.
#' @param ... Arguments passed to the [ggplot2::theme].
#'
#' @return A list containing ggplot2 theme objects and annotation objects.
#' If `add_coord` is `TRUE`, returns a list with coordinate arrows;
#' otherwise returns a list with theme only.
#'
#' @export
#'
#' @examples
#' library(ggplot2)
#' p <- ggplot(mtcars, aes(x = wt, y = mpg, colour = factor(cyl))) +
#'   geom_point()
#' p + theme_blank()
#' p + theme_blank(xlab = "x-axis", ylab = "y-axis", lab_size = 16)
theme_blank <- function(
  add_coord = TRUE,
  xlen_npc = 0.15,
  ylen_npc = 0.15,
  xlab = "",
  ylab = "",
  lab_size = 12, ...
) {
  args1 <- list(
    panel.border = element_blank(),
    panel.grid = element_blank(),
    axis.title = element_blank(),
    axis.line = element_blank(),
    axis.ticks = element_blank(),
    axis.text = element_blank(),
    legend.background = element_blank(),
    legend.box.margin = margin(0, 0, 0, 0),
    legend.margin = margin(0, 0, 0, 0),
    legend.key.size = grid::unit(10, "pt"),
    plot.margin = margin(
      lab_size + 10,
      lab_size + 10,
      lab_size + 10,
      lab_size + 10,
      unit = "points"
    )
  )
  args2 <- as.list(match.call())[-1]
  call_envir <- parent.frame(1)
  args2 <- lapply(
    args2, function(arg) {
      if (is.symbol(arg)) {
        eval(arg, envir = call_envir)
      } else if (is.call(arg)) {
        eval(arg, envir = call_envir)
      } else {
        arg
      }
    }
  )
  for (n in names(args2)) {
    args1[[n]] <- args2[[n]]
  }
  args <- args1[names(args1) %in% methods::formalArgs(theme)]
  out <- do.call(
    what = theme,
    args = args
  )
  if (isTRUE(add_coord)) {
    g <- grid::grobTree(
      grid::gList(
        grid::linesGrob(
          x = grid::unit(c(0, xlen_npc), "npc"),
          y = grid::unit(c(0, 0), "npc"),
          arrow = grid::arrow(
            length = grid::unit(0.02, "npc")
          ),
          gp = grid::gpar(lwd = 2)
        ),
        grid::textGrob(
          label = xlab,
          x = grid::unit(0, "npc"),
          y = grid::unit(-0.55, "lines"),
          vjust = 1,
          hjust = 0,
          gp = grid::gpar(fontsize = lab_size)
        ),
        grid::linesGrob(
          x = grid::unit(c(0, 0), "npc"),
          y = grid::unit(c(0, ylen_npc), "npc"),
          arrow = grid::arrow(length = grid::unit(0.02, "npc")),
          gp = grid::gpar(lwd = 2)
        ),
        grid::textGrob(
          label = ylab,
          x = grid::unit(-0.85, "lines"),
          y = grid::unit(ylen_npc, "npc") + grid::unit(0.25, "lines"),
          vjust = 0.5,
          hjust = 0.5,
          rot = 90,
          gp = grid::gpar(fontsize = lab_size)
        )
      )
    )
    return(list(
      list(ggplot2::annotation_custom(g)),
      list(theme_this() + out),
      list(ggplot2::coord_cartesian(clip = "off"))
    ))
  } else {
    return(list(
      list(theme_this() + out)
    ))
  }
}

#' @title A list of palettes for use in data visualization
#'
#' @concept data
#' @name palette_list
NULL

#' @title Chinese traditional colors dataset
#'
#' @description
#' A dataset containing optimized traditional Chinese colors.
#' These colors are extracted from those books:
#' \itemize{
#' \item{Chinese Traditional Colors - Color Aesthetics in the Forbidden City (ISBN: 9787521716054)}
#' \item{Chinese Beautiful Colors - The Most Chinese Culture Vol.3 (ISBN: 9781672897198)}
#' \item{Chinese Colors (ISBN: 9787558016479)}
#' \item{Chinese Journal of Chromatography (ISSN 1000-8713)}
#' \item{Chinese Color Atlas}
#' }
#'
#' Thanks to the author of the \href{https://book.douban.com/review/14384195/}{blog} for providing the data.
#'
#' @concept data
#' @name chinese_colors
#'
#' @examples
#' data(chinese_colors)
#' color_sets <- attr(chinese_colors, "color_sets")
#' show_palettes(
#'   list(
#'     color_sets$Chinese,
#'     color_sets$ChineseSet8,
#'     color_sets$ChineseSet16,
#'     color_sets$ChineseSet32
#'   )
#' )
#'
#' # Use ChineseColors class
#' cc <- ChineseColors()
#' cc$visual_colors(
#'   loc_range = c(1, 180),
#'   num_per_row = 30,
#'   title = "Chinese Traditional Colors",
#'   name_type = "chinese"
#' )
NULL

#' @title Set major grid lines on a ggplot
#'
#' @description
#' Add or remove major grid lines from a ggplot object.
#'
#' @md
#' @param plot A ggplot object.
#' @param grid_major Whether to show major grid lines. Default is `TRUE`.
#' @param grid_major_colour Grid line colour. Default is `"grey80"`.
#' @param grid_major_linetype Grid line type. Default is `2` (dashed).
#' @param grid_major_linewidth Grid line width. Default is `0.3`.
#'
#' @return A ggplot object with modified `panel.grid.major` theme element.
#'
#' @export
#'
#' @examples
#' library(ggplot2)
#' p <- ggplot(mtcars, aes(wt, mpg)) + geom_point()
#' major_grid(p, grid_major = TRUE)
major_grid <- function(
  plot,
  grid_major = TRUE,
  grid_major_colour = "grey80",
  grid_major_linetype = 2,
  grid_major_linewidth = 0.3
) {
  grid_major_element <- if (isTRUE(grid_major)) {
    ggplot2::element_line(
      colour = grid_major_colour,
      linetype = grid_major_linetype,
      linewidth = grid_major_linewidth
    )
  } else {
    ggplot2::element_blank()
  }
  plot + ggplot2::theme(panel.grid.major = grid_major_element)
}

#' Shorten and offset the segment
#'
#' This function takes a data frame representing segments in a plot and shortens
#' and offsets them based on the provided arguments.
#'
#' @param data A data frame containing the segments.
#' It should have columns 'x', 'y', 'xend', and 'yend' representing the start and end points of each segment.
#' @param shorten_start The amount to shorten the start of each segment by.
#' @param shorten_end The amount to shorten the end of each segment by.
#' @param offset The amount to offset each segment by.
#'
#' @return The modified data frame with the shortened and offset segments.
#'
#' @export
#'
#' @examples
#' library(ggplot2)
#' temp_nodes <- data.frame(
#'   "x" = c(10, 40),
#'   "y" = c(10, 30)
#' )
#' data <- data.frame(
#'   "x" = c(10, 40),
#'   "y" = c(10, 30),
#'   "xend" = c(40, 10),
#'   "yend" = c(30, 10)
#' )
#'
#' ggplot(temp_nodes, aes(x = x, y = y)) +
#'   geom_point(size = 12) +
#'   xlim(0, 50) +
#'   ylim(0, 50) +
#'   geom_segment(
#'     data = data,
#'     aes(x = x, xend = xend, y = y, yend = yend)
#'   )
#'
#' ggplot(temp_nodes, aes(x = x, y = y)) +
#'   geom_point(size = 12) +
#'   xlim(0, 50) +
#'   ylim(0, 50) +
#'   geom_segment(
#'     data = segements_df(
#'       data,
#'       shorten_start = 2,
#'       shorten_end = 3,
#'       offset = 1
#'     ),
#'     aes(x = x, xend = xend, y = y, yend = yend)
#'   )
segements_df <- function(
  data,
  shorten_start,
  shorten_end, offset
) {
  data$dx <- data$xend - data$x
  data$dy <- data$yend - data$y
  data$dist <- sqrt(data$dx^2 + data$dy^2)
  data$px <- data$dx / data$dist
  data$py <- data$dy / data$dist

  data$x <- data$x + data$px * shorten_start
  data$y <- data$y + data$py * shorten_start
  data$xend <- data$xend - data$px * shorten_end
  data$yend <- data$yend - data$py * shorten_end
  data$x <- data$x - data$py * offset
  data$xend <- data$xend - data$py * offset
  data$y <- data$y + data$px * offset
  data$yend <- data$yend + data$px * offset

  data
}

#' @title Extract grobs from a list
#'
#' @description
#' Extract grobs from a named list of grobs based on the specified x and y indices.
#'
#' @md
#' @param vlnplots A named list of grobs.
#' @param x_nm A character vector of names for the x dimension.
#' @param y_nm A character vector of names for the y dimension.
#' @param x An integer index for the x dimension.
#' @param y An integer index for the y dimension.
#'
#' @return The extracted grob(s).
#'
#' @export
extractgrobs <- function(vlnplots, x_nm, y_nm, x, y) {
  grobs <- vlnplots[paste0(x_nm[x], ":", y_nm[y])]
  if (length(grobs) == 1) {
    grobs <- grobs[[1]]
  }
  grobs
}

#' @title Draw grobs at specified positions
#'
#' @description
#' Draw a list of grobs at specified positions with given widths and heights.
#'
#' @md
#' @param groblist A grob or a list of grobs to draw.
#' @param x A numeric vector of x positions for each grob.
#' @param y A numeric vector of y positions for each grob.
#' @param width A numeric vector of widths for each grob.
#' @param height A numeric vector of heights for each grob.
#'
#' @return No return value, called for side effects (drawing grobs).
#'
#' @export
grid_draw <- function(groblist, x, y, width, height) {
  if (grid::is.grob(groblist)) {
    groblist <- list(groblist)
  }
  for (i in seq_along(groblist)) {
    groblist[[i]]$vp <- grid::viewport(
      x = x[i],
      y = y[i],
      width = width[i],
      height = height[i]
    )
    grid::grid.draw(groblist[[i]])
  }
}

#' @title Standardize data by rows
#'
#' @description
#' Standardize each row of a data matrix by subtracting the mean and dividing by the standard deviation.
#'
#' @md
#' @param data A matrix or data frame to standardize.
#'
#' @return The standardized data with the same structure as input.
#'
#' @export
standardise <- function(data) {
  data[] <- t(apply(data, 1, scale))
  data
}

#' @title Estimate the fuzzifier parameter m
#'
#' @description
#' Estimate the fuzzifier parameter m for fuzzy clustering based on the data dimensions.
#'
#' @md
#' @param data A matrix or data frame.
#'
#' @return The estimated fuzzifier parameter m.
#'
#' @export
mestimate <- function(data) {
  N <- nrow(data)
  D <- ncol(data)
  m.sj <- 1 +
    (1418 / N + 22.05) * D^(-2) +
    (12.33 / N + 0.243) *
      D^(-0.0406 * log(N) - 0.1134)
  m.sj
}

#' @title Adjust graph layout to avoid node overlaps
#'
#' @description
#' Adjust the layout of a graph to prevent node overlaps by considering node widths and heights.
#'
#' @md
#' @param graph An igraph graph object.
#' @param layout A matrix with two columns representing the initial layout coordinates.
#' @param width A numeric vector of node widths.
#' @param height The height constraint for nodes.
#' Default is `2`.
#' @param scale The scaling factor for the layout.
#' Default is `100`.
#' @param iter The number of iterations for the adjustment algorithm.
#' Default is `100`.
#'
#' @return A matrix with adjusted layout coordinates.
#'
#' @export
adjustlayout <- function(
  graph,
  layout,
  width,
  height = 2,
  scale = 100,
  iter = 100
) {
  w <- width / 2
  layout[, 1] <- layout[, 1] / diff(range(layout[, 1])) * scale
  layout[, 2] <- layout[, 2] / diff(range(layout[, 2])) * scale

  adjusted <- c()
  for (v in order(igraph::degree(graph), decreasing = TRUE)) {
    adjusted <- c(adjusted, v)
    neighbors <- as.numeric(
      igraph::neighbors(
        graph, igraph::V(graph)[v]
      )
    )
    neighbors <- setdiff(neighbors, adjusted)
    x <- layout[v, 1]
    y <- layout[v, 2]
    r <- w[v]
    for (neighbor in neighbors) {
      nx <- layout[neighbor, 1]
      ny <- layout[neighbor, 2]
      ndist <- sqrt((nx - x)^2 + (ny - y)^2)
      nr <- w[neighbor]
      expect <- r + nr
      if (ndist < expect) {
        dx <- (x - nx) * (expect - ndist) / ndist
        dy <- (y - ny) * (expect - ndist) / ndist
        layout[neighbor, 1] <- nx - dx
        layout[neighbor, 2] <- ny - dy
        adjusted <- c(adjusted, neighbor)
      }
    }
  }

  for (i in seq_len(iter)) {
    dist_obj <- stats::dist(layout, method = "euclidean")
    dist_matrix <- as_matrix(dist_obj)
    nearest_neighbors <- apply(
      dist_matrix,
      2,
      function(x) which(x == min(x[x > 0])),
      simplify = FALSE
    )

    for (v in sample(seq_len(nrow(layout)))) {
      neighbors <- unique(nearest_neighbors[[v]])
      x <- layout[v, 1]
      y <- layout[v, 2]
      r <- w[v]
      for (neighbor in neighbors) {
        nx <- layout[neighbor, 1]
        ny <- layout[neighbor, 2]
        nr <- w[neighbor]
        if (abs(nx - x) < (r + nr) && abs(ny - y) < height) {
          dx <- r + nr - (nx - x)
          dy <- height - (ny - y)
          if (sample(c(1, 0), 1) == 1) {
            dx <- 0
          } else {
            dy <- 0
          }
          layout[neighbor, 1] <- nx - dx
          layout[neighbor, 2] <- ny - dy
        }
      }
    }
  }
  return(layout)
}

#' @title Annotate plot quadrants with percentages
#'
#' @param plot A `ggplot` object.
#' @param x Name of the x variable in `plot$data`.
#' @param y Name of the y variable in `plot$data`.
#' @param cutoffs Cut points for x and y axes. Can be a numeric vector of length
#'   1 or 2, or a list with the x and y cut points.
#' @param line_color Quadrant line color.
#' @param line_type Quadrant line type.
#' @param line_width Quadrant line width.
#' @param label_size Label size.
#' @param group Optional grouping column used to compute percentages within each
#'   group.
#'
#' @return A `ggplot` object.
#'
#' @export
#' @examples
#' df <- data.frame(
#'   x = c(0.1, 0.2, 0.8, 0.9),
#'   y = c(0.1, 0.8, 0.2, 0.9)
#' )
#' p <- ggplot2::ggplot(df, ggplot2::aes(x = x, y = y)) +
#'   ggplot2::geom_point()
#' p
#'
#' annotate_quadrants(p, x = "x", y = "y", cutoffs = 0.5)
annotate_quadrants <- function(
  plot,
  x,
  y,
  cutoffs,
  line_color = "grey30",
  line_type = "solid",
  line_width = 0.5,
  label_size = 3,
  group = NULL
) {
  cutoffs_x <- NULL
  cutoffs_y <- NULL
  if (is.list(cutoffs)) {
    cutoffs_x <- cutoffs[[1]]
    cutoffs_y <- cutoffs[[2]]
  } else if (is.numeric(cutoffs)) {
    if (length(cutoffs) == 1) {
      cutoffs_x <- cutoffs
      cutoffs_y <- cutoffs
    } else {
      cutoffs_x <- cutoffs[1]
      cutoffs_y <- cutoffs[2]
    }
  }

  if (!is.null(cutoffs_x)) {
    plot <- plot +
      ggplot2::geom_vline(
        xintercept = cutoffs_x,
        linetype = line_type,
        color = line_color,
        linewidth = line_width
      )
  }
  if (!is.null(cutoffs_y)) {
    plot <- plot +
      ggplot2::geom_hline(
        yintercept = cutoffs_y,
        linetype = line_type,
        color = line_color,
        linewidth = line_width
      )
  }

  plot_data <- plot$data
  plot_limits <- ggplot2::ggplot_build(plot)$layout$panel_params[[1]]

  x_breaks <- c(plot_limits$x.range[1], cutoffs_x, plot_limits$x.range[2])
  x_breaks <- unique(sort(x_breaks))
  y_breaks <- c(plot_limits$y.range[1], cutoffs_y, plot_limits$y.range[2])
  y_breaks <- unique(sort(y_breaks))

  x_breaks <- x_breaks[
    x_breaks >= plot_limits$x.range[1] & x_breaks <= plot_limits$x.range[2]
  ]
  y_breaks <- y_breaks[
    y_breaks >= plot_limits$y.range[1] & y_breaks <= plot_limits$y.range[2]
  ]

  x_pos <- (utils::head(x_breaks, -1) + utils::tail(x_breaks, -1)) / 2
  y_pos <- (utils::head(y_breaks, -1) + utils::tail(y_breaks, -1)) / 2

  plot_data$x_cat <- as.integer(cut(
    plot_data[[x]],
    breaks = x_breaks,
    include.lowest = TRUE
  ))
  plot_data$y_cat <- as.integer(cut(
    plot_data[[y]],
    breaks = y_breaks,
    include.lowest = TRUE
  ))

  if (is.null(group)) {
    counts <- as.data.frame(
      table(plot_data[, c("x_cat", "y_cat")]),
      stringsAsFactors = FALSE
    )
    counts$value <- to_percent(counts$Freq)
  } else {
    counts <- as.data.frame(
      table(plot_data[, c(group, "x_cat", "y_cat")]),
      stringsAsFactors = FALSE
    )
    counts_list <- split(counts, counts[[group]])
    counts <- do.call(
      rbind,
      lapply(counts_list, function(df) {
        df$value <- to_percent(df$Freq)
        df
      })
    )
  }

  counts$x_cat <- as.integer(counts$x_cat)
  counts$y_cat <- as.integer(counts$y_cat)

  annot_df <- counts[counts$Freq > 0, , drop = FALSE]
  if (nrow(annot_df) == 0) {
    return(plot)
  }

  annot_df$x <- x_pos[annot_df$x_cat]
  annot_df$y <- y_pos[annot_df$y_cat]
  annot_df$label <- paste0(round(annot_df$value, 1), "%")

  plot +
    ggplot2::geom_text(
      data = annot_df,
      ggplot2::aes(x = x, y = y, label = label),
      size = label_size
    )
}

#' @title Clip values to a symmetric range
#'
#' @param data A data frame.
#' @param value_col Name of the numeric column to clip.
#'
#' @return A list containing the clipped data frame and clipping limits.
#'
#' @export
#' @examples
#' df <- data.frame(avg_log2FC = c(-10, -2, 0, 2, 10))
#' clip_symmetric_range(df)
clip_symmetric_range <- function(data, value_col = "avg_log2FC") {
  values <- data[[value_col]][is.finite(data[[value_col]])]
  if (length(values) == 0) {
    return(list(data = data, limits = c(-1, 1)))
  }

  upper <- stats::quantile(values, c(0.99, 1))
  lower <- stats::quantile(values, c(0.01, 0))
  upper <- ifelse(upper[1] > 0, upper[1], upper[2])
  lower <- ifelse(lower[1] < 0, lower[1], lower[2])

  if (upper > 0 && lower < 0) {
    value_range <- min(abs(c(upper, lower)), na.rm = TRUE)
    upper <- value_range
    lower <- -value_range
  }

  data[data[[value_col]] > upper, value_col] <- upper
  data[data[[value_col]] < lower, value_col] <- lower

  list(data = data, limits = c(lower, upper))
}

#' @title Jitter highlighted points deterministically
#'
#' @param data A data frame containing `x`, `y`, and `border` columns.
#' @param jitter_width Width of the x jitter.
#' @param jitter_height Height of the y jitter.
#' @param seed Numeric value used to offset the deterministic sequence.
#'
#' @return A data frame with `x_plot` and `y_plot` columns.
#'
#' @export
#' @examples
#' df <- data.frame(
#'   x = c(0, 1),
#'   y = c(0, 1),
#'   border = c(FALSE, TRUE)
#' )
#' jitter_highlighted_points(df, jitter_width = 0.2, jitter_height = 0.2)
jitter_highlighted_points <- function(
  data,
  jitter_width = 0.2,
  jitter_height = 0.2,
  seed = 11
) {
  data[, "x_plot"] <- data[, "x"]
  data[, "y_plot"] <- data[, "y"]

  border_idx <- which(
    data[, "border"] & is.finite(data[, "x"]) & is.finite(data[, "y"])
  )
  if (length(border_idx) == 0) {
    return(data)
  }

  idx <- seq_along(border_idx)
  x_offset <- ((((idx * 0.61803398875) + (seed * 0.01)) %% 1) - 0.5) *
    2 *
    jitter_width
  y_offset <- ((((idx * 0.41421356237) + (seed * 0.01)) %% 1) - 0.5) *
    2 *
    jitter_height

  data[border_idx, "x_plot"] <- data[border_idx, "x"] + x_offset
  data[border_idx, "y_plot"] <- data[border_idx, "y"] + y_offset
  data
}

#' @title Compute velocity on grid
#'
#' @md
#' @param x_emb A matrix of dimension n_obs x n_dim specifying the embedding coordinates of the cells.
#' @param v_emb A matrix of dimension n_obs x n_dim specifying the velocity vectors of the cells.
#' @param density A numeric value specifying the density of the grid points along each dimension.
#' Default is `1`.
#' @param smooth A numeric value specifying the smoothing factor for the velocity vectors.
#' Default is `0.5`.
#' @param n_neighbors A numeric value specifying the number of nearest neighbors for each grid point.
#' Default is `ceiling(n_obs / 50)`.
#' @param min_mass A numeric value specifying the minimum mass required for a grid point to be considered.
#' Default is `1`.
#' @param scale A numeric value specifying the scaling factor for the velocity vectors.
#' Default is `1`.
#' @param adjust_for_stream Whether to adjust the velocity vectors for streamlines.
#' Default is `FALSE`.
#' @param cutoff_perc A numeric value specifying the percentile cutoff for removing low-density grid points.
#' Default is `5`.
#'
#' @return
#' A list with two components:
#' `x_grid`, the grid coordinates, and `v_grid`, the smoothed velocity vectors on the grid.
#'
#' @references
#' \url{https://github.com/theislab/scvelo/blob/master/scvelo/plotting/velocity_embedding_grid.py}
#'
#' @export
#'
#' @examples
#' x_emb <- matrix(
#'   c(
#'     0, 0,
#'     1, 0,
#'     0, 1,
#'     1, 1
#'   ),
#'   ncol = 2,
#'   byrow = TRUE
#' )
#' v_emb <- matrix(
#'   c(
#'     1, 0,
#'     1, 0,
#'     0, 1,
#'     0, 1
#'   ),
#'   ncol = 2,
#'   byrow = TRUE
#' )
#'
#' velocity_grid <- compute_velocity_on_grid(
#'   x_emb = x_emb,
#'   v_emb = v_emb,
#'   density = 0.1,
#'   n_neighbors = 2,
#'   adjust_for_stream = TRUE
#' )
#'
#' names(velocity_grid)
#' dim(velocity_grid$x_grid)
#' dim(velocity_grid$v_grid)
#' head(velocity_grid$x_grid)
#' head(velocity_grid$v_grid)
#'
#' grid_df <- expand.grid(
#'   x = velocity_grid$x_grid[1, ],
#'   y = velocity_grid$x_grid[2, ]
#' )
#'
#' plot_df <- data.frame(
#'   x = grid_df$x,
#'   y = grid_df$y,
#'   xend = grid_df$x + c(velocity_grid$v_grid[1, , ]) * 0.2,
#'   yend = grid_df$y + c(velocity_grid$v_grid[2, , ]) * 0.2
#' )
#'
#' ggplot2::ggplot(plot_df) +
#'   ggplot2::geom_segment(
#'     ggplot2::aes(x = x, y = y, xend = xend, yend = yend),
#'     arrow = grid::arrow(length = grid::unit(0.12, "inches")),
#'     na.rm = TRUE
#'   ) +
#'   ggplot2::coord_equal()
compute_velocity_on_grid <- function(
  x_emb,
  v_emb,
  density = 1,
  smooth = 0.5,
  n_neighbors = ceiling(n_obs / 50),
  min_mass = 1,
  scale = 1,
  adjust_for_stream = FALSE,
  cutoff_perc = 5
) {
  reshape_rowmajor <- function(x, dim) {
    values <- if (is.null(dim(x))) {
      as.vector(x)
    } else {
      as.vector(aperm(x, rev(seq_along(dim(x)))))
    }
    aperm(array(values, dim = rev(dim)), rev(seq_along(dim)))
  }

  n_obs <- nrow(x_emb)
  n_dim <- ncol(x_emb)

  grs <- list()
  for (dim_i in 1:n_dim) {
    m1 <- min(x_emb[, dim_i], na.rm = TRUE)
    m2 <- max(x_emb[, dim_i], na.rm = TRUE)
    # m1 <- m1 - 0.01 * abs(m2 - m1)
    # m2 <- m2 + 0.01 * abs(m2 - m1)
    gr <- seq(m1, m2, length.out = ceiling(50 * density))
    grs <- c(grs, list(gr))
  }
  x_grid <- as_matrix(expand.grid(grs))

  d <- proxyC::dist(
    x = thisutils::as_matrix(x_emb, return_sparse = TRUE),
    y = thisutils::as_matrix(x_grid, return_sparse = TRUE),
    method = "euclidean",
    use_nan = TRUE
  )
  neighbors <- Matrix::t(
    as_matrix(
      apply(
        d,
        2,
        function(x) {
          order(x, decreasing = FALSE)[1:n_neighbors]
        }
      )
    )
  )
  dists <- Matrix::t(
    as_matrix(
      apply(
        d,
        2,
        function(x) {
          x[order(x, decreasing = FALSE)[1:n_neighbors]]
        }
      )
    )
  )

  weight <- stats::dnorm(
    dists,
    sd = mean(sapply(grs, function(g) g[2] - g[1])) * smooth
  )
  p_mass <- p_mass_v <- Matrix::rowSums(weight)
  p_mass_v[p_mass_v < 1] <- 1

  neighbors_emb <- array(
    v_emb[neighbors, seq_len(ncol(v_emb))],
    dim = c(dim(neighbors), dim(v_emb)[2])
  )
  v_grid <- apply((neighbors_emb * c(weight)), c(1, 3), sum)
  v_grid <- v_grid / p_mass_v

  if (isTRUE(adjust_for_stream)) {
    x_grid <- matrix(
      c(unique(x_grid[, 1]), unique(x_grid[, 2])),
      nrow = 2,
      byrow = TRUE
    )
    ns <- floor(sqrt(length(v_grid[, 1])))
    v_grid <- reshape_rowmajor(Matrix::t(v_grid), c(2, ns, ns))

    mass <- sqrt(apply(v_grid**2, c(2, 3), sum))
    min_mass <- 10**(min_mass - 6) # default min_mass = 1e-5
    min_mass[min_mass > max(mass, na.rm = TRUE) * 0.9] <- max(
      mass,
      na.rm = TRUE
    ) *
      0.9
    cutoff <- reshape_rowmajor(mass, c(ns, ns)) < min_mass

    length <- Matrix::t(apply(apply(abs(neighbors_emb), c(1, 3), mean), 1, sum))
    length <- reshape_rowmajor(length, c(ns, ns))
    cutoff <- cutoff | length < stats::quantile(length, cutoff_perc / 100)
    v_grid[1, , ][cutoff] <- NA
  } else {
    min_mass <- min_mass * stats::quantile(p_mass, 0.99) / 100
    x_grid <- x_grid[p_mass > min_mass, ]
    v_grid <- v_grid[p_mass > min_mass, ]
    if (!is.null(scale)) {
      v_grid <- v_grid * scale
    }
  }
  return(list(x_grid = x_grid, v_grid = v_grid))
}

#' @title Color palettes collected
#'
#' @description This function creates a color palette for a given vector of values.
#'
#' @md
#' @param x A vector of character/factor or numeric values.
#' If missing, numeric values 1:n will be used as x.
#' @param n The number of colors to return for numeric values.
#' @param palette Palette name.
#' All available palette names can be queried with [show_palettes].
#' @param palcolor Custom colors used to create a color palette.
#' @param type Type of `x`.
#' Can be one of `"auto"`, `"discrete"` or `"continuous"`.
#' The default is `"auto"`, which automatically detects if `x` is a numeric value.
#' @param matched Whether to return a color vector of the same length as `x`.
#' Default is `FALSE`.
#' @param reverse Whether to invert the colors.
#' Default is `FALSE`.
#' @param NA_keep Whether to keep the color assignment to NA in `x`.
#' Default is `FALSE`.
#' @param NA_color Color assigned to NA if `NA_keep` is `TRUE`.
#' Default is `"grey80"`.
#'
#' @return A character vector of color codes (hexadecimal format) corresponding to the input values `x`. The length and structure depend on the `matched` parameter.
#'
#' @seealso [show_palettes], [palette_list]
#'
#' @export
#'
#' @examples
#' x <- c(1:3, NA, 3:5)
#' (pal1 <- palette_colors(
#'   x,
#'   palette = "Spectral"
#' ))
#' (pal2 <- palette_colors(
#'   x,
#'   palcolor = c("red", "white", "blue")
#' ))
#' (pal3 <- palette_colors(
#'   x,
#'   palette = "Spectral",
#'   n = 10
#' ))
#' (pal4 <- palette_colors(
#'   x,
#'   palette = "Spectral",
#'   n = 10,
#'   reverse = TRUE
#' ))
#' (pal5 <- palette_colors(
#'   x,
#'   palette = "Spectral",
#'   matched = TRUE
#' ))
#' (pal6 <- palette_colors(
#'   x,
#'   palette = "Spectral",
#'   matched = TRUE,
#'   NA_keep = TRUE
#' ))
#' show_palettes(
#'   list(pal1, pal2, pal3, pal4, pal5, pal6)
#' )
#'
#' # Use Chinese color palettes
#' palette_colors(
#'   x = letters[1:5],
#'   palette = "ChineseRed",
#'   type = "discrete"
#' )
#' palette_colors(
#'   x = letters[1:5],
#'   palette = "Chinese",
#'   type = "discrete"
#' )
#'
#' all_palettes <- show_palettes(return_palettes = TRUE)
#' names(all_palettes)
palette_colors <- function(
  x,
  n = 100,
  palette = "Chinese",
  palcolor = NULL,
  type = c("auto", "discrete", "continuous"),
  matched = FALSE,
  reverse = FALSE,
  NA_keep = FALSE,
  NA_color = "grey80"
) {
  palette_list <- c(
    thisplot::palette_list,
    get_chinese_palettes()
  )
  if (missing(x)) {
    x <- 1:n
    type <- "continuous"
  }
  if (!palette %in% names(palette_list)) {
    log_message(
      "The palette {.val {palette}} is invalid.\n",
      "Check the available palette names with {.fn show_palettes}.\n",
      "Or pass palette colors via the {.arg palcolor} parameter",
      message_type = "error"
    )
  }
  if (is.list(palcolor)) {
    palcolor <- unlist(palcolor)
  }
  if (all(palcolor == "")) {
    palcolor <- palette_list[[palette]]
  }
  if (is.null(palcolor) || length(palcolor) == 0) {
    palcolor <- palette_list[[palette]]
  }
  if (!is.null(names(palcolor))) {
    if (all(x %in% names(palcolor))) {
      palcolor <- palcolor[intersect(names(palcolor), x)]
    }
  }
  pal_n <- length(palcolor)
  type <- match.arg(type)

  if (type == "auto") {
    if (is.numeric(x)) {
      type <- "continuous"
    } else {
      type <- "discrete"
    }
  }

  if (type == "discrete") {
    if (!is.factor(x)) {
      x <- factor(x, levels = unique(x))
    }
    n_x <- nlevels(x)
    if (isTRUE(attr(palcolor, "type") == "continuous")) {
      color <- grDevices::colorRampPalette(palcolor)(n_x)
    } else {
      color <- ifelse(rep(n_x, n_x) <= pal_n,
        palcolor[1:n_x],
        grDevices::colorRampPalette(palcolor)(n_x)
      )
    }
    names(color) <- levels(x)
    if (any(is.na(x))) {
      color <- c(color, stats::setNames(NA_color, "NA"))
    }
    if (isTRUE(matched)) {
      color <- color[x]
      color[is.na(color)] <- NA_color
    }
  } else if (type == "continuous") {
    if (!is.numeric(x) && all(!is.na(x))) {
      log_message(
        "'x' must be type of numeric when use continuous color palettes.",
        message_type = "error"
      )
    }
    if (all(is.na(x))) {
      values <- as.factor(rep(0, n))
    } else if (length(unique(stats::na.omit(as.numeric(x)))) == 1) {
      values <- as.factor(rep(unique(stats::na.omit(as.numeric(x))), n))
    } else {
      if (isTRUE(matched)) {
        values <- cut(
          x,
          breaks = seq(min(x, na.rm = TRUE),
            max(x, na.rm = TRUE),
            length.out = n + 1
          ),
          include.lowest = TRUE
        )
      } else {
        values <- cut(
          1:100,
          breaks = seq(min(x, na.rm = TRUE),
            max(x, na.rm = TRUE),
            length.out = n + 1
          ),
          include.lowest = TRUE
        )
      }
    }

    n_x <- nlevels(values)
    color <- ifelse(rep(n_x, n_x) <= pal_n,
      palcolor[1:n_x],
      grDevices::colorRampPalette(palcolor)(n_x)
    )
    names(color) <- levels(values)
    if (any(is.na(x))) {
      color <- c(color, stats::setNames(NA_color, "NA"))
    }
    if (isTRUE(matched)) {
      if (all(is.na(x))) {
        color <- NA_color
      } else if (length(unique(stats::na.omit(x))) == 1) {
        color <- color[as.character(unique(stats::na.omit(x)))]
        color[is.na(color)] <- NA_color
      } else {
        color <- color[as.character(values)]
        color[is.na(color)] <- NA_color
      }
    }
  }

  if (isTRUE(reverse)) {
    color <- rev(color)
  }
  if (isFALSE(NA_keep)) {
    color <- color[names(color) != "NA"]
  }
  return(color)
}

#' @title Show the color palettes
#'
#' @description This function displays color palettes using ggplot2.
#'
#' @md
#' @param palettes A list of color palettes.
#' Default is `NULL`.
#' @param type The type of palettes to include.
#' Default is `"discrete"`.
#' @param index The indices of the palettes to include.
#' Default is `NULL`.
#' @param palette_names The names of the palettes to include.
#' Default is `NULL`.
#' @param return_names Whether to return the names of the selected palettes.
#' Default is `TRUE`.
#' @param return_palettes Whether to return the colors of selected palettes.
#' Default is `FALSE`.
#'
#' @return If `return_palettes` is `TRUE`, returns a list of color palettes. If `return_names` is `TRUE` (default), returns a character vector of palette names. Otherwise, returns `NULL` (called for side effects to display the plot).
#'
#' @seealso [palette_colors], [palette_list]
#'
#' @export
#'
#' @examples
#' show_palettes(
#'   palettes = list(
#'     c("red", "blue", "green"),
#'     c("yellow", "purple", "orange")
#'   )
#' )
#' all_palettes <- show_palettes(return_palettes = TRUE)
#' names(all_palettes)
#' all_palettes[["simspec"]]
#' show_palettes(index = 1:10)
#' show_palettes(
#'   type = "discrete",
#'   index = 1:10
#' )
#' show_palettes(
#'   type = "continuous",
#'   index = 1:10
#' )
#' show_palettes(
#'   palette_names = c(
#'     "Paired", "nejm", "simspec", "Spectral", "jet", "Chinese"
#'   ),
#'   return_palettes = TRUE
#' )
#' # Include Chinese palettes via prefix
#' show_palettes(
#'   palette_names = c("ChineseRed", "ChineseBlue"),
#'   return_palettes = TRUE
#' )
show_palettes <- function(
  palettes = NULL,
  type = c("discrete", "continuous"),
  index = NULL,
  palette_names = NULL,
  return_names = TRUE,
  return_palettes = FALSE
) {
  palette_list <- c(
    thisplot::palette_list,
    get_chinese_palettes()
  )
  if (!is.null(palettes)) {
    palette_list <- palettes
  } else {
    choose <- unlist(
      lapply(
        palette_list, function(x) isTRUE(attr(x, "type") %in% type)
      )
    )
    palette_list <- palette_list[choose]
  }
  index <- index[index %in% seq_along(palette_list)]
  if (!is.null(index)) {
    palette_list <- palette_list[index]
  }
  if (is.null(names(palette_list))) {
    names(palette_list) <- seq_along(palette_list)
  }
  if (is.null(palette_names)) {
    palette_names <- palette_names %||% names(palette_list)
  }
  palette_not_found <- !palette_names %in% names(palette_list)
  if (any(palette_not_found)) {
    log_message(
      "Can not find the palettes: {.val {palette_names[palette_not_found]}}",
      message_type = "error"
    )
  }
  palette_list <- palette_list[palette_names]

  df <- data.frame(
    palette = rep(
      names(palette_list), sapply(palette_list, length)
    ), color = unlist(palette_list)
  )
  df[["palette"]] <- factor(
    df[["palette"]],
    levels = rev(unique(df[["palette"]]))
  )
  df[["color_order"]] <- factor(
    seq_len(nrow(df)),
    levels = seq_len(nrow(df))
  )
  df[["proportion"]] <- as.numeric(
    1 / table(df$palette)[df$palette]
  )

  p <- ggplot(
    data = df,
    aes(
      y = .data[["palette"]],
      x = .data[["proportion"]],
      fill = .data[["color_order"]]
    )
  ) +
    geom_col(show.legend = FALSE) +
    scale_fill_manual(values = df[["color"]]) +
    scale_x_continuous(expand = c(0, 0), trans = "reverse") +
    theme_this(
      axis.title = element_blank(),
      axis.ticks = element_blank(),
      axis.text.x = element_blank(),
      panel.border = element_blank()
    )
  print(p)

  if (isTRUE(return_palettes)) {
    return(palette_list)
  }
  if (isTRUE(return_names)) {
    return(palette_names)
  }
}

#' @title Get Chinese color palettes
#'
#' @param prefix The prefix of the palette names.
#' Default is `"Chinese_"`.
#'
#' @return A list of Chinese color palettes.
#' @export
#'
#' @examples
#' show_palettes(get_chinese_palettes())
get_chinese_palettes <- function(
  prefix = "Chinese"
) {
  cc_obj <- ChineseColors()

  category_names <- c(
    "blue", "cyan", "green", "gray_brown",
    "orange", "purple", "red", "yellow"
  )

  palettes <- list()

  all_palette_colors <- list()
  for (name in category_names) {
    discrete_cols <- cc_obj[[name]]
    all_palette_colors[[name]] <- discrete_cols
    attr(discrete_cols, "type") <- "discrete"
    palettes[[paste0(prefix, thisutils::capitalize(name))]] <- discrete_cols
  }

  if (length(all_palette_colors) > 1) {
    all_colors_vec <- unlist(all_palette_colors)
    dup_colors <- all_colors_vec[duplicated(all_colors_vec)]

    if (length(dup_colors) > 0) {
      for (dup_color in unique(dup_colors)) {
        containing_palettes <- names(all_palette_colors)[
          sapply(all_palette_colors, function(x) dup_color %in% x)
        ]

        if (length(containing_palettes) > 1) {
          keep_in <- sort(containing_palettes)[1]
          remove_from <- containing_palettes[containing_palettes != keep_in]

          for (pal_name in remove_from) {
            pal_key <- paste0(prefix, thisutils::capitalize(pal_name))
            if (pal_key %in% names(palettes)) {
              palettes[[pal_key]] <- palettes[[pal_key]][palettes[[pal_key]] != dup_color]
            }
          }
        }
      }
    }
  }

  color_sets <- attr(thisplot::chinese_colors, "color_sets", exact = TRUE)
  chinese_default <- color_sets[["Chinese"]]
  if (is.null(chinese_default) || length(chinese_default) == 0) {
    fallback_names <- c(
      "hushuilan", "qunqing", "jialingshuilv", "baoshilv",
      "yangfei", "he", "zhizi", "qianlaohuang",
      "chengse", "feise", "xueqing", "biandouzi"
    )
    fallback_idx <- match(fallback_names, thisplot::chinese_colors$name)
    if (anyNA(fallback_idx)) {
      missing_names <- fallback_names[is.na(fallback_idx)]
      log_message(
        "Missing default Chinese color definitions for {.val {missing_names}}",
        message_type = "error"
      )
    }
    chinese_default <- thisplot::chinese_colors$hex[fallback_idx]
  }
  for (set_name in names(color_sets)) {
    palettes[[set_name]] <- color_sets[[set_name]]
  }

  attr(chinese_default, "type") <- "discrete"
  palettes[["Chinese"]] <- chinese_default

  continuous_cols <- grDevices::colorRampPalette(chinese_default)(256)
  attr(continuous_cols, "type") <- "continuous"
  palettes[["ChineseContinuous"]] <- continuous_cols

  palette_names <- names(palettes)
  special_names <- c("Chinese", "ChineseContinuous")
  set_names <- grep("^ChineseSet[0-9]+$", palette_names, value = TRUE)

  if (length(set_names) > 0) {
    set_numbers <- as.numeric(gsub("ChineseSet", "", set_names))
    set_names <- set_names[order(set_numbers)]
  }

  other_names <- setdiff(palette_names, c(special_names, set_names))
  sorted_names <- c(
    intersect(special_names, palette_names),
    set_names,
    sort(other_names)
  )
  palettes <- palettes[sorted_names]
  palettes
}

# -*- coding: utf-8 -*-

#' @title Utility Functions for Data Visualization and Plotting
#'
#' @description
#' Provides utility functions for data visualization and plotting in R. Includes functions for color manipulation, plot customization, panel size control, data optimization for plots, and layout adjustments. Designed to enhance workflows with ggplot2, patchwork, and ComplexHeatmap.
#'
#' @author Meng Xu (Maintainer), \email{mengxu98@qq.com}
#'
#' @source \url{https://mengxu98.github.io/thisplot/}
#'
#' @md
#' @docType package
#' @name thisplot-package
"_PACKAGE"

#' @title The logo of thisplot
#'
#' @description
#' The thisplot logo, using ASCII or Unicode characters
#' Use [cli::ansi_strip] to get rid of the colors.
#'
#' @md
#' @param unicode Unicode symbols on UTF-8 platforms.
#' Default is [cli::is_utf8_output].
#'
#' @return
#' A character vector with class `thisplot_logo`.
#'
#' @references
#' \url{https://github.com/tidyverse/tidyverse/blob/main/R/logo.R}
#'
#' @export
#' @examples
#' thisplot_logo()
thisplot_logo <- function(unicode = cli::is_utf8_output()) {
  logo <- c(
    "          0          1        2             3     4
             __  __    _              __       __
            / /_/ /_  (_)_____ ____  / /____  / /_
           / __/ __ ./ // ___// __ ./ // __ ./ __/
          / /_/ / / / /(__  )/ /_/ / // /_/ / /_
          .__/_/ /_/_//____// .___/_/ .____/.__/
                           /_/
      5               6      7        8          9"
  )

  hexa <- c("*", ".", "o", "*", ".", "o", "*", ".", "o", "*")
  if (unicode) {
    hexa <- c("*" = "\u2b22", "o" = "\u2b21", "." = ".")[hexa]
  }

  cols <- c(
    "red", "yellow", "green", "magenta", "cyan",
    "yellow", "green", "white", "magenta", "cyan"
  )

  col_hexa <- mapply(
    function(x, y) cli::make_ansi_style(y)(x),
    hexa, cols,
    SIMPLIFY = FALSE
  )

  for (i in 0:9) {
    pat <- paste0("\\b", i, "\\b")
    logo <- sub(pat, col_hexa[[i + 1]], logo)
  }

  structure(
    cli::col_blue(logo),
    class = "thisplot_logo"
  )
}

#' @title Print logo
#'
#' @param x Input information.
#' @param ... Other parameters.
#'
#' @return Print the ASCII logo
#'
#' @method print thisplot_logo
#'
#' @export
#'
print.thisplot_logo <- function(x, ...) {
  cat(x, ..., sep = "\n")
  invisible(x)
}

.onAttach <- function(libname, pkgname) {
  verbose <- thisutils::get_verbose()
  if (isTRUE(verbose)) {
    version <- utils::packageVersion(pkgname)
    date <- utils::packageDate(pkgname)
    url <- utils::packageDescription(
      pkgname, fields = "URL"
    )

    msg <- paste0(
      cli::col_grey(strrep("-", 60)),
      "\n",
      cli::col_blue("Version: ", version, " (", date, " update)"),
      "\n",
      cli::col_blue("Website: ", cli::style_italic(url)),
      "\n\n",
      cli::col_grey("This message can be suppressed by:"),
      "\n",
      cli::col_grey("  suppressPackageStartupMessages(library(thisplot))"),
      "\n",
      cli::col_grey("  or options(log_message.verbose = FALSE)"),
      "\n",
      cli::col_grey(strrep("-", 60))
    )

    packageStartupMessage(thisplot_logo())
    packageStartupMessage(msg)
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap_border.R
\name{heatmap_border_spec}
\alias{heatmap_border_spec}
\alias{heatmap_border_enabled}
\alias{heatmap_border_color}
\alias{heatmap_border_size_value}
\alias{heatmap_border_gp}
\alias{heatmap_legend_border}
\title{Normalize ComplexHeatmap border settings}
\usage{
heatmap_border_spec(border = NULL, palcolor = "black", size = 1)

heatmap_border_enabled(border)

heatmap_border_color(border, palcolor = "black")

heatmap_border_size_value(size)

heatmap_border_gp(border, color, size = 1)

heatmap_legend_border(border, color)
}
\arguments{
\item{border}{`NULL`, a logical border flag, or a non-empty colour string.}

\item{palcolor}{Fallback border colour when `border` is not a colour string.}

\item{size}{Border line width. Invalid or negative values use `1`.}

\item{color}{Fallback border colour used by the `gp` and legend helpers.}
}
\value{
A list with `enabled`, `color`, `size`, `gp`, and `legend_border`.
}
\description{
Normalize a heatmap border flag, optional colour override, and line width
into values accepted by `ComplexHeatmap::Heatmap` and
`ComplexHeatmap::Legend`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{annotation_block_fill_graphics}
\alias{annotation_block_fill_graphics}
\title{Build block fill panel function}
\usage{
annotation_block_fill_graphics(
  levels,
  palette = NULL,
  palcolor = NULL,
  fill_values = NULL,
  border = TRUE,
  border_color = "black",
  border_size = 1
)
}
\arguments{
\item{levels}{Character/factor levels used as keys for fill mapping.}

\item{palette}{Palette name.
All available palette names can be queried with \link{show_palettes}.}

\item{palcolor}{Custom colors used to create a color palette.}

\item{fill_values}{Optional named fill vector. Names should match \code{levels}.}

\item{border}{Whether to draw block border.}

\item{border_color}{Border color when \code{border = TRUE}. Default is \code{"black"}.}

\item{border_size}{Border line width when \code{border = TRUE}. Default is \code{1}.}
}
\value{
A function with signature \verb{(index, levels)} suitable for \code{panel_fun} in \link[ComplexHeatmap:anno_block]{ComplexHeatmap::anno_block}.
}
\description{
Build block fill panel function
}
\examples{
library(ggplot2)
lv <- c("A", "B", "C")
panel_fun <- annotation_block_fill_graphics(
  levels = lv,
  fill_values = c(A = "#1b9e77", B = "#d95f02", C = "#7570b3")
)
ComplexHeatmap::anno_block(panel_fun = panel_fun)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_legend}
\alias{get_legend}
\title{Extract legend from a plot}
\usage{
get_legend(plot)
}
\arguments{
\item{plot}{A plot object.}
}
\value{
The legend grob.
}
\description{
Extract the legend grob from a plot object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/major_grid.R
\name{major_grid}
\alias{major_grid}
\title{Set major grid lines on a ggplot}
\usage{
major_grid(
  plot,
  grid_major = TRUE,
  grid_major_colour = "grey80",
  grid_major_linetype = 2,
  grid_major_linewidth = 0.3
)
}
\arguments{
\item{plot}{A ggplot object.}

\item{grid_major}{Whether to show major grid lines. Default is \code{TRUE}.}

\item{grid_major_colour}{Grid line colour. Default is \code{"grey80"}.}

\item{grid_major_linetype}{Grid line type. Default is \code{2} (dashed).}

\item{grid_major_linewidth}{Grid line width. Default is \code{0.3}.}
}
\value{
A ggplot object with modified \code{panel.grid.major} theme element.
}
\description{
Add or remove major grid lines from a ggplot object.
}
\examples{
library(ggplot2)
p <- ggplot(mtcars, aes(wt, mpg)) + geom_point()
major_grid(p, grid_major = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colors.R
\name{BlendRGBList}
\alias{BlendRGBList}
\title{Blend a list of colors}
\usage{
BlendRGBList(Clist, mode = "blend", RGB_BackGround = c(1, 1, 1))
}
\arguments{
\item{Clist}{A list of colors, where each color is a list containing
RGB values and alpha channel.}

\item{mode}{The blending mode to use.
One of \code{"blend"}, \code{"average"}, \code{"screen"}, or \code{"multiply"}.
Default is \code{"blend"}.}

\item{RGB_BackGround}{The background RGB color to composite with.
Default is \code{c(1, 1, 1)} (white).}
}
\value{
A numeric vector of RGB values.
}
\description{
Blend multiple colors with alpha channels into a single color using a specified blending mode.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_colors.R
\name{print.colors}
\alias{print.colors}
\title{Print colors object}
\usage{
\method{print}{colors}(x, ...)
}
\arguments{
\item{x}{A colors object (data frame with color information).}

\item{...}{Additional arguments passed to print.}
}
\value{
Details of the colors objec.
}
\description{
Print colors object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sankey.R
\name{theme_sankey}
\alias{theme_sankey}
\alias{theme_alluvial}
\alias{theme_sankey_bump}
\title{Themes for sankey plot}
\usage{
theme_sankey(
  base_size = 11,
  base_family = "",
  base_line_size = base_size/22,
  base_rect_size = base_size/22
)

theme_alluvial(
  base_size = 11,
  base_family = "",
  base_line_size = base_size/22,
  base_rect_size = base_size/22
)

theme_sankey_bump(
  base_size = 11,
  base_family = "",
  base_line_size = base_size/22,
  base_rect_size = base_size/22
)
}
\arguments{
\item{base_size}{Base font size, given in pts.}

\item{base_family}{Base font family.}

\item{base_line_size}{Base size for line elements.}

\item{base_rect_size}{Base size for rect elements.}
}
\description{
Minimal themes for sankey, alluvial and sankey bump plots
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colors.R
\name{adjcolors}
\alias{adjcolors}
\title{Convert a color with specified alpha level}
\usage{
adjcolors(colors, alpha)
}
\arguments{
\item{colors}{Color vectors.}

\item{alpha}{Alpha level in \verb{[0,1]}.}
}
\value{
A character vector of hexadecimal color codes with the specified alpha level.
}
\description{
Convert a color with specified alpha level
}
\examples{
colors <- c("red", "blue", "green")
adjcolors(colors, 0.5)
ggplot2::alpha(colors, 0.5)

show_palettes(
  list(
    "raw" = colors,
    "adjcolors" = adjcolors(colors, 0.5),
    "ggplot2::alpha" = ggplot2::alpha(colors, 0.5)
  )
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{patchwork_grob}
\alias{patchwork_grob}
\title{Convert a patchwork object to a grob}
\usage{
patchwork_grob(x, ...)
}
\arguments{
\item{x}{A patchwork object.}

\item{...}{Additional arguments passed to other functions.}
}
\value{
A gtable object.
}
\description{
Convert a patchwork object to a gtable grob by processing annotations
and building the patchwork layout.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colors.R
\name{Blend2Color}
\alias{Blend2Color}
\title{Blend two colors using a specified mode}
\usage{
Blend2Color(C1, C2, mode = "blend")
}
\arguments{
\item{C1}{A list containing the first color RGB values and alpha channel.}

\item{C2}{A list containing the second color RGB values and alpha channel.}

\item{mode}{The blending mode to use.
One of \code{"blend"}, \code{"average"}, \code{"screen"}, or \code{"multiply"}.
Default is \code{"blend"}.}
}
\value{
A list containing the blended RGB values and alpha channel.
}
\description{
Blend two colors with alpha channels using one of several blending modes:
blend, average, screen, or multiply.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{chinese_colors}
\alias{chinese_colors}
\title{Chinese traditional colors dataset}
\description{
A dataset containing optimized traditional Chinese colors.
These colors are extracted from those books:
\itemize{
\item{Chinese Traditional Colors - Color Aesthetics in the Forbidden City (ISBN: 9787521716054)}
\item{Chinese Beautiful Colors - The Most Chinese Culture Vol.3 (ISBN: 9781672897198)}
\item{Chinese Colors (ISBN: 9787558016479)}
\item{Chinese Journal of Chromatography (ISSN 1000-8713)}
\item{Chinese Color Atlas}
}

Thanks to the author of the \href{https://book.douban.com/review/14384195/}{blog} for providing the data.
}
\examples{
data(chinese_colors)
color_sets <- attr(chinese_colors, "color_sets")
show_palettes(
  list(
    color_sets$Chinese,
    color_sets$ChineseSet8,
    color_sets$ChineseSet16,
    color_sets$ChineseSet32
  )
)

# Use ChineseColors class
cc <- ChineseColors()
cc$visual_colors(
  loc_range = c(1, 180),
  num_per_row = 30,
  title = "Chinese Traditional Colors",
  name_type = "chinese"
)
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sankey.R
\name{make_long}
\alias{make_long}
\title{Make a long data frame for sankey plot}
\usage{
make_long(.df, ..., value = NULL)
}
\arguments{
\item{.df}{a data frame}

\item{...}{unquoted columnnames of df that you want to include in the plot.}

\item{value}{if each row have a weight this weight could be kept by providing column name of weight.}
}
\value{
a longer data frame
}
\description{
Prepares a 'wide' data frame into a format that `geom_sankey` or `geom_alluvial` understands.
Useful to show flows between dimensions in dataset.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{heatmap_fixsize}
\alias{heatmap_fixsize}
\title{Compute fixed heatmap device size}
\usage{
heatmap_fixsize(
  width,
  width_sum,
  height,
  height_sum,
  units,
  ht_list,
  legend_list
)
}
\arguments{
\item{width}{Optional user-provided target width. If \code{NULL}, width is
estimated from the drawn heatmap layout.}

\item{width_sum}{Numeric baseline width used when heatmap body size is in
\code{npc} units.}

\item{height}{Optional user-provided target height. If \code{NULL}, height is
estimated from the drawn heatmap layout.}

\item{height_sum}{Numeric baseline height used when heatmap body size is in
\code{npc} units.}

\item{units}{Output unit string passed to grid conversion helpers, such as
\code{"in"}, \code{"cm"}, or \code{"mm"}.}

\item{ht_list}{A \code{ComplexHeatmap::Heatmap} or
\code{ComplexHeatmap::HeatmapList} object.}

\item{legend_list}{A list of \code{ComplexHeatmap::Legend} objects.}
}
\value{
A list with two converted units: \code{ht_width} and \code{ht_height}.
}
\description{
Compute fixed heatmap device size
}
\examples{
mat <- matrix(rnorm(100), nrow = 10)
ht <- ComplexHeatmap::Heatmap(mat, name = "expr")
lgd <- list(ComplexHeatmap::Legend(title = "expr", at = c(-2, 0, 2)))
out <- heatmap_fixsize(
  width = NULL,
  width_sum = 6,
  height = NULL,
  height_sum = 4,
  units = "in",
  ht_list = ht,
  legend_list = lgd
)
out
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/themes.R
\name{theme_blank}
\alias{theme_blank}
\title{Blank theme}
\usage{
theme_blank(
  add_coord = TRUE,
  xlen_npc = 0.15,
  ylen_npc = 0.15,
  xlab = "",
  ylab = "",
  lab_size = 12,
  ...
)
}
\arguments{
\item{add_coord}{Whether to add coordinate arrows. Default is \code{TRUE}.}

\item{xlen_npc}{The length of the x-axis arrow in "npc".}

\item{ylen_npc}{The length of the y-axis arrow in "npc".}

\item{xlab}{The label of the x-axis.}

\item{ylab}{The label of the y-axis.}

\item{lab_size}{The size of the axis labels.}

\item{...}{Arguments passed to the \link[ggplot2:theme]{ggplot2::theme}.}
}
\value{
A list containing ggplot2 theme objects and annotation objects.
If \code{add_coord} is \code{TRUE}, returns a list with coordinate arrows;
otherwise returns a list with theme only.
}
\description{
This function creates a theme with all elements blank except for axis lines and labels.
It can optionally add coordinate axes in the plot.
}
\examples{
library(ggplot2)
p <- ggplot(mtcars, aes(x = wt, y = mpg, colour = factor(cyl))) +
  geom_point()
p + theme_blank()
p + theme_blank(xlab = "x-axis", ylab = "y-axis", lab_size = 16)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{clip_symmetric_range}
\alias{clip_symmetric_range}
\title{Clip values to a symmetric range}
\usage{
clip_symmetric_range(data, value_col = "avg_log2FC")
}
\arguments{
\item{data}{A data frame.}

\item{value_col}{Name of the numeric column to clip.}
}
\value{
A list containing the clipped data frame and clipping limits.
}
\description{
Clip values to a symmetric range
}
\examples{
df <- data.frame(avg_log2FC = c(-10, -2, 0, 2, 10))
clip_symmetric_range(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{as_grob}
\alias{as_grob}
\title{Convert a plot object to a grob}
\usage{
as_grob(plot, ...)
}
\arguments{
\item{plot}{A plot object (gList, patchwork, or ggplot).}

\item{...}{Additional arguments passed to other functions.}
}
\value{
A grob object.
}
\description{
Convert various plot objects (gList, patchwork, ggplot) to a grob object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{build_heatmap_annotation}
\alias{build_heatmap_annotation}
\title{Build HeatmapAnnotation with safe parameter merge}
\usage{
build_heatmap_annotation(
  annotations,
  which,
  show_annotation_name = TRUE,
  annotation_name_side = NULL,
  border = NULL,
  params = NULL
)
}
\arguments{
\item{annotations}{Named list of annotation components (for example, \code{anno_simple}, \code{anno_block}, \code{anno_customize}).}

\item{which}{Annotation direction (\code{"row"} or \code{"column"}).}

\item{show_annotation_name}{Whether to show annotation names.}

\item{annotation_name_side}{Side for annotation names.}

\item{border}{Whether to draw a rectangle border around the block.}

\item{params}{Additional user parameters; duplicated keys are ignored if already set explicitly.}
}
\value{
A \link[ComplexHeatmap:HeatmapAnnotation]{ComplexHeatmap::HeatmapAnnotation} object.
}
\description{
Build HeatmapAnnotation with safe parameter merge
}
\examples{
anno <- list(
  group = ComplexHeatmap::anno_simple(
    x = c("A", "B", "A"),
    col = c(A = "#1b9e77", B = "#d95f02"),
    which = "column"
  )
)
ha <- build_heatmap_annotation(
  annotations = anno,
  which = "column",
  show_annotation_name = TRUE,
  annotation_name_side = "left",
  params = list(gap = grid::unit(1, "mm"))
)
ha
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visual_colors.R
\name{visual_colors}
\alias{visual_colors}
\title{Visualize colors in HTML widget}
\usage{
visual_colors(
  colors,
  names = NULL,
  num_per_row = 30,
  title = NULL,
  label_mode = c("auto", "chinese", "pinyin", "rgb", "hex")
)
}
\arguments{
\item{colors}{A character vector of hex color codes.}

\item{names}{Optional. A character vector of names for each color.
Default is \code{NULL}, which means hex color codes will be displayed.
You can pass any labels (e.g., RGB values, custom names) via this parameter.}

\item{num_per_row}{Number of colors per row.
Default is \code{30}.}

\item{title}{Optional title for the visualization.
Default is \code{NULL}.}

\item{label_mode}{Label layout mode. Default is \code{"auto"}, which detects
the label type from \code{names}. Use \code{"chinese"}, \code{"pinyin"}, \code{"rgb"},
or \code{"hex"} to force a layout optimized for that label style.}
}
\value{
An HTML widget.
}
\description{
Display a grid of color swatches with optional names or color codes.
}
\examples{
# Visualize a simple color palette
visual_colors(
  colors = c("#FF0000", "#00FF00", "#0000FF"),
  names = c("Red", "Green", "Blue")
)

visual_colors(
  colors = c("#FF0000", "#00FF00"),
  names = c("(255, 0, 0)", "(0, 255, 0)")
)

visual_colors(thisplot::palette_list$Paired)

# Use with ChineseColors
cc <- ChineseColors()
visual_colors(
  colors = cc$blue[1:60],
  title = "Chinese Blue Colors"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/themes.R
\name{theme_this}
\alias{theme_this}
\title{The default theme for scop plot function.}
\usage{
theme_this(aspect.ratio = NULL, base_size = 12, ...)
}
\arguments{
\item{aspect.ratio}{Aspect ratio of the panel.}

\item{base_size}{Base font size}

\item{...}{Arguments passed to the \link[ggplot2:theme]{ggplot2::theme}.}
}
\value{
A ggplot2 theme object (class \code{theme}, \code{gg}).
}
\description{
The default theme for scop plot function.
}
\examples{
library(ggplot2)
p <- ggplot(
  data = mtcars,
  aes(x = wt, y = mpg, colour = factor(cyl))
) +
  geom_point()
p + theme_this()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sankey.R
\name{geom_sankey}
\alias{geom_sankey}
\title{geom_sankey}
\usage{
geom_sankey(
  mapping = NULL,
  data = NULL,
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  space = NULL,
  type = "sankey",
  width = 0.1,
  smooth = 8,
  inherit.aes = TRUE,
  ...
)
}
\arguments{
\item{mapping}{Provide you own mapping. Both x and y need to be numeric.}

\item{data}{Provide you own data.}

\item{position}{Change position.}

\item{na.rm}{Remove missing values.}

\item{show.legend}{Show legend in plot.}

\item{space}{Space between nodes in the y-direction.}

\item{type}{Either `sankey` or `alluvial`.}

\item{width}{Width of nodes.}

\item{smooth}{How much smooth should the curve have? More means steeper curve.}

\item{inherit.aes}{Should the geom inherit aesthetics.}

\item{...}{Other arguments to be passed to the geom.}
}
\value{
A ggplot layer.
}
\description{
Creates a sankey plot which visualize flows between nodes.
Each observation needs to have a `x` aesthetic as well as a `next_x` column which declares where that observation should flow.
Also each observation should have a `node` and a `next_node` aesthetic which provide information about which group in the y-direction.
By default each row of the data frame is counted to calculate the size of flows. A manual flow value can be added with the `value` aesthetic.
}
\section{Aesthetics}{

geom_sankey understand the following aesthetics (required aesthetics are in
bold):

- **x0**
- **y0**
- **a**
- **b**
- **angle**
- m1
- m2
- color
- fill
- size
- linetype
- alpha
- lineend
}

\examples{
dat <- data.frame(
  Group = c("A", "A", "B", "B", "C"),
  Type = c("X", "Y", "X", "Y", "X")
)
long <- make_long(dat, Group, Type)
ggplot2::ggplot(
  long,
  ggplot2::aes(
    x = x,
    next_x = next_x,
    node = node,
    next_node = next_node,
    fill = node
  )
) +
  geom_sankey() +
  theme_sankey()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slim_data.R
\name{slim_data}
\alias{slim_data}
\alias{slim_data.ggplot}
\alias{slim_data.patchwork}
\title{Slim unused data in the plot}
\usage{
slim_data(p)

\method{slim_data}{ggplot}(p)

\method{slim_data}{patchwork}(p)
}
\arguments{
\item{p}{A \code{ggplot} object or a \code{patchwork} object.}
}
\value{
A \code{ggplot} or \code{patchwork} object with unused data columns removed.
}
\description{
Remove unused columns from the data in a ggplot or patchwork object.
This function keeps only the columns that are actually used in the plot
(e.g., in mappings, aesthetics, or facets), which can significantly reduce
the object size when the original data contains many unused columns.
}
\examples{
library(ggplot2)
p <- ggplot(
  data = mtcars,
  aes(x = mpg, y = wt, colour = cyl)
) +
  geom_point()
object.size(p)
colnames(p$data)

p_slim <- slim_data(p)
object.size(p_slim)
colnames(p_slim$data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colors.R
\name{col2hex}
\alias{col2hex}
\title{Convert color names to hexadecimal format}
\usage{
col2hex(cname)
}
\arguments{
\item{cname}{A character vector of color names.}
}
\value{
A character vector of hexadecimal color codes.
}
\description{
Convert color names to hexadecimal RGB color codes.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{adjustlayout}
\alias{adjustlayout}
\title{Adjust graph layout to avoid node overlaps}
\usage{
adjustlayout(graph, layout, width, height = 2, scale = 100, iter = 100)
}
\arguments{
\item{graph}{An igraph graph object.}

\item{layout}{A matrix with two columns representing the initial layout coordinates.}

\item{width}{A numeric vector of node widths.}

\item{height}{The height constraint for nodes.
Default is \code{2}.}

\item{scale}{The scaling factor for the layout.
Default is \code{100}.}

\item{iter}{The number of iterations for the adjustment algorithm.
Default is \code{100}.}
}
\value{
A matrix with adjusted layout coordinates.
}
\description{
Adjust the layout of a graph to prevent node overlaps by considering node widths and heights.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{segements_df}
\alias{segements_df}
\title{Shorten and offset the segment}
\usage{
segements_df(data, shorten_start, shorten_end, offset)
}
\arguments{
\item{data}{A data frame containing the segments.
It should have columns 'x', 'y', 'xend', and 'yend' representing the start and end points of each segment.}

\item{shorten_start}{The amount to shorten the start of each segment by.}

\item{shorten_end}{The amount to shorten the end of each segment by.}

\item{offset}{The amount to offset each segment by.}
}
\value{
The modified data frame with the shortened and offset segments.
}
\description{
This function takes a data frame representing segments in a plot and shortens
and offsets them based on the provided arguments.
}
\examples{
library(ggplot2)
temp_nodes <- data.frame(
  "x" = c(10, 40),
  "y" = c(10, 30)
)
data <- data.frame(
  "x" = c(10, 40),
  "y" = c(10, 30),
  "xend" = c(40, 10),
  "yend" = c(30, 10)
)

ggplot(temp_nodes, aes(x = x, y = y)) +
  geom_point(size = 12) +
  xlim(0, 50) +
  ylim(0, 50) +
  geom_segment(
    data = data,
    aes(x = x, xend = xend, y = y, yend = yend)
  )

ggplot(temp_nodes, aes(x = x, y = y)) +
  geom_point(size = 12) +
  xlim(0, 50) +
  ylim(0, 50) +
  geom_segment(
    data = segements_df(
      data,
      shorten_start = 2,
      shorten_end = 3,
      offset = 1
    ),
    aes(x = x, xend = xend, y = y, yend = yend)
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_colors.R
\name{head.colors}
\alias{head.colors}
\title{Return the first part of a colors object}
\usage{
\method{head}{colors}(x, n = 6L, ...)
}
\arguments{
\item{x}{A colors object (data frame with color information).}

\item{n}{Number of rows to return. Default is \code{6}.}

\item{...}{Additional arguments passed to \code{\link[=head]{head()}}.}
}
\value{
A colors object with the first \code{n} rows.
}
\description{
Returns the first part of a colors object, similar to \code{\link[=head]{head()}} for data frames.
}
\examples{
head(get_colors())

head(get_colors(), n = 10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{build_patchwork}
\alias{build_patchwork}
\title{Build a patchwork gtable}
\usage{
build_patchwork(
  x,
  guides = "auto",
  table_rows = 18,
  table_cols = 15,
  panel_row = 10,
  panel_col = 8
)
}
\arguments{
\item{x}{A patchwork object.}

\item{guides}{How to handle guides.
Default is \code{"auto"}.}

\item{table_rows}{The number of rows in the table grid.
Default is \code{18}.}

\item{table_cols}{The number of columns in the table grid.
Default is \code{15}.}

\item{panel_row}{The row index for panels.
Default is \code{10}.}

\item{panel_col}{The column index for panels.
Default is \code{8}.}
}
\value{
A gtable object.
}
\description{
Build a gtable from a patchwork object by arranging multiple plots
according to the layout specification.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/palettes.R
\name{show_palettes}
\alias{show_palettes}
\title{Show the color palettes}
\usage{
show_palettes(
  palettes = NULL,
  type = c("discrete", "continuous"),
  index = NULL,
  palette_names = NULL,
  return_names = TRUE,
  return_palettes = FALSE
)
}
\arguments{
\item{palettes}{A list of color palettes.
Default is \code{NULL}.}

\item{type}{The type of palettes to include.
Default is \code{"discrete"}.}

\item{index}{The indices of the palettes to include.
Default is \code{NULL}.}

\item{palette_names}{The names of the palettes to include.
Default is \code{NULL}.}

\item{return_names}{Whether to return the names of the selected palettes.
Default is \code{TRUE}.}

\item{return_palettes}{Whether to return the colors of selected palettes.
Default is \code{FALSE}.}
}
\value{
If \code{return_palettes} is \code{TRUE}, returns a list of color palettes. If \code{return_names} is \code{TRUE} (default), returns a character vector of palette names. Otherwise, returns \code{NULL} (called for side effects to display the plot).
}
\description{
This function displays color palettes using ggplot2.
}
\examples{
show_palettes(
  palettes = list(
    c("red", "blue", "green"),
    c("yellow", "purple", "orange")
  )
)
all_palettes <- show_palettes(return_palettes = TRUE)
names(all_palettes)
all_palettes[["simspec"]]
show_palettes(index = 1:10)
show_palettes(
  type = "discrete",
  index = 1:10
)
show_palettes(
  type = "continuous",
  index = 1:10
)
show_palettes(
  palette_names = c(
    "Paired", "nejm", "simspec", "Spectral", "jet", "Chinese"
  ),
  return_palettes = TRUE
)
# Include Chinese palettes via prefix
show_palettes(
  palette_names = c("ChineseRed", "ChineseBlue"),
  return_palettes = TRUE
)
}
\seealso{
\link{palette_colors}, \link{palette_list}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GraphPlot.R
\name{GraphPlot}
\alias{GraphPlot}
\title{Graph Plot}
\usage{
GraphPlot(
  node,
  edge,
  transition = NULL,
  node_coord = c("x", "y"),
  node_group = NULL,
  node_palette = "Chinese",
  node_palcolor = NULL,
  node_size = 4,
  node_alpha = 1,
  node_highlight = NULL,
  node_highlight_color = "red",
  label = FALSE,
  label.size = 3.5,
  label.fg = "white",
  label.bg = "black",
  label.bg.r = 0.1,
  label_insitu = FALSE,
  label_repel = FALSE,
  label_repulsion = 20,
  label_point_size = 1,
  label_point_color = "black",
  label_segment_color = "black",
  edge_threshold = 0.01,
  use_triangular = c("upper", "lower", "both"),
  edge_line = c("straight", "curved"),
  edge_line_curvature = 0.3,
  edge_line_angle = 90,
  edge_color = "grey30",
  edge_size = c(0.2, 1),
  edge_alpha = 0.5,
  edge_shorten = 0,
  edge_offset = 0,
  edge_highlight = NULL,
  edge_highlight_color = "red",
  transition_threshold = 0.01,
  transition_line = c("straight", "curved"),
  transition_line_curvature = 0.3,
  transition_line_angle = 90,
  transition_color = "black",
  transition_size = c(0.2, 1),
  transition_alpha = 1,
  transition_arrow_type = "closed",
  transition_arrow_angle = 20,
  transition_arrow_length = grid::unit(0.02, "npc"),
  transition_shorten = 0.05,
  transition_offset = 0,
  transition_highlight = NULL,
  transition_highlight_color = "red",
  aspect.ratio = 1,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  theme_use = "theme_this",
  theme_args = list(),
  return_layer = FALSE
)
}
\arguments{
\item{node}{A data frame representing the nodes of the graph.}

\item{edge}{A matrix representing the edges of the graph.}

\item{transition}{A matrix representing the transitions between nodes.}

\item{node_coord}{A character vector specifying the names of the columns in \code{node} that represent the x and y coordinates.}

\item{node_group}{A character vector specifying the name of the column in \code{node} that represents the grouping of the nodes.}

\item{node_palette}{A character vector specifying the name of the color palette for node groups.}

\item{node_palcolor}{A character vector specifying the names of the colors for each node group.}

\item{node_size}{A numeric value or column name of \code{node} specifying the size of the nodes.}

\item{node_alpha}{A numeric value or column name of \code{node} specifying the transparency of the nodes.}

\item{node_highlight}{A character vector specifying the names of nodes to highlight.}

\item{node_highlight_color}{A character vector specifying the color for highlighting nodes.}

\item{label}{Whether to show labels for the nodes.}

\item{label.size}{The size of the labels.}

\item{label.fg}{A character vector specifying the foreground color of the labels.}

\item{label.bg}{A character vector specifying the background color of the labels.}

\item{label.bg.r}{The background color transparency of the labels.}

\item{label_insitu}{Whether to display the node group labels in situ or as numeric values.}

\item{label_repel}{Whether to use force-directed label repulsion.}

\item{label_repulsion}{The repulsion force for labels.}

\item{label_point_size}{The size of the label points.}

\item{label_point_color}{A character vector specifying the color of the label points.}

\item{label_segment_color}{A character vector specifying the color for the label segments.}

\item{edge_threshold}{The threshold for removing edges.}

\item{use_triangular}{A character vector specifying which part of the edge matrix to use (upper, lower, both).}

\item{edge_line}{A character vector specifying the type of line for edges (straight, curved).}

\item{edge_line_curvature}{The curvature of curved edges.}

\item{edge_line_angle}{The angle of curved edges.}

\item{edge_color}{A character vector specifying the color of the edges.}

\item{edge_size}{A numeric vector specifying the range of edge sizes.}

\item{edge_alpha}{The transparency of the edges.}

\item{edge_shorten}{The length of the edge shorten.}

\item{edge_offset}{The length of the edge offset.}

\item{edge_highlight}{A character vector specifying the names of edges to highlight.}

\item{edge_highlight_color}{A character vector specifying the color for highlighting edges.}

\item{transition_threshold}{The threshold for removing transitions.}

\item{transition_line}{A character vector specifying the type of line for transitions (straight, curved).}

\item{transition_line_curvature}{The curvature of curved transitions.}

\item{transition_line_angle}{The angle of curved transitions.}

\item{transition_color}{A character vector specifying the color of the transitions.}

\item{transition_size}{A numeric vector specifying the range of transition sizes.}

\item{transition_alpha}{The transparency of the transitions.}

\item{transition_arrow_type}{A character vector specifying the type of arrow for transitions (closed, open).}

\item{transition_arrow_angle}{The angle of the transition arrow.}

\item{transition_arrow_length}{The length of the transition arrow.}

\item{transition_shorten}{The length of the transition shorten.}

\item{transition_offset}{The length of the transition offset.}

\item{transition_highlight}{A character vector specifying the names of transitions to highlight.}

\item{transition_highlight_color}{A character vector specifying the color for highlighting transitions.}

\item{aspect.ratio}{Aspect ratio passed to \code{theme()}.}

\item{title}{Plot title.}

\item{subtitle}{Plot subtitle.}

\item{xlab}{Label for the x axis.}

\item{ylab}{Label for the y axis.}

\item{legend.position}{Legend position passed to \code{theme()}.}

\item{legend.direction}{Legend direction passed to \code{theme()}.}

\item{theme_use}{Theme function applied to the plot.}

\item{theme_args}{A named list of arguments passed to \code{theme_use}.}

\item{return_layer}{Whether to return the plot layers as a list.
Defaults is \code{FALSE}.}
}
\description{
A function to plot a graph with nodes and edges.
}
\examples{
node <- data.frame(
  x = c(0.2, 0.3, 0.5, 0.3),
  y = c(0.2, 0, 0.6, 0.7),
  group = c("A", "B", "C", "B"),
  row.names = c("n1", "n2", "n3", "n4")
)
edge <- matrix(
  c(
    0, 0.4, 0.2, 0.1,
    0.4, 0, 0.3, 0.2,
    0.2, 0.3, 0 , 0.5,
    0.2, 0.2, 0.5, 0.3
  ),
  nrow = 4,
  byrow = TRUE,
  dimnames = list(rownames(node), rownames(node))
)
GraphPlot(
  node = node,
  edge = edge
)

GraphPlot(
  node = node,
  edge = edge,
  node_group = "group",
  node_highlight = "n1",
  label = TRUE,
  label_repel = TRUE
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{annotate_quadrants}
\alias{annotate_quadrants}
\title{Annotate plot quadrants with percentages}
\usage{
annotate_quadrants(
  plot,
  x,
  y,
  cutoffs,
  line_color = "grey30",
  line_type = "solid",
  line_width = 0.5,
  label_size = 3,
  group = NULL
)
}
\arguments{
\item{plot}{A `ggplot` object.}

\item{x}{Name of the x variable in `plot$data`.}

\item{y}{Name of the y variable in `plot$data`.}

\item{cutoffs}{Cut points for x and y axes. Can be a numeric vector of length
1 or 2, or a list with the x and y cut points.}

\item{line_color}{Quadrant line color.}

\item{line_type}{Quadrant line type.}

\item{line_width}{Quadrant line width.}

\item{label_size}{Label size.}

\item{group}{Optional grouping column used to compute percentages within each
group.}
}
\value{
A `ggplot` object.
}
\description{
Annotate plot quadrants with percentages
}
\examples{
df <- data.frame(
  x = c(0.1, 0.2, 0.8, 0.9),
  y = c(0.1, 0.8, 0.2, 0.9)
)
p <- ggplot2::ggplot(df, ggplot2::aes(x = x, y = y)) +
  ggplot2::geom_point()
p

annotate_quadrants(p, x = "x", y = "y", cutoffs = 0.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colors.R
\name{blendcolors}
\alias{blendcolors}
\title{Blends a list of colors using the specified blend mode}
\usage{
blendcolors(colors, mode = c("blend", "average", "screen", "multiply"))
}
\arguments{
\item{colors}{Color vectors.}

\item{mode}{Blend mode.
One of \code{"blend"}, \code{"average"}, \code{"screen"}, or \code{"multiply"}.}
}
\value{
A character vector of hexadecimal color codes representing the blended color.
}
\description{
Blends a list of colors using the specified blend mode
}
\examples{
blend <- c(
  "red",
  "green",
  blendcolors(c("red", "green"),
    mode = "blend"
  )
)
average <- c(
  "red",
  "green",
  blendcolors(c("red", "green"),
    mode = "average"
  )
)
screen <- c(
  "red",
  "green",
  blendcolors(c("red", "green"),
    mode = "screen"
  )
)
multiply <- c(
  "red",
  "green",
  blendcolors(c("red", "green"),
    mode = "multiply"
  )
)
show_palettes(
  list(
    "blend" = blend,
    "average" = average,
    "screen" = screen,
    "multiply" = multiply
  )
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{mestimate}
\alias{mestimate}
\title{Estimate the fuzzifier parameter m}
\usage{
mestimate(data)
}
\arguments{
\item{data}{A matrix or data frame.}
}
\value{
The estimated fuzzifier parameter m.
}
\description{
Estimate the fuzzifier parameter m for fuzzy clustering based on the data dimensions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sankey.R
\name{geom_alluvial}
\alias{geom_alluvial}
\title{geom_alluvial}
\usage{
geom_alluvial(
  mapping = NULL,
  data = NULL,
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  space = 0,
  width = 0.1,
  smooth = 8,
  inherit.aes = TRUE,
  ...
)
}
\arguments{
\item{mapping}{Provide you own mapping. Both x and y need to be numeric.}

\item{data}{Provide you own data.}

\item{position}{Change position.}

\item{na.rm}{Remove missing values.}

\item{show.legend}{Show legend in plot.}

\item{space}{Space between nodes in the y-direction.}

\item{width}{Width of nodes.}

\item{smooth}{How much smooth should the curve have? More means steeper curve.}

\item{inherit.aes}{Should the geom inherit aesthetics.}

\item{...}{Other arguments to be passed to the geom.}
}
\value{
A ggplot layer.
}
\description{
Creates an alluvial plot which visualize flows between nodes.
Each observation needs to have a `x` aesthetic as well as a `next_x` column which declares where that observation should flow.
Also each observation should have a `node` and a `next_node` aesthetic which provide information about which group in the y-direction.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{annotation_block_graphics}
\alias{annotation_block_graphics}
\title{Build block panel function}
\usage{
annotation_block_graphics(
  subplot,
  name,
  border = TRUE,
  border_color = "black",
  border_size = 1
)
}
\arguments{
\item{subplot}{A single drawable subplot object.}

\item{name}{A name assigned to the generated grob.}

\item{border}{Whether to draw a rectangle border around the block.}

\item{border_color}{Border color when \code{border = TRUE}. Default is \code{"black"}.}

\item{border_size}{Border line width when \code{border = TRUE}. Default is \code{1}.}
}
\value{
A function with signature \verb{(index, levels)} suitable for \code{panel_fun} in \link[ComplexHeatmap:anno_block]{ComplexHeatmap::anno_block}.
}
\description{
Build block panel function
}
\examples{
library(ggplot2)
p <- ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg)) +
  ggplot2::geom_point()
panel_fun <- annotation_block_graphics(
  subplot = p, name = "demo-block"
)
ComplexHeatmap::anno_block(panel_fun = panel_fun)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chinese_colors.R
\name{print.ChineseColors}
\alias{print.ChineseColors}
\title{Print \code{ChineseColors} object}
\usage{
\method{print}{ChineseColors}(x, ...)
}
\arguments{
\item{x}{A \code{ChineseColors} object.}

\item{...}{Additional arguments.}
}
\value{
Details of the \code{ChineseColors} object.
}
\description{
Print \code{ChineseColors} object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{standardise}
\alias{standardise}
\title{Standardize data by rows}
\usage{
standardise(data)
}
\arguments{
\item{data}{A matrix or data frame to standardize.}
}
\value{
The standardized data with the same structure as input.
}
\description{
Standardize each row of a data matrix by subtracting the mean and dividing by the standard deviation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/StatPlot.R
\name{StatPlot}
\alias{StatPlot}
\title{Statistic Plot}
\usage{
StatPlot(
  meta.data,
  stat.by,
  group.by = NULL,
  split.by = NULL,
  bg.by = NULL,
  flip = FALSE,
  NA_color = "grey",
  NA_stat = TRUE,
  keep_empty = FALSE,
  individual = FALSE,
  stat_level = NULL,
  plot_type = c("bar", "rose", "ring", "pie", "trend", "trend_alluvial", "area", "dot",
    "sankey", "chord", "venn", "upset"),
  venn_engine = c("ggVennDiagram", "venny"),
  venn_args = list(),
  stat_type = c("percent", "count", "value"),
  value.by = NULL,
  top_n = Inf,
  rank.by = c("absolute", "value"),
  complete_groups = NULL,
  value_cutoff = NULL,
  value_limits = NULL,
  value_midpoint = 0,
  value_legend_title = NULL,
  bar_fill = NULL,
  bar_width = 0.8,
  point_size = c(2.5, 7),
  character_width = 50,
  return_data = FALSE,
  position = c("stack", "dodge"),
  palette = "Chinese",
  palcolor = NULL,
  alpha = 1,
  bg_palette = "Chinese",
  bg_palcolor = NULL,
  bg_alpha = 0.2,
  label = FALSE,
  label_cutoff = NULL,
  label.size = 3.5,
  label.fg = "black",
  label.bg = "white",
  label.bg.r = 0.1,
  aspect.ratio = NULL,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  theme_use = "theme_this",
  theme_args = list(),
  x_text_angle = 45,
  grid_major = TRUE,
  grid_major_colour = "grey80",
  grid_major_linetype = 2,
  grid_major_linewidth = 0.3,
  combine = TRUE,
  nrow = NULL,
  ncol = NULL,
  byrow = TRUE,
  force = FALSE,
  seed = 11
)
}
\arguments{
\item{meta.data}{The data frame containing the data to be plotted.}

\item{stat.by}{The column name(s) in \code{meta.data} specifying the variable(s) to be plotted.}

\item{group.by}{The column name(s) in \code{meta.data} specifying the grouping variable(s).
Default is \code{NULL}.}

\item{split.by}{The column name in \code{meta.data} specifying the variable to split plots by.
Default is \code{NULL}.}

\item{bg.by}{The column name in \code{meta.data} specifying the background variable for bar plots.}

\item{flip}{Whether to flip the plot.
Default is \code{FALSE}.}

\item{NA_color}{The color to use for missing values.}

\item{NA_stat}{Whether to include missing values in the plot.
Default is \code{TRUE}.}

\item{keep_empty}{Whether to keep empty groups in the plot.
Default is \code{FALSE}.}

\item{individual}{Whether to plot individual groups separately.
Default is \code{FALSE}.}

\item{stat_level}{The level(s) of the variable(s) specified in \code{stat.by} to include in the plot.
Default is \code{NULL}.}

\item{plot_type}{The type of plot to create.
Can be one of \code{"bar"}, \code{"rose"}, \code{"ring"}, \code{"pie"}, \code{"trend"},
\code{"trend_alluvial"}, \code{"area"}, \code{"dot"}, \code{"sankey"}, \code{"chord"},
\code{"venn"}, or \code{"upset"}.}

\item{venn_engine}{The engine used when \code{plot_type = "venn"}.
Can be one of \code{"ggVennDiagram"} or \code{"venny"}.
Default is \code{"ggVennDiagram"}.
The \code{"venny"} engine supports 2 to 4 sets.}

\item{venn_args}{A list of additional arguments passed to
\code{venny::venny} when \code{plot_type = "venn"} and \code{venn_engine = "venny"}.
The \code{data} argument is generated internally and cannot be supplied here.}

\item{stat_type}{The type of statistic to compute for the plot.
Can be one of \code{"percent"}, \code{"count"}, or \code{"value"}. Continuous value mode
supports \code{plot_type = "bar"} and \code{plot_type = "dot"}.}

\item{value.by}{Numeric column used when \code{stat_type = "value"}.}

\item{top_n}{Maximum number of records retained per group and split in value
mode. Default is \code{Inf}.}

\item{rank.by}{Rank continuous values by absolute magnitude or signed value.}

\item{complete_groups}{For value dot plots, retain every available group for
a label selected in any group within the same split.}

\item{value_cutoff}{Optional minimum absolute value.}

\item{value_limits}{Optional continuous colour limits. Symmetric limits are
inferred by default.}

\item{value_midpoint}{Midpoint of the continuous colour scale.}

\item{value_legend_title}{Legend title for continuous values.}

\item{bar_fill}{Optional fixed fill used by value bars.}

\item{bar_width}{Width of bars in value mode.}

\item{point_size}{Size range for value dot plots.}

\item{character_width}{Maximum wrapped label width in value mode.}

\item{return_data}{Return the plot and prepared data in value mode.}

\item{position}{The position adjustment for the plot.
Can be one of \code{"stack"} or \code{"dodge"}.}

\item{palette}{The name of the color palette to use.
Default is \code{"Chinese"}.}

\item{palcolor}{Custom colors to use instead of palette.
Default is \code{NULL}.}

\item{alpha}{The transparency level for the plot.}

\item{bg_palette}{The name of the background color palette to use for bar plots.}

\item{bg_palcolor}{The color to use in the background color palette.}

\item{bg_alpha}{The transparency level for the background color in bar plots.}

\item{label}{Whether to add labels on the plot.
Default is \code{FALSE}.}

\item{label_cutoff}{Optional minimum plotted value required to show a label.
Labels are shown only for values strictly greater than this cutoff. For
percentage plots, use proportions; for example, \code{label_cutoff = 0.1} shows
labels only for values greater than 10\%. Default is \code{NULL}, which labels all
values.}

\item{label.size}{The size of the labels.}

\item{label.fg}{The foreground color of the labels.}

\item{label.bg}{The background color of the labels.}

\item{label.bg.r}{The radius of the rounded corners of the label background.}

\item{aspect.ratio}{Aspect ratio of the panel. Default is \code{NULL}.}

\item{title}{The title of the plot.
Default is \code{NULL}.}

\item{subtitle}{The subtitle of the plot.
Default is \code{NULL}.}

\item{xlab}{The label for the x-axis.
Default is \code{NULL}.}

\item{ylab}{The label for the y-axis.
Default is \code{NULL}.}

\item{legend.position}{The position of the legend.
Can be one of \code{"none"}, \code{"left"}, \code{"right"}, \code{"bottom"}, \code{"top"},
or a two-element numeric vector.
Default is \code{"right"}.}

\item{legend.direction}{The direction of the legend.
Can be one of \code{"vertical"} or \code{"horizontal"}.
Default is \code{"vertical"}.}

\item{theme_use}{The theme to use for the plot.
Default is \code{"theme_this"}.}

\item{theme_args}{Additional arguments to pass to the theme function.
Default is \code{list()}.}

\item{x_text_angle}{Rotation angle for x-axis labels.
Default is \code{45}. Must be a finite number.}

\item{grid_major}{Whether to show major panel grid lines.
Default is \code{TRUE}.}

\item{grid_major_colour}{Color of major panel grid lines.}

\item{grid_major_linetype}{Linetype of major panel grid lines.}

\item{grid_major_linewidth}{Line width of major panel grid lines.}

\item{combine}{Whether to combine multiple plots into one.
Default is \code{TRUE}.}

\item{nrow}{Number of rows when combining plots.
Default is \code{NULL}.}

\item{ncol}{Number of columns when combining plots.
Default is \code{NULL}.}

\item{byrow}{Whether to fill plots by row when combining.
Default is \code{TRUE}.}

\item{force}{Whether to force plotting even when variables have more than 100 levels.
Default is \code{FALSE}.}

\item{seed}{Random seed for reproducibility.
Default is \code{11}.}
}
\description{
Visualizes data using various plot types such as bar plots,
rose plots, ring plots, pie charts, trend plots, area plots,
dot plots, sankey plots, chord plots, venn diagrams, and upset plots.
}
\examples{
set.seed(1)
meta_data <- data.frame(
  Type = factor(
    sample(c("A", "B", "C"),
      50,
      replace = TRUE,
      prob = c(0.5, 0.3, 0.2)
    )
  ),
  Group = factor(sample(c("X", "Y", "Z"), 50, replace = TRUE)),
  Batch = factor(sample(c("B1", "B2"), 50, replace = TRUE))
)
meta_data$Region <- factor(
  ifelse(meta_data$Group \%in\% c("X", "Y"), "R1", "R2"),
  levels = c("R1", "R2")
)

StatPlot(
  meta_data,
  stat.by = "Type",
  group.by = "Group",
  split.by = "Batch",
  plot_type = "bar",
  position = "dodge"
)

StatPlot(
  meta_data,
  stat.by = "Type",
  group.by = "Group",
  stat_type = "count",
  plot_type = "ring",
  position = "dodge"
)

StatPlot(
  meta_data,
  stat.by = "Type",
  group.by = "Group",
  stat_type = "count"
)

StatPlot(
  meta_data,
  stat.by = "Type",
  plot_type = "pie"
)

StatPlot(
  meta_data,
  stat.by = "Type",
  group.by = "Group",
  stat_type = "count",
  plot_type = "area"
)

StatPlot(
  meta_data,
  stat.by = "Type",
  group.by = "Group",
  plot_type = "dot"
)

StatPlot(
  meta_data,
  stat.by = "Type",
  group.by = "Group",
  stat_type = "count",
  plot_type = "trend"
)

rank_data <- data.frame(
  Feature = paste0("Feature", 1:12),
  Group = rep(c("Control", "Treatment"), each = 6),
  Value = c(
    -2.4, 1.8, 1.2, -0.8, 0.6, 0.2,
    2.7, -2.1, 1.5, 0.9, -0.5, 0.3
  )
)

StatPlot(
  rank_data,
  stat.by = "Feature",
  value.by = "Value",
  group.by = "Group",
  stat_type = "value",
  plot_type = "bar",
  top_n = 4,
  flip = TRUE,
  palette = "RdBu"
)

StatPlot(
  rank_data,
  stat.by = "Feature",
  value.by = "Value",
  group.by = "Group",
  stat_type = "value",
  plot_type = "dot",
  top_n = 3,
  palette = "RdBu"
)

importance_data <- data.frame(
  Variable = paste0("Variable", 1:8),
  Importance = c(0.91, 0.78, 0.64, 0.52, 0.43, 0.31, 0.18, 0.09)
)
StatPlot(
  importance_data,
  stat.by = "Variable",
  value.by = "Importance",
  stat_type = "value",
  top_n = 5,
  flip = TRUE,
  bar_fill = "#3C8DBC"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{extractgrobs}
\alias{extractgrobs}
\title{Extract grobs from a list}
\usage{
extractgrobs(vlnplots, x_nm, y_nm, x, y)
}
\arguments{
\item{vlnplots}{A named list of grobs.}

\item{x_nm}{A character vector of names for the x dimension.}

\item{y_nm}{A character vector of names for the y dimension.}

\item{x}{An integer index for the x dimension.}

\item{y}{An integer index for the y dimension.}
}
\value{
The extracted grob(s).
}
\description{
Extract grobs from a named list of grobs based on the specified x and y indices.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drop_data.R
\name{drop_data}
\alias{drop_data}
\alias{drop_data.ggplot}
\alias{drop_data.patchwork}
\alias{drop_data.default}
\title{Drop unused data in the plot}
\usage{
drop_data(p)

\method{drop_data}{ggplot}(p)

\method{drop_data}{patchwork}(p)

\method{drop_data}{default}(p)
}
\arguments{
\item{p}{A \code{ggplot} object or a \code{patchwork} object.}
}
\value{
A \code{ggplot} or \code{patchwork} object with unused data points removed.
}
\description{
Drop unused data points from a ggplot or patchwork object while preserving
the plot structure. This function keeps only a single row of data for each
unique combination of used variables, which can significantly reduce the
object size when the original data contains many rows that are not displayed
in the plot (e.g., due to scale limits or filtering).
}
\examples{
library(ggplot2)
library(patchwork)
p <- ggplot(
  data = mtcars,
  aes(x = mpg, y = wt, colour = cyl)
) +
  geom_point() +
  scale_x_continuous(limits = c(10, 30)) +
  scale_y_continuous(limits = c(1, 6))
object.size(p)

p_drop <- drop_data(p)
object.size(p_drop)

p / p_drop
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/thisplot-package.R
\docType{package}
\name{thisplot-package}
\alias{thisplot}
\alias{thisplot-package}
\title{Utility Functions for Data Visualization and Plotting}
\source{
\url{https://mengxu98.github.io/thisplot/}
}
\description{
Provides utility functions for data visualization and plotting in R. Includes functions for color manipulation, plot customization, panel size control, data optimization for plots, and layout adjustments. Designed to enhance workflows with ggplot2, patchwork, and ComplexHeatmap.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://mengxu98.github.io/thisplot/}
  \item Report bugs at \url{https://github.com/mengxu98/thisplot/issues}
}

}
\author{
Meng Xu (Maintainer), \email{mengxu98@qq.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colors.R
\name{RGBA2RGB}
\alias{RGBA2RGB}
\title{Convert RGBA color to RGB with background}
\usage{
RGBA2RGB(RGBA, BackGround = c(1, 1, 1))
}
\arguments{
\item{RGBA}{A list containing RGB values and alpha channel.}

\item{BackGround}{The background RGB color to composite with.
Default is \code{c(1, 1, 1)} (white).}
}
\value{
A numeric vector of RGB values.
}
\description{
Convert an RGBA (Red, Green, Blue, Alpha) color to RGB by compositing
it with a background color based on the alpha channel.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{as_gtable}
\alias{as_gtable}
\title{Convert a plot object to a gtable}
\usage{
as_gtable(plot, ...)
}
\arguments{
\item{plot}{A plot object (gtable, grob, patchwork, or ggplot).}

\item{...}{Additional arguments passed to other functions.}
}
\value{
A gtable object.
}
\description{
Convert various plot objects (gtable, grob, patchwork, ggplot) to a gtable object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chinese_colors.R
\name{ChineseColors}
\alias{ChineseColors}
\title{Chinese traditional colors system}
\usage{
ChineseColors()
}
\value{
A \code{ChineseColors} object.
Detailed information can be found in \code{\link[=print.ChineseColors]{print.ChineseColors()}}.
}
\description{
A color system based on Chinese traditional colors with 1058 colors.
}
\examples{
cc <- ChineseColors()
cc

# Get a color by pinyin
get_colors("pinlan")

# By number
get_colors(44)

# By hex code
get_colors("#2B73AF")

# Multiple colors
get_colors("pinlan", "piao")
get_colors(91:100)

# Chinese names
cc$visual_colors(
  loc_range = c(1, 180),
  num_per_row = 30,
  title = "Chinese Traditional Colors",
  name_type = "chinese"
)

# pinyin as names
cc$visual_colors(
  loc_range = c(1, 120),
  title = "Chinese Traditional Colors",
  name_type = "pinyin"
)

# rgb as names
cc$visual_colors(
  loc_range = c(1, 120),
  title = "Colors with RGB values",
  name_type = "rgb"
)

# hex as names
cc$visual_colors(
  loc_range = c(1, 120),
  title = "Colors with hex codes",
  name_type = "hex"
)
}
\seealso{
\link{chinese_colors} for the dataset of Chinese traditional colors.
\link{get_chinese_palettes} for getting Chinese color palettes.
\link{visual_colors} for visualizing any color vector.
\link{get_colors} for searching colors in dataset and palettes.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{heatmap_rendersize}
\alias{heatmap_rendersize}
\title{Estimate heatmap render size}
\usage{
heatmap_rendersize(
  width,
  height,
  units,
  ha_top_list,
  ha_left,
  ha_right,
  ht_list,
  legend_list,
  flip
)
}
\arguments{
\item{width}{Numeric vector of heatmap body widths.
Interpretation depends on \code{flip}.}

\item{height}{Numeric vector of heatmap body heights.
Interpretation depends on \code{flip}.}

\item{units}{Unit string for numeric-to-grid conversion (for example, \code{"in"}, \code{"cm"}, \code{"mm"}).}

\item{ha_top_list}{A list of top annotations (typically \code{ComplexHeatmap::HeatmapAnnotation} objects).}

\item{ha_left}{Optional left annotation.}

\item{ha_right}{Optional right annotation.}

\item{ht_list}{A \code{ComplexHeatmap::Heatmap} or
\code{ComplexHeatmap::HeatmapList} object.}

\item{legend_list}{A list containing legend objects; \code{NULL} entries are ignored.}

\item{flip}{Logical; when \code{TRUE}, width/height interpretation follows the flipped layout branch.}
}
\value{
A list with numeric \code{width_sum} and \code{height_sum} in \code{units}.
}
\description{
Estimate heatmap render size
}
\examples{
mat <- matrix(rnorm(100), nrow = 10)
ht <- ComplexHeatmap::Heatmap(mat, name = "expr")
size <- heatmap_rendersize(
  width = c(4),
  height = c(3),
  units = "in",
  ha_top_list = list(),
  ha_left = NULL,
  ha_right = NULL,
  ht_list = ht,
  legend_list = list(),
  flip = FALSE
)
size
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{normalize_drawable}
\alias{normalize_drawable}
\title{Normalize drawable objects to grobs}
\usage{
normalize_drawable(obj)
}
\arguments{
\item{obj}{A drawable object, such as a ggplot, patchwork, or grid grob.}
}
\value{
A grid grob object. If the input is not drawable, returns a null grob.
}
\description{
Normalize drawable objects to grobs
}
\examples{
library(ggplot2)
p <- ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg)) +
  ggplot2::geom_point()
g <- normalize_drawable(p)
p + g
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{grid_draw}
\alias{grid_draw}
\title{Draw grobs at specified positions}
\usage{
grid_draw(groblist, x, y, width, height)
}
\arguments{
\item{groblist}{A grob or a list of grobs to draw.}

\item{x}{A numeric vector of x positions for each grob.}

\item{y}{A numeric vector of y positions for each grob.}

\item{width}{A numeric vector of widths for each grob.}

\item{height}{A numeric vector of heights for each grob.}
}
\value{
No return value, called for side effects (drawing grobs).
}
\description{
Draw a list of grobs at specified positions with given widths and heights.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{normalize_to_sum}
\alias{normalize_to_sum}
\title{Normalize values to a unit sum}
\usage{
normalize_to_sum(x)
}
\arguments{
\item{x}{A numeric vector.}
}
\value{
A numeric vector scaled to sum to 1.
}
\description{
Normalize values to a unit sum
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{jitter_highlighted_points}
\alias{jitter_highlighted_points}
\title{Jitter highlighted points deterministically}
\usage{
jitter_highlighted_points(
  data,
  jitter_width = 0.2,
  jitter_height = 0.2,
  seed = 11
)
}
\arguments{
\item{data}{A data frame containing `x`, `y`, and `border` columns.}

\item{jitter_width}{Width of the x jitter.}

\item{jitter_height}{Height of the y jitter.}

\item{seed}{Numeric value used to offset the deterministic sequence.}
}
\value{
A data frame with `x_plot` and `y_plot` columns.
}
\description{
Jitter highlighted points deterministically
}
\examples{
df <- data.frame(
  x = c(0, 1),
  y = c(0, 1),
  border = c(FALSE, TRUE)
)
jitter_highlighted_points(df, jitter_width = 0.2, jitter_height = 0.2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/panel_fix.R
\name{panel_fix}
\alias{panel_fix}
\alias{panel_fix_overall}
\title{Set the panel width/height of a plot to a fixed value}
\usage{
panel_fix(
  x = NULL,
  panel_index = NULL,
  respect = NULL,
  width = NULL,
  height = NULL,
  margin = 1,
  padding = 0,
  units = "in",
  raster = FALSE,
  dpi = 300,
  return_grob = FALSE,
  bg_color = "white",
  save = NULL,
  verbose = FALSE,
  ...
)

panel_fix_overall(
  x,
  panel_index = NULL,
  respect = NULL,
  width = NULL,
  height = NULL,
  margin = 1,
  units = "in",
  raster = FALSE,
  dpi = 300,
  return_grob = FALSE,
  bg_color = "white",
  save = NULL,
  verbose = TRUE
)
}
\arguments{
\item{x}{A ggplot object, a grob object, or a combined plot made by patchwork or cowplot package.}

\item{panel_index}{Specify the panel to be fixed.
If \code{NULL}, will fix all panels.}

\item{respect}{Whether row heights and column widths should respect each other.}

\item{width}{The desired width of the fixed panels.}

\item{height}{The desired height of the fixed panels.}

\item{margin}{The margin to add around each panel, in inches.
Default is \code{1}.}

\item{padding}{The padding to add around each panel, in inches.
Default is \code{0}.}

\item{units}{The units in which \code{height}, \code{width} and \code{margin} are given.
Can be \code{"mm"}, \code{"cm"}, \code{"in"}, etc. See \link[grid:unit]{grid::unit}.}

\item{raster}{Whether to rasterize the panel.}

\item{dpi}{Plot resolution.}

\item{return_grob}{Whether to return a grob object instead of a wrapped \code{patchwork} object.
Default is \code{FALSE}.}

\item{bg_color}{The background color of the plot.}

\item{save}{\code{NULL} or the file name used to save the plot.}

\item{verbose}{Whether to print the message.
Default is \code{TRUE}.}

\item{...}{Additional arguments passed to other functions.}
}
\value{
If \code{return_grob} is \code{TRUE}, returns a gtable object.
Otherwise, returns a patchwork object with fixed panel sizes.
The returned object has a \code{size} attribute containing width, height, and units.
}
\description{
The ggplot object, when stored, can only specify the height and width of the entire plot, not the panel.
The latter is obviously more important to control the final result of a plot.
This function can set the panel width/height of plot to a fixed value and rasterize it.
}
\examples{
library(ggplot2)
p <- ggplot(
  data = mtcars, aes(x = mpg, y = wt, colour = cyl)
) +
  geom_point() +
  facet_wrap(~gear, nrow = 2)
# fix the size of panel
panel_fix(
  p,
  width = 5,
  height = 3,
  units = "cm"
)
# rasterize the panel
panel_fix(
  p,
  width = 5,
  height = 3,
  units = "cm",
  raster = TRUE,
  dpi = 90
)

# `panel_fix` will build and render the plot when input a ggplot object.
# so after `panel_fix`, the size of the object will be changed.
object.size(p)
object.size(
  panel_fix(
    p,
    width = 5,
    height = 3,
    units = "cm"
  )
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_vars}
\alias{get_vars}
\title{Get used vars in a ggplot object}
\usage{
get_vars(p, reverse = FALSE, verbose = TRUE)
}
\arguments{
\item{p}{A \code{ggplot} object.}

\item{reverse}{Whether to return unused vars.
Default is \code{FALSE}.}

\item{verbose}{Whether to print the message.
Default is \code{TRUE}.}
}
\value{
A character vector of variable names.
If \code{reverse} is \code{FALSE}, returns used variables;
if \code{TRUE}, returns unused variables.
}
\description{
Get used vars in a ggplot object
}
\examples{
library(ggplot2)
p <- ggplot(
  data = mtcars,
  aes(x = mpg, y = wt, colour = cyl)
) +
  geom_point()
get_vars(p)
get_vars(p, reverse = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{palette_list}
\alias{palette_list}
\title{A list of palettes for use in data visualization}
\description{
A list of palettes for use in data visualization
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_colors.R
\name{get_colors}
\alias{get_colors}
\title{Get colors from Chinese colors dataset or palettes}
\usage{
get_colors(..., palettes = NULL)
}
\arguments{
\item{...}{One or more search values.
Can be palette names, color names (pinyin or Chinese), numbers, or hex codes.
If NULL, using all Chinese colors.}

\item{palettes}{Optional. A named list of palettes to search in.
If \code{NULL} (default), searches in all available palettes.}
}
\value{
A data frame with class \code{colors} containing matching color information.
The result is automatically printed using \code{\link[=print.colors]{print.colors()}}.
}
\description{
Search for colors in the Chinese colors dataset and all available palettes.
This function can search by palette names, color names (pinyin or Chinese), numbers, or hex codes.
It automatically searches in all palettes and reports which palette(s) contain the found colors.
}
\examples{
get_colors("Paired")

get_colors("#FF7F00")

get_colors("pinlan")
get_colors(44)
get_colors("#2B73AF")

get_colors("cyan", palettes = "ChineseSet64")
}
\seealso{
\link{chinese_colors} for the dataset of Chinese traditional colors.
\link{get_chinese_palettes} for getting Chinese color palettes.
\link{ChineseColors} for the ChineseColors object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{to_percent}
\alias{to_percent}
\title{Convert proportions to percentages}
\usage{
to_percent(x)
}
\arguments{
\item{x}{A numeric vector.}
}
\value{
A numeric vector expressed as percentages.
}
\description{
Convert proportions to percentages
}
\examples{
to_percent(c(1, 1, 2))
to_percent(c(0, 0))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/palettes.R
\name{palette_colors}
\alias{palette_colors}
\title{Color palettes collected}
\usage{
palette_colors(
  x,
  n = 100,
  palette = "Chinese",
  palcolor = NULL,
  type = c("auto", "discrete", "continuous"),
  matched = FALSE,
  reverse = FALSE,
  NA_keep = FALSE,
  NA_color = "grey80"
)
}
\arguments{
\item{x}{A vector of character/factor or numeric values.
If missing, numeric values 1:n will be used as x.}

\item{n}{The number of colors to return for numeric values.}

\item{palette}{Palette name.
All available palette names can be queried with \link{show_palettes}.}

\item{palcolor}{Custom colors used to create a color palette.}

\item{type}{Type of \code{x}.
Can be one of \code{"auto"}, \code{"discrete"} or \code{"continuous"}.
The default is \code{"auto"}, which automatically detects if \code{x} is a numeric value.}

\item{matched}{Whether to return a color vector of the same length as \code{x}.
Default is \code{FALSE}.}

\item{reverse}{Whether to invert the colors.
Default is \code{FALSE}.}

\item{NA_keep}{Whether to keep the color assignment to NA in \code{x}.
Default is \code{FALSE}.}

\item{NA_color}{Color assigned to NA if \code{NA_keep} is \code{TRUE}.
Default is \code{"grey80"}.}
}
\value{
A character vector of color codes (hexadecimal format) corresponding to the input values \code{x}. The length and structure depend on the \code{matched} parameter.
}
\description{
This function creates a color palette for a given vector of values.
}
\examples{
x <- c(1:3, NA, 3:5)
(pal1 <- palette_colors(
  x,
  palette = "Spectral"
))
(pal2 <- palette_colors(
  x,
  palcolor = c("red", "white", "blue")
))
(pal3 <- palette_colors(
  x,
  palette = "Spectral",
  n = 10
))
(pal4 <- palette_colors(
  x,
  palette = "Spectral",
  n = 10,
  reverse = TRUE
))
(pal5 <- palette_colors(
  x,
  palette = "Spectral",
  matched = TRUE
))
(pal6 <- palette_colors(
  x,
  palette = "Spectral",
  matched = TRUE,
  NA_keep = TRUE
))
show_palettes(
  list(pal1, pal2, pal3, pal4, pal5, pal6)
)

# Use Chinese color palettes
palette_colors(
  x = letters[1:5],
  palette = "ChineseRed",
  type = "discrete"
)
palette_colors(
  x = letters[1:5],
  palette = "Chinese",
  type = "discrete"
)

all_palettes <- show_palettes(return_palettes = TRUE)
names(all_palettes)
}
\seealso{
\link{show_palettes}, \link{palette_list}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_velocity_on_grid.R
\name{compute_velocity_on_grid}
\alias{compute_velocity_on_grid}
\title{Compute velocity on grid}
\usage{
compute_velocity_on_grid(
  x_emb,
  v_emb,
  density = 1,
  smooth = 0.5,
  n_neighbors = ceiling(n_obs/50),
  min_mass = 1,
  scale = 1,
  adjust_for_stream = FALSE,
  cutoff_perc = 5
)
}
\arguments{
\item{x_emb}{A matrix of dimension n_obs x n_dim specifying the embedding coordinates of the cells.}

\item{v_emb}{A matrix of dimension n_obs x n_dim specifying the velocity vectors of the cells.}

\item{density}{A numeric value specifying the density of the grid points along each dimension.
Default is \code{1}.}

\item{smooth}{A numeric value specifying the smoothing factor for the velocity vectors.
Default is \code{0.5}.}

\item{n_neighbors}{A numeric value specifying the number of nearest neighbors for each grid point.
Default is \code{ceiling(n_obs / 50)}.}

\item{min_mass}{A numeric value specifying the minimum mass required for a grid point to be considered.
Default is \code{1}.}

\item{scale}{A numeric value specifying the scaling factor for the velocity vectors.
Default is \code{1}.}

\item{adjust_for_stream}{Whether to adjust the velocity vectors for streamlines.
Default is \code{FALSE}.}

\item{cutoff_perc}{A numeric value specifying the percentile cutoff for removing low-density grid points.
Default is \code{5}.}
}
\value{
A list with two components:
\code{x_grid}, the grid coordinates, and \code{v_grid}, the smoothed velocity vectors on the grid.
}
\description{
Compute velocity on grid
}
\examples{
x_emb <- matrix(
  c(
    0, 0,
    1, 0,
    0, 1,
    1, 1
  ),
  ncol = 2,
  byrow = TRUE
)
v_emb <- matrix(
  c(
    1, 0,
    1, 0,
    0, 1,
    0, 1
  ),
  ncol = 2,
  byrow = TRUE
)

velocity_grid <- compute_velocity_on_grid(
  x_emb = x_emb,
  v_emb = v_emb,
  density = 0.1,
  n_neighbors = 2,
  adjust_for_stream = TRUE
)

names(velocity_grid)
dim(velocity_grid$x_grid)
dim(velocity_grid$v_grid)
head(velocity_grid$x_grid)
head(velocity_grid$v_grid)

grid_df <- expand.grid(
  x = velocity_grid$x_grid[1, ],
  y = velocity_grid$x_grid[2, ]
)

plot_df <- data.frame(
  x = grid_df$x,
  y = grid_df$y,
  xend = grid_df$x + c(velocity_grid$v_grid[1, , ]) * 0.2,
  yend = grid_df$y + c(velocity_grid$v_grid[2, , ]) * 0.2
)

ggplot2::ggplot(plot_df) +
  ggplot2::geom_segment(
    ggplot2::aes(x = x, y = y, xend = xend, yend = yend),
    arrow = grid::arrow(length = grid::unit(0.12, "inches")),
    na.rm = TRUE
  ) +
  ggplot2::coord_equal()
}
\references{
\url{https://github.com/theislab/scvelo/blob/master/scvelo/plotting/velocity_embedding_grid.py}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{cluster_within_group2}
\alias{cluster_within_group2}
\title{Cluster within group}
\usage{
cluster_within_group2(mat, factor)
}
\arguments{
\item{mat}{A matrix of data}

\item{factor}{A factor}
}
\value{
A dendrogram with ordered leaves
}
\description{
Cluster within group
}
\examples{
mat <- matrix(rnorm(100), 10, 10)
factor <- factor(rep(1:2, each = 5))
dend <- cluster_within_group2(mat, factor)
dend
plot(dend)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{add_grob}
\alias{add_grob}
\title{Add a grob to a gtable}
\usage{
add_grob(
  gtable,
  grob,
  position = c("top", "bottom", "left", "right", "none"),
  space = NULL,
  clip = "on"
)
}
\arguments{
\item{gtable}{A gtable object.}

\item{grob}{A grob or gtable object to add.}

\item{position}{The position to add the grob.
One of \code{"top"}, \code{"bottom"}, \code{"left"}, \code{"right"}, or \code{"none"}.}

\item{space}{The space to allocate for the grob.
If \code{NULL}, will be calculated automatically.}

\item{clip}{The clipping mode.
Default is \code{"on"}.}
}
\value{
A gtable object with the grob added.
}
\description{
Add a grob to a gtable at a specified position (top, bottom, left, or right).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colors.R
\name{simple_colors}
\alias{simple_colors}
\title{Simple random color selection}
\usage{
simple_colors(n = 10, palette = NULL)
}
\arguments{
\item{n}{The number of colors to return. Default is \code{10}.}

\item{palette}{The name of the palette to use.
Default is \code{NULL}, colors will be selected from ChineseColors.
Otherwise, colors will be selected from the specified palette.
Available palette names can be queried with \link{show_palettes}.}
}
\value{
A character vector of hexadecimal color codes.
}
\description{
Simple random color selection
}
\examples{
simple_colors()

show_palettes(simple_colors(n = 5))

# Get colors from a specific palette
simple_colors(n = 10, palette = "Paired")
simple_colors(n = 10, palette = "ChineseBlue")
simple_colors(n = 10, palette = "Spectral")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/palettes.R
\name{get_chinese_palettes}
\alias{get_chinese_palettes}
\title{Get Chinese color palettes}
\usage{
get_chinese_palettes(prefix = "Chinese")
}
\arguments{
\item{prefix}{The prefix of the palette names.
Default is `"Chinese_"`.}
}
\value{
A list of Chinese color palettes.
}
\description{
Get Chinese color palettes
}
\examples{
show_palettes(get_chinese_palettes())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/thisplot-package.R
\name{thisplot_logo}
\alias{thisplot_logo}
\title{The logo of thisplot}
\usage{
thisplot_logo(unicode = cli::is_utf8_output())
}
\arguments{
\item{unicode}{Unicode symbols on UTF-8 platforms.
Default is \link[cli:is_utf8_output]{cli::is_utf8_output}.}
}
\value{
A character vector with class \code{thisplot_logo}.
}
\description{
The thisplot logo, using ASCII or Unicode characters
Use \link[cli:ansi_strip]{cli::ansi_strip} to get rid of the colors.
}
\examples{
thisplot_logo()
}
\references{
\url{https://github.com/tidyverse/tidyverse/blob/main/R/logo.R}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{annotation_graphics}
\alias{annotation_graphics}
\title{Build graphics callback list for anno_customize}
\usage{
annotation_graphics(subplots, prefix)
}
\arguments{
\item{subplots}{A named list of subplot objects.}

\item{prefix}{Prefix used to generate stable grob names.}
}
\value{
A named list of callback functions accepted by \link[ComplexHeatmap:anno_customize]{ComplexHeatmap::anno_customize}.
}
\description{
Build graphics callback list for anno_customize
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DimDataPlot.R
\name{DimDataPlot}
\alias{DimDataPlot}
\title{Data Dimensional Plot}
\usage{
DimDataPlot(
  data,
  x,
  y,
  group.by = NULL,
  split.by = NULL,
  shape.by = NULL,
  show_na = FALSE,
  palette = "Chinese",
  palcolor = NULL,
  NA_color = "grey80",
  pt.size = 2,
  pt.alpha = 1,
  shape.values = NULL,
  add_mark = FALSE,
  mark_type = c("hull", "ellipse", "rect", "circle"),
  mark_expand = grid::unit(3, "mm"),
  mark_alpha = 0.1,
  mark_linetype = 1,
  label = FALSE,
  label.size = 4,
  label.fg = "black",
  label.bg = "white",
  label.bg.r = 0.1,
  label_repel = FALSE,
  label_repulsion = 20,
  label_point_size = 1,
  label_point_color = "black",
  label_segment_color = "black",
  add_origin = FALSE,
  origin.color = "grey30",
  origin.linetype = "dashed",
  origin.linewidth = 0.4,
  aspect.ratio = 1,
  title = NULL,
  subtitle = NULL,
  xlab = NULL,
  ylab = NULL,
  legend.position = "right",
  legend.direction = "vertical",
  legend.title = NULL,
  theme_use = "theme_this",
  theme_args = list(),
  seed = 11
)
}
\arguments{
\item{data}{A data frame containing coordinates and optional grouping columns.}

\item{x}{Name of the column used as the x-axis coordinate.}

\item{y}{Name of the column used as the y-axis coordinate.}

\item{group.by}{Name of the column used to color observations.
Default is \code{NULL}.}

\item{split.by}{Name of the column used to split the plot into facets.
Default is \code{NULL}.}

\item{shape.by}{Name of the column used to map point shapes.
Default is \code{NULL}.}

\item{show_na}{Whether to keep missing values in \code{group.by}, \code{split.by},
or \code{shape.by} as an explicit \code{"NA"} level. Default is \code{FALSE}.}

\item{palette}{Color palette name. Available palettes can be found in
\link{show_palettes}. Default is \code{"Chinese"}.}

\item{palcolor}{Custom colors used to create a color palette.
Default is \code{NULL}.}

\item{NA_color}{Color used for missing values.}

\item{pt.size}{Point size.}

\item{pt.alpha}{Point transparency.}

\item{shape.values}{Point shapes used when \code{shape.by} is set.}

\item{add_mark}{Whether to add marks around groups. Default is \code{FALSE}.}

\item{mark_type}{Type of mark to add around groups.
One of \code{"hull"}, \code{"ellipse"}, \code{"rect"}, or \code{"circle"}.
Default is \code{"hull"}. \code{"ellipse"} uses \link[ggplot2:stat_ellipse]{ggplot2::stat_ellipse}
with \code{type = "norm"} and \code{level = 0.95}; the other mark types use ggforce marks.}

\item{mark_expand}{Expansion of the mark around each group.
This is used for \code{"hull"}, \code{"rect"}, and \code{"circle"} marks.
Default is \code{grid::unit(3, "mm")}.}

\item{mark_alpha}{Transparency of the mark.
Default is \code{0.1}.}

\item{mark_linetype}{Line type of the mark border.
Default is \code{1}.}

\item{label}{Whether to label group centers.}

\item{label.size}{Size of labels.}

\item{label.fg}{Foreground color of labels.}

\item{label.bg}{Background color of labels.}

\item{label.bg.r}{Background ratio of labels.}

\item{label_repel}{Whether labels repel from their group centers.}

\item{label_repulsion}{Repulsion force for labels.}

\item{label_point_size}{Size of center points for repelled labels.}

\item{label_point_color}{Color of center points for repelled labels.}

\item{label_segment_color}{Color of label segments.}

\item{add_origin}{Whether to add dashed x = 0 and y = 0 reference lines.}

\item{origin.color}{Color of origin reference lines.}

\item{origin.linetype}{Line type of origin reference lines.}

\item{origin.linewidth}{Line width of origin reference lines.}

\item{aspect.ratio}{Aspect ratio passed to \code{theme()}.}

\item{title}{Plot title.}

\item{subtitle}{Plot subtitle.}

\item{xlab}{Label for the x axis.}

\item{ylab}{Label for the y axis.}

\item{legend.position}{Position of the legend.}

\item{legend.direction}{Direction of the legend.}

\item{legend.title}{Title of the group legend.}

\item{theme_use}{Theme function applied to the plot.}

\item{theme_args}{A named list of arguments passed to \code{theme_use}.}

\item{seed}{Random seed.}
}
\value{
A ggplot object.
}
\description{
Visualize observations on precomputed two-dimensional coordinates.
}
\examples{
set.seed(1)
plot_data <- data.frame(
  x = c(rnorm(40, -1), rnorm(40, 1)),
  y = c(rnorm(40, 0), rnorm(40, 1)),
  group = rep(c("A", "B"), each = 40)
)
DimDataPlot(
  plot_data,
  x = "x",
  y = "y",
  group.by = "group"
)

DimDataPlot(
  plot_data,
  x = "x",
  y = "y",
  group.by = "group",
  add_origin = TRUE,
  mark_type = "ellipse"
)

DimDataPlot(
  plot_data,
  x = "x",
  y = "y",
  group.by = "group",
  mark_type = "circle",
  mark_alpha = 0.3,
  mark_expand = grid::unit(1, "mm"),
  mark_linetype = 2
)
}
